/************************************************************************
*                                                                       *
*   NuiGesture.h                                                        *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/

#pragma once



#ifndef NUIAPI
#error "You must include nuiapi.h rather than including NuiGesture.h directly"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define NUI_GESTURE_FLAG_SUPPRESS_NO_FRAME_DATA         0x00000001


//
// Gesture preprocess flags for input skeletons
//
#define NUI_GESTURE_PREPROCESS_NONE                     0x00000000
#define NUI_GESTURE_PREPROCESS_MIRROR_LEFT_RIGHT        0x00000001
#define NUI_GESTURE_PREPROCESS_MAKE_SEATED              0x00000002


#define NUI_GESTURE_DIGEST_DWORD_COUNT 8

//
// NuiGesture output for a single frame
//
typedef struct _NUI_GESTURE_FRAME
{
    LARGE_INTEGER   TimeStamp;
    DWORD           FrameNumber;
    DWORD           Signature;
    PVOID           pData;
    DWORD           DataSize;
} NUI_GESTURE_FRAME;


NUIAPI
HRESULT
NuiGestureInitialize(
    _In_z_   CONST CHAR* GestureDatabasePath,
    _In_     CONST DWORD GestureDatabaseDigest[NUI_GESTURE_DIGEST_DWORD_COUNT],
    _In_     DWORD       Flags,
    _In_opt_ HANDLE      NextFrameEvent);


NUIAPI
HRESULT
NuiGestureInitializeFromMemory(
    _In_bytecount_(GestureDatabaseSize) CONST VOID* pGestureDatabase,
    _In_                                DWORD       GestureDatabaseSize,
    _In_                                CONST DWORD GestureDatabaseDigest[NUI_GESTURE_DIGEST_DWORD_COUNT],
    _In_                                DWORD       Flags,
    _In_opt_                            HANDLE      NextFrameEvent);


NUIAPI
VOID
NuiGestureShutdown();


NUIAPI
HRESULT
NuiGestureEnableGesture(
    _In_ DWORD GestureID,
    _In_ BOOL  Enable);


NUIAPI
HRESULT
NuiGestureSetActiveSkeletons(
    _In_opt_ DWORD  TrackingIDs[NUI_SKELETON_MAX_TRACKED_COUNT]);


NUIAPI
HRESULT
NuiGestureSetPreprocessFlags(
    _In_ DWORD Flags);


NUIAPI
HRESULT
NuiGestureGetNextFrame(
    _In_    DWORD               MillisecondsToWait,
    _Inout_ NUI_GESTURE_FRAME*  pFrame);

#ifdef __cplusplus
} //close extern "C"
#endif
