/************************************************************************
*                                                                       *
*   nuihandles.h -- This module defines the Natal Handles API           *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/
#pragma once

#ifndef _NUI_HANDLES_H_
#define _NUI_HANDLES_H_


#include <sal.h>
#include <nuiapi.h>

//-----------------------------------------------------------------------------
// Error definitions
//
#define FACILITY_NUI_HANDLES                0x301
#define NUI_HANDLES_BASE_ERROR_CODE         100


#define NUI_HANDLES_S_OK                    S_OK                    // 0x00000000
#define NUI_HANDLES_E_INVALID_ARG           E_INVALIDARG            // 0x80070057
#define NUI_HANDLES_E_OUT_OF_MEMORY         E_OUTOFMEMORY           // 0x8007000E
#define NUI_HANDLES_E_UNEXPECTED_FAILURE    E_FAIL                  // 0x80004005

#define NUI_HANDLES_E_MAX_RAILS_EXCEEDED            MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI_HANDLES, NUI_HANDLES_BASE_ERROR_CODE)
#define NUI_HANDLES_E_MAX_LISTITEMS_EXCEEDED        MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI_HANDLES, NUI_HANDLES_BASE_ERROR_CODE + 1)
#define NUI_HANDLES_E_RAIL_ID_INVALID               MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI_HANDLES, NUI_HANDLES_BASE_ERROR_CODE + 2)
#define NUI_HANDLES_E_ILLEGAL_WHILE_ATTACHED        MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI_HANDLES, NUI_HANDLES_BASE_ERROR_CODE + 3)
#define NUI_HANDLES_E_WRONG_IMAGE_FRAME_RESOLUTION  MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI_HANDLES, NUI_HANDLES_BASE_ERROR_CODE + 4)

#define NUI_HANDLES_INVALID_RAIL_ID 0xFFFFFFFF

#define NUIHANDLES_STANDING_GLOBAL_PARAMETERS_PHIZ_WIDTH        0.9471f
#define NUIHANDLES_STANDING_GLOBAL_PARAMETERS_PHIZ_HEIGHT       0.7854f
#define NUIHANDLES_STANDING_GLOBAL_PARAMETERS_HORZ_ALIGNMENT    0.2618f
#define NUIHANDLES_STANDING_GLOBAL_PARAMETERS_VERT_OFFSET       0.3491f

#define NUIHANDLES_SEATED_GLOBAL_PARAMETERS_PHIZ_WIDTH        0.7330f
#define NUIHANDLES_SEATED_GLOBAL_PARAMETERS_PHIZ_HEIGHT       0.6108f
#define NUIHANDLES_SEATED_GLOBAL_PARAMETERS_HORZ_ALIGNMENT    0.2618f
#define NUIHANDLES_SEATED_GLOBAL_PARAMETERS_VERT_OFFSET       0.0f

#ifdef __cplusplus
extern "C" {
#endif


//  4324: structure padded due to __declspec(align())
#pragma warning(disable: 4324)


#define NUI_HANDLES_MAX_RAILS_PER_GRIP 4

#define NUI_HANDLES_GRIP_RESERVED_VECTOR_COUNT 41
#define NUI_HANDLES_ARMS_RESERVED_VECTOR_COUNT 96

typedef struct _NUI_HANDLES_GRIP {
    XMVECTOR Reserved[NUI_HANDLES_GRIP_RESERVED_VECTOR_COUNT];
} NUI_HANDLES_GRIP, *PNUI_HANDLES_GRIP;
typedef const NUI_HANDLES_GRIP *PCNUI_HANDLES_GRIP;

typedef struct _NUI_HANDLES_ARMS {
    XMVECTOR Reserved[NUI_HANDLES_ARMS_RESERVED_VECTOR_COUNT];
} NUI_HANDLES_ARMS, *PNUI_HANDLES_ARMS;
typedef const NUI_HANDLES_ARMS *PCNUI_HANDLES_ARMS;

typedef enum _NUI_HANDLES_ARMS_HANDEDNESS {
    NUI_HANDLES_ARMS_HANDEDNESS_DEFAULT         = 0,
    NUI_HANDLES_ARMS_HANDEDNESS_LEFT_ARM        = 0,
    NUI_HANDLES_ARMS_HANDEDNESS_RIGHT_ARM,
    NUI_HANDLES_ARMS_HANDEDNESS_COUNT
} NUI_HANDLES_ARMS_HANDEDNESS;

typedef enum _NUI_HANDLES_GRIP_HANDEDNESS {
    NUI_HANDLES_GRIP_HANDEDNESS_DEFAULT         = 0,
    NUI_HANDLES_GRIP_HANDEDNESS_AMBIDEXTROUS    = 0,
    NUI_HANDLES_GRIP_HANDEDNESS_LEFT_ARM,
    NUI_HANDLES_GRIP_HANDEDNESS_RIGHT_ARM,
    NUI_HANDLES_GRIP_HANDEDNESS_COUNT
} NUI_HANDLES_GRIP_HANDEDNESS;

typedef enum _NUI_HANDLES_ARM_STATUS
{
    NUI_HANDLES_ARM_STATUS_ENGAGED              = 0,
    NUI_HANDLES_ARM_STATUS_NOT_TRACKED,
    NUI_HANDLES_ARM_STATUS_INVALID_ARM_POS,
    NUI_HANDLES_ARM_STATUS_USER_NOT_FACING_DEVICE
}NUI_HANDLES_ARM_STATUS;

typedef enum _NUI_HANDLES_APPROACH_RANGE
{
    NUI_HANDLES_APPROACH_RANGE_FAST_ATTACH      = 0,
    NUI_HANDLES_APPROACH_RANGE_SLOW_ATTACH,
    NUI_HANDLES_APPROACH_RANGE_APPROACH,
    NUI_HANDLES_APPROACH_RANGE_SLOW_DETACH,
    NUI_HANDLES_APPROACH_RANGE_FAST_DETACH,
    NUI_HANDLES_APPROACH_RANGE_NOT_EXTENDED
}NUI_HANDLES_APPROACH_RANGE;

typedef struct _NUI_HANDLES_APPROACH_INFO
{
    PNUI_HANDLES_GRIP           pGrip;
    NUI_HANDLES_ARMS_HANDEDNESS eArmHandedness;
    XMVECTOR                    vFalloffVector;
    NUI_HANDLES_APPROACH_RANGE  eApproachRange;
    FLOAT                       fAttachProgress;
    BOOL                        bIsAttached;
}NUI_HANDLES_APPROACH_INFO, *PNUI_HANDLES_APPROACH_INFO;
typedef const NUI_HANDLES_APPROACH_INFO *PCNUI_HANDLES_APPROACH_INFO;

typedef struct _NUI_HANDLES_PLAYER_INFO
{
    DWORD dwTrackingID;
    DWORD dwUserIndex;    
}NUI_HANDLES_PLAYER_INFO, *PNUI_HANDLES_PLAYER_INFO;

typedef struct _NUI_HANDLES_BODY_SPACE
{
    XMVECTOR                    vOrigin;
    XMVECTOR                    vForward;
    XMVECTOR                    vRight;
    XMVECTOR                    vUp;
    XMVECTOR                    vRightShoulder;
    XMVECTOR                    vLeftShoulder;
    XMVECTOR                    vRefinedRightHand;
    XMVECTOR                    vRefinedLeftHand;
}NUI_HANDLES_BODY_SPACE, *PNUI_HANDLES_BODY_SPACE;
typedef const NUI_HANDLES_BODY_SPACE *PCNUI_HANDLES_BODY_SPACE;


// 
// Rail State
//

typedef struct _NUI_HANDLES_RAILSTATE_ARC
{
    FLOAT               fProgress;
    FLOAT               fDenormalizedProgress;
}NUI_HANDLES_RAILSTATE_ARC, *PNUI_HANDLES_RAILSTATE_ARC;

typedef struct _NUI_HANDLES_RAILSTATE_GEARSHIFT
{
    INT                 nIndex;
    FLOAT               fToothProgress;
    FLOAT               fSpineProgress;
    FLOAT               fSpineProgressAtToothCommit;
}NUI_HANDLES_RAILSTATE_GEARSHIFT, *PNUI_HANDLES_RAILSTATE_GEARSHIFT;

typedef struct _NUI_HANDLES_RAILSTATE_ANALOGSTICK 
{
    FLOAT               fAngleInRadians;
    FLOAT               fDistance;
}NUI_HANDLES_RAILSTATE_ANALOGSTICK, *PNUI_HANDLES_RAILSTATE_ANALOGSTICK;

typedef struct _NUI_HANDLES_RAILSTATE_HOVERTIMER
{
    DWORD               dbTimePassedInMilliseconds;
    FLOAT               fPercentageComplete;
}NUI_HANDLES_RAILSTATE_HOVERTIMER, *PNUI_HANDLES_RAILSTATE_HOVERTIMER;

typedef struct _NUI_HANDLES_RAILSTATE
{
    BOOL                bEnabled;
    BOOL                bCommitted;
    DWORD               dwRailID;
    
    union
    {
        NUI_HANDLES_RAILSTATE_ANALOGSTICK     AnalogStick;
        NUI_HANDLES_RAILSTATE_ARC             Arc;
        NUI_HANDLES_RAILSTATE_GEARSHIFT       Gearshift;
        NUI_HANDLES_RAILSTATE_HOVERTIMER      HoverTimer;
    };
}NUI_HANDLES_RAILSTATE, *PNUI_HANDLES_RAILSTATE;

//
// Grip Shape
//

typedef struct _NUI_HANDLES_GRIPSHAPE
{
    FLOAT fFastAttachRadius;
    FLOAT fSlowAttachRadius;
    FLOAT fApproachRadius;
    FLOAT fSlowDetachRadius;
}NUI_HANDLES_GRIPSHAPE, *PNUI_HANDLES_GRIPSHAPE;
typedef const NUI_HANDLES_GRIPSHAPE *PCNUI_HANDLES_GRIPSHAPE;

//
// Config Structs
//

typedef struct _NUI_HANDLES_TUNING_PARAMS_GLOBAL
{
    FLOAT fPhizWidthInRadians;     
    FLOAT fPhizHeightInRadians;       
    FLOAT fPhizHorizontalAlignmentInRadians; 
    FLOAT fPhizVerticalOffsetInRadians; 
    FLOAT fRequiredUserAlignmentInRadians;
}NUI_HANDLES_TUNING_PARAMS_GLOBAL, *PNUI_HANDLES_TUNING_PARAMS_GLOBAL;
typedef const NUI_HANDLES_TUNING_PARAMS_GLOBAL *PCNUI_HANDLES_TUNING_PARAMS_GLOBAL;

typedef struct _NUI_HANDLES_TUNING_PARAMS_GRIP
{
    DWORD dwSlowAttachTimeInMilliseconds;             
    DWORD dwFastAttachTimeInMilliseconds;             
    DWORD dwSlowDetachTimeInMilliseconds;    
    DWORD dwFastDetachTimeInMilliseconds;  
}NUI_HANDLES_TUNING_PARAMS_GRIP, *PNUI_HANDLES_TUNING_PARAMS_GRIP;
typedef const NUI_HANDLES_TUNING_PARAMS_GRIP *PCNUI_HANDLES_TUNING_PARAMS_GRIP;

typedef struct _NUI_HANDLES_TUNING_PARAMS_ARMS
{
    DWORD dwCooldownTimeInMilliseconds;                   
    DWORD dwReattachTimeInMilliseconds;                 
    FLOAT fPreAttachExtentDistance;        
    FLOAT fAttachExtentDistance;           
    FLOAT fDetachExtentDistance;           
}NUI_HANDLES_TUNING_PARAMS_ARMS, *PNUI_HANDLES_TUNING_PARAMS_ARMS;
typedef const NUI_HANDLES_TUNING_PARAMS_ARMS *PCNUI_HANDLES_TUNING_PARAMS_ARMS;


//
// Global
//

XBOXAPI
HRESULT
WINAPI            
NuiHandlesGlobalSetTuningParams(
                    __in                PCNUI_HANDLES_TUNING_PARAMS_GLOBAL              pGlobalParams
                    );
                    
XBOXAPI
HRESULT
WINAPI            
NuiHandlesGlobalGetTuningParams(
                    __out               PNUI_HANDLES_TUNING_PARAMS_GLOBAL               pGlobalParams
                    );
                    
XBOXAPI
HRESULT
WINAPI            
NuiHandlesGlobalGetDefaultTuningParams(
                    __out               PNUI_HANDLES_TUNING_PARAMS_GLOBAL               pGlobalParams
                    );
                    
XBOXAPI
HRESULT
WINAPI            
NuiHandlesGripGetDefaultTuningParams(
                    __out               PNUI_HANDLES_TUNING_PARAMS_GRIP                 pGripParams
                    );

XBOXAPI
HRESULT
WINAPI            
NuiHandlesArmsGetDefaultTuningParams(
                    __out               PNUI_HANDLES_TUNING_PARAMS_ARMS                 pArmsParams
                    );


//
// Arms
//

XBOXAPI
HRESULT
WINAPI            
NuiHandlesArmsInit(
                    __out               PNUI_HANDLES_ARMS                               pArms 
                    );
                    
XBOXAPI
HRESULT
WINAPI            
NuiHandlesArmsSetTuningParams(
                    __inout             PNUI_HANDLES_ARMS                               pArms, 
                    __in                PCNUI_HANDLES_TUNING_PARAMS_ARMS                pArmParams
                    );
                    
XBOXAPI
HRESULT
WINAPI            
NuiHandlesArmsGetTuningParams(
                    __in                PCNUI_HANDLES_ARMS                              pArms, 
                    __out               PNUI_HANDLES_TUNING_PARAMS_ARMS                 pArmParams
                    );

XBOXAPI
HRESULT
WINAPI                 
NuiHandlesArmsUpdate(
                    __inout             PNUI_HANDLES_ARMS       pArms, 
                    __in        const   INT                     iSkeletonDataIndex,
                    __in        const   NUI_SKELETON_FRAME*     pUnfilteredSkeletonFrame,
                    __in        const   NUI_IMAGE_FRAME*        pDepth320x240, 
                    __in_opt    const   D3DLOCKED_RECT*         pDepthRect320x240, 
                    __in        const   NUI_IMAGE_FRAME*        pDepth80x60, 
                    __in_opt    const   D3DLOCKED_RECT*         pDepthRect80x60
                    );

XBOXAPI
HRESULT
WINAPI                  
NuiHandlesArmsUpdateRailProgress(
                    __inout             PNUI_HANDLES_ARMS               pArms,
                    __in                DWORD                           dwMillisecondsPassedSinceLastSkeletonUpdate
                    );
                    
XBOXAPI
BOOL
WINAPI                
NuiHandlesArmsIsAttached(
                    __in                PCNUI_HANDLES_ARMS              pArms,
                    __in                NUI_HANDLES_ARMS_HANDEDNESS     eHandedness 
                    );
 
XBOXAPI
XMVECTOR
WINAPI              
NuiHandlesArmGetScreenSpaceLocation(
                    __in                PCNUI_HANDLES_ARMS              pArms,
                    __in                NUI_HANDLES_ARMS_HANDEDNESS     eHandedness
                    );
 
XBOXAPI
XMVECTOR
WINAPI              
NuiHandlesArmGetPositionRay(
                    __in                PCNUI_HANDLES_ARMS              pArms,
                    __in                NUI_HANDLES_ARMS_HANDEDNESS     eHandedness
                    );

XBOXAPI
HRESULT
WINAPI              
NuiHandlesArmSetPositionRay(
                    __inout             PNUI_HANDLES_ARMS               pArms,
                    __in                NUI_HANDLES_ARMS_HANDEDNESS     eHandedness,
                    __in                XMVECTOR                        vPositionRay
                    );

XBOXAPI
HRESULT
WINAPI              
NuiHandlesArmsGetPlayerInfo(
                    __in                PCNUI_HANDLES_ARMS              pArms,
                    __out               PNUI_HANDLES_PLAYER_INFO        pPlayerInfo
                    );
        
XBOXAPI
NUI_HANDLES_ARM_STATUS
WINAPI              
NuiHandlesArmsGetArmStatus(
                    __in                PCNUI_HANDLES_ARMS              pArms
                    ); 
                        
XBOXAPI
HRESULT
WINAPI              
NuiHandlesArmsGetApproachInfo(
                    __in                PCNUI_HANDLES_ARMS              pArms,
                    __in                NUI_HANDLES_ARMS_HANDEDNESS     eHandedness,
                    __out               PNUI_HANDLES_APPROACH_INFO      pApproachInfo             
                    );
                    
XBOXAPI
BOOL
WINAPI              
NuiHandlesArmGetTwoHandedMode(
                    __in                PCNUI_HANDLES_ARMS              pArms
                    );
                    
XBOXAPI
HRESULT
WINAPI              
NuiHandlesArmSetTwoHandedMode(
                    __in                PNUI_HANDLES_ARMS               pArms,
                    __in                BOOL                            bIsTwoHanded
                    );
                    
XBOXAPI
HRESULT
WINAPI              
NuiHandlesArmSetEnabled(
                    __inout             PNUI_HANDLES_ARMS               pArms,
                    __in                NUI_HANDLES_ARMS_HANDEDNESS     eHandedness,
                    __in                BOOL                            bEnabled
                    );
XBOXAPI
BOOL
WINAPI              
NuiHandlesArmGetEnabled(
                    __in                PCNUI_HANDLES_ARMS               pArms,
                    __in                NUI_HANDLES_ARMS_HANDEDNESS      eHandedness
                    );
 
XBOXAPI
HRESULT
WINAPI              
NuiHandlesArmGetSensorToPhizTransformationMatrix(
                    __in                PCNUI_HANDLES_ARMS               pArms,
                    __in                NUI_HANDLES_ARMS_HANDEDNESS      eHandedness,
                    __out               XMMATRIX*                        pMatrix
                    );                   
                     
                     
XBOXAPI
HRESULT
WINAPI              
NuiHandlesArmsGetBodySpace(
                    __in                PCNUI_HANDLES_ARMS               pArms,
                    __out               PNUI_HANDLES_BODY_SPACE          pBodySpace
                    );
                                           
//                           
// Grips
//

XBOXAPI
HRESULT
WINAPI                  
NuiHandlesGripInit(     
                    __out               PNUI_HANDLES_GRIP               pGrip 
                    );
                    
XBOXAPI
HRESULT
WINAPI            
NuiHandlesGripGetTuningParams(
                    __in                PCNUI_HANDLES_GRIP                  pGrip, 
                    __out               PNUI_HANDLES_TUNING_PARAMS_GRIP     pGripParams
                    );                    

XBOXAPI
HRESULT
WINAPI            
NuiHandlesGripSetTuningParams(
                    __inout             PNUI_HANDLES_GRIP                   pGrip, 
                    __in                PCNUI_HANDLES_TUNING_PARAMS_GRIP    pGripParams
                    );

XBOXAPI
HRESULT
WINAPI                 
NuiHandlesGripUpdate(   
                    __inout             PNUI_HANDLES_GRIP               pGrip, 
                    __inout             PNUI_HANDLES_ARMS               pArms 
                    );
                    
XBOXAPI
HRESULT
WINAPI
NuiHandlesGripAddRailArc(
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __in                DWORD                           dwRailID,
                    __in                XMVECTOR                        vDirection,
                    __in                FLOAT                           fLengthInRadians,
                    __in                FLOAT                           fDeadzoneLengthInRadians,
                    __in                FLOAT                           fToleranceConeAngleInRadians
                    );

XBOXAPI
HRESULT
WINAPI
NuiHandlesGripRailSetPersistentProgress(
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __in                DWORD                           dwRailID,
                    __in                FLOAT                           fProgress
                                        );

XBOXAPI
FLOAT
WINAPI
NuiHandlesGripRailGetPersistentProgress(
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __in                DWORD                           dwRailID
                                        );
                        

XBOXAPI
HRESULT
WINAPI                 
NuiHandlesGripAddRailGearShift(
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __in                DWORD                           dwRailID,
                    __in                XMVECTOR                        vDirection,
                    __in                FLOAT                           fSpineLengthInRadians,
                    __in                FLOAT                           fToothLowAngleInRadians,
                    __in                FLOAT                           fToothHighAngleInRadians,
                    __in                FLOAT                           fToothDeadzoneInRadians,
                    __in                FLOAT                           fToothToleranceConeAngleInRadians,
                    __in                UINT                            nNumTeeth,
                    __in                FLOAT                           fScrollZoneTopLengthInRadians,
                    __in                FLOAT                           fScrollZoneBottomLengthInRadians
                    );
                
XBOXAPI
HRESULT
WINAPI                 
NuiHandlesGripAddRailAnalogStick(
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __in                DWORD                           dwRailID,
                    __in                FLOAT                           fSizeInRadians,
                    __in                FLOAT                           fDeadzoneSizeInRadians 
                    );
                    
XBOXAPI
HRESULT
WINAPI                 
NuiHandlesGripAddRailHoverTimer(
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __in                DWORD                           dwRailID,
                    __in                DWORD                           dwTotalHoverTimeInMilliseconds,
                    __in                DWORD                           dwHoverTimeToCommitInMilliseconds 
                    );

XBOXAPI
HRESULT
WINAPI              
NuiHandlesGripGetNumRails(
                    __in                PCNUI_HANDLES_GRIP              pGrip,
                    __out               DWORD*                          pdwNumRails
                    );

XBOXAPI
HRESULT
WINAPI              
NuiHandlesGripGetCommittedRailId(
                    __in                PCNUI_HANDLES_GRIP              pGrip,
                    __out               DWORD*                          pdwRailId
                    );

XBOXAPI
HRESULT
WINAPI              
NuiHandlesGripGetRailState(
                    __in                PCNUI_HANDLES_GRIP              pGrip,
                    __in                DWORD                           dwRailId,
                    __out               PNUI_HANDLES_RAILSTATE          pRailState
                    );
                                                            
XBOXAPI
BOOL
WINAPI
NuiHandlesGripGetRailEnabled(
                    __in                PCNUI_HANDLES_GRIP              pGrip,
                    __in                DWORD                           dwRailID
                    );

XBOXAPI
HRESULT
WINAPI
NuiHandlesGripSetRailEnabled(
                    __in                PNUI_HANDLES_GRIP               pGrip,
                    __in                DWORD                           dwRailID,
                    __in                BOOL                            bEnabled
                    );
                    
XBOXAPI
HRESULT
WINAPI               
NuiHandlesGripGetShape( 
                    __in                PCNUI_HANDLES_GRIP              pGrip,
                    __out               PNUI_HANDLES_GRIPSHAPE         pShape 
                    );
                    
XBOXAPI
HRESULT
WINAPI               
NuiHandlesGripSetShape( 
                    __inout             PNUI_HANDLES_GRIP               pGrip, 
                    __in                PCNUI_HANDLES_GRIPSHAPE         pShape
                    );
                                                                     
XBOXAPI
XMVECTOR
WINAPI               
NuiHandlesGripGetPositionRay( 
                    __in                PCNUI_HANDLES_GRIP              pGrip 
                    );

XBOXAPI
HRESULT
WINAPI                  
NuiHandlesGripSetPositionRay( 
                    __inout             PNUI_HANDLES_GRIP               pGrip, 
                    __in                XMVECTOR                        vPositionRay
                    );
  
XBOXAPI
NUI_HANDLES_GRIP_HANDEDNESS
WINAPI 
NuiHandlesGripGetHandedness( 
                    __in                PCNUI_HANDLES_GRIP              pGrip  
                    );

XBOXAPI
HRESULT
WINAPI                  
NuiHandlesGripSetHandedness( 
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __in                NUI_HANDLES_GRIP_HANDEDNESS     eHandedness 
                    );
 
XBOXAPI
BOOL
WINAPI                     
NuiHandlesGripGetEnabled( 
                    __in                PCNUI_HANDLES_GRIP              pGrip 
                    );

XBOXAPI
HRESULT
WINAPI                  
NuiHandlesGripSetEnabled( 
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __in                BOOL                            bEnabled 
                    );
 
XBOXAPI
PVOID
WINAPI                 
NuiHandlesGripGetUserContext( 
                    __in                PCNUI_HANDLES_GRIP              pGrip 
                    );

XBOXAPI
VOID
WINAPI                     
NuiHandlesGripSetUserContext( 
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __inout             PVOID                           pContext
                    );

XBOXAPI
BOOL
WINAPI                     
NuiHandlesGripGetIsAttached(
                    __in                PCNUI_HANDLES_GRIP              pGrip
                    );

XBOXAPI
HRESULT
WINAPI                 
NuiHandlesGripForceDetach(
                    __inout             PNUI_HANDLES_GRIP               pGrip 
                    );

XBOXAPI
XMVECTOR
WINAPI                
NuiHandlesGripGetScreenSpaceLocation(
                    __in                PCNUI_HANDLES_GRIP              pGrip
                    );

XBOXAPI
HRESULT
WINAPI                  
NuiHandlesGripSetScreenSpaceLocation(
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __in                XMVECTOR                        vScreenSpaceLoc         
                    );
     
XBOXAPI
HRESULT
WINAPI              
NuiHandlesGripGetLastAttachPlayerInfo(
                    __in                PCNUI_HANDLES_GRIP              pGrip,
                    __out               PNUI_HANDLES_PLAYER_INFO        pPlayerInfo
                    );

XBOXAPI
HRESULT
WINAPI              
NuiHandlesGripGetApproachInfo(
                    __in                PCNUI_HANDLES_GRIP              pGrip,
                    __out               PNUI_HANDLES_APPROACH_INFO      pApproachInfo             
                    );
                    
XBOXAPI
HRESULT
WINAPI              
NuiHandlesGripUninit(
                    __inout             PNUI_HANDLES_GRIP               pGrip,
                    __inout             PNUI_HANDLES_ARMS               pArms
                    );


#ifdef __cplusplus
}
#endif

#endif  //_NUI_HANDLES include

