/************************************************************************
*                                                                       *
*   NUIHeadOrientation.h                                                *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/

#pragma once



#ifndef NUIAPI
#error "You must include nuiapi.h rather than including NuiHeadOrientation.h directly"
#endif



#ifdef __cplusplus
extern "C" {
#endif

#define NUI_HEAD_ORIENTATION_COUNT 2

#define NUI_HEAD_ORIENTATION_FLAG_SUPPRESS_NO_FRAME_DATA    0x00000001
#define NUI_HEAD_ORIENTATION_FLAG_TITLE_SETS_ACTIVE_HEADS   0x00000002

#define NUI_HEAD_ORIENTATION_QUALITY_FACE_DETECT_FAILURE  0x00000001
#define NUI_HEAD_ORIENTATION_QUALITY_HANDS_OCCLUDING_FACE 0x00000002
#define NUI_HEAD_ORIENTATION_QUALITY_CLOSE_TO_TOP         0x00000004
#define NUI_HEAD_ORIENTATION_QUALITY_CLOSE_TO_BOTTOM      0x00000008
#define NUI_HEAD_ORIENTATION_QUALITY_CLOSE_TO_RIGHT       0x00000010
#define NUI_HEAD_ORIENTATION_QUALITY_CLOSE_TO_LEFT        0x00000020
#define NUI_HEAD_ORIENTATION_QUALITY_TOO_DARK             0x00000040
#define NUI_HEAD_ORIENTATION_QUALITY_FAILURE              0x00000080

typedef struct _NUI_HEAD_ORIENTATION_DATA {
    DWORD    TrackingID;
    DWORD    QualityFlags;
    DWORD    Reserved0;
    DWORD    Reserved1;

    XMVECTOR    Orientation;
    RECT        FaceBounds;
} NUI_HEAD_ORIENTATION_DATA;

typedef struct _NUI_HEAD_ORIENTATION_FRAME {
    UINT64 TimeStamp;
    DWORD FrameNumber;
    DWORD Reserved;
    NUI_HEAD_ORIENTATION_DATA HeadOrientationData[NUI_HEAD_ORIENTATION_COUNT];
} NUI_HEAD_ORIENTATION_FRAME;

typedef struct _NUI_HEAD_ORIENTATION_SMOOTH_PARAMETERS
{
    FLOAT   fSmoothing;
    FLOAT   fCorrection;
    FLOAT   fPrediction;
    FLOAT   fActiveZone;
    FLOAT   fCenterDeadZone;
    FLOAT   fEdgeDeadZone;
} NUI_HEAD_ORIENTATION_SMOOTH_PARAMETERS;

NUIAPI
HRESULT 
NuiHeadOrientationEnable(
    _In_opt_ HANDLE hNextFrameEvent, 
    _In_     DWORD Flags);

NUIAPI
HRESULT 
NuiHeadOrientationDisable();

NUIAPI
HRESULT 
NuiHeadOrientationSetTrackedHeads(
    _In_ DWORD TrackingIDs[NUI_HEAD_ORIENTATION_COUNT]);

NUIAPI
HRESULT 
NuiHeadOrientationGetNextFrame(
    _In_  DWORD MillisecondsToWait, 
    _Out_ NUI_HEAD_ORIENTATION_FRAME* pHeadOrientationFrame);

NUIAPI
HRESULT 
NuiHeadOrientationSmooth(
    _Inout_ NUI_HEAD_ORIENTATION_FRAME* pHeadOrientationFrame, 
    _In_opt_ CONST NUI_HEAD_ORIENTATION_SMOOTH_PARAMETERS *pSmoothingParams );

#ifdef __cplusplus
} //close extern "C"
#endif

