/************************************************************************
*                                                                       *
*   NuiImageCamera.h -- This module defines the APIs for the Natural    *
*                       User Interface(NUI) image and camera services.  *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/
#pragma once

#ifndef NUIAPI
#error "You must include nuiapi.h rather than including nuiimagecamera.h directly"
#endif


#ifdef __cplusplus
extern "C" {
#endif

typedef enum _NUI_IMAGE_TYPE
{
  NUI_IMAGE_TYPE_DEPTH_AND_PLAYER_INDEX = 0,
  NUI_IMAGE_TYPE_COLOR,
  NUI_IMAGE_TYPE_COLOR_YUV,
  NUI_IMAGE_TYPE_DEPTH,
  NUI_IMAGE_TYPE_DEPTH_AND_PLAYER_INDEX_IN_COLOR_SPACE,
  NUI_IMAGE_TYPE_DEPTH_IN_COLOR_SPACE,
  NUI_IMAGE_TYPE_COLOR_IN_DEPTH_SPACE,
  NUI_IMAGE_TYPE_DEPTH_AND_PLAYER_INDEX_80x60,
  NUI_IMAGE_TYPE_DEPTH_80x60,
} NUI_IMAGE_TYPE;

typedef enum _NUI_IMAGE_RESOLUTION
{
  NUI_IMAGE_RESOLUTION_80x60,
  NUI_IMAGE_RESOLUTION_320x240,
  NUI_IMAGE_RESOLUTION_640x480
} NUI_IMAGE_RESOLUTION;

#define NUI_IMAGE_PLAYER_INDEX_SHIFT          3
#define NUI_IMAGE_PLAYER_INDEX_MASK           ((1 << NUI_IMAGE_PLAYER_INDEX_SHIFT)-1)
#define NUI_IMAGE_DEPTH_MAXIMUM               ((4000 << NUI_IMAGE_PLAYER_INDEX_SHIFT) | NUI_IMAGE_PLAYER_INDEX_MASK)
#define NUI_IMAGE_DEPTH_MINIMUM               (800 << NUI_IMAGE_PLAYER_INDEX_SHIFT)
#define NUI_IMAGE_DEPTH_NO_VALUE              0
#define NUI_IMAGE_DEPTH_BUFFER_SIZE           (384*240*2)
#define NUI_IMAGE_DEPTH_80x60_BUFFER_SIZE     (128*60*2)
#define NUI_IMAGE_COLOR_640x480_BUFFER_SIZE   (640*480*4)
#define NUI_IMAGE_COLOR_YUV_640x480_BUFFER_SIZE   (640*480*2)

#define NUI_CAMERA_DEPTH_NOMINAL_FOCAL_LENGTH_IN_PIXELS         (285.63f)   // Based on 320x240 pixel size.
#define NUI_CAMERA_DEPTH_NOMINAL_INVERSE_FOCAL_LENGTH_IN_PIXELS (3.501e-3f) // (1/NUI_CAMERA_DEPTH_NOMINAL_FOCAL_LENGTH_IN_PIXELS)
#define NUI_CAMERA_DEPTH_NOMINAL_DIAGONAL_FOV                   (70.0f)
#define NUI_CAMERA_DEPTH_NOMINAL_HORIZONTAL_FOV                 (58.5f)
#define NUI_CAMERA_DEPTH_NOMINAL_VERTICAL_FOV                   (45.6f)

#define NUI_CAMERA_COLOR_NOMINAL_FOCAL_LENGTH_IN_PIXELS         (531.15f)   // Based on 640x480 pixel size.
#define NUI_CAMERA_COLOR_NOMINAL_INVERSE_FOCAL_LENGTH_IN_PIXELS (1.83e-3f)  // (1/NUI_CAMERA_COLOR_NOMINAL_FOCAL_LENGTH_IN_PIXELS)
#define NUI_CAMERA_COLOR_NOMINAL_DIAGONAL_FOV                   ( 73.9f)
#define NUI_CAMERA_COLOR_NOMINAL_HORIZONTAL_FOV                 ( 62.0f)
#define NUI_CAMERA_COLOR_NOMINAL_VERTICAL_FOV                   ( 48.6f)

typedef enum _NUI_IMAGE_DIGITALZOOM
{
    NUI_IMAGE_DIGITAL_ZOOM_1X = 0,
    NUI_IMAGE_DIGITAL_ZOOM_2X = 1
} NUI_IMAGE_DIGITALZOOM;

typedef UINT32 NUI_TILT_FLAGS;

#define NUI_TILT_FLAGS_NONE                     (0)
#define NUI_TILT_FLAGS_FORCE_FAR_SPACE          (1<<3)
#define NUI_TILT_FLAGS_FORCE_NEAR_SPACE         (1<<4)
#define NUI_TILT_FLAGS_NO_TILT                  (1<<5)
#define NUI_TILT_FLAGS_OVERRIDE_TUNER           (1<<6)

#define NUI_TILT_HEAD_SPACE_MIN                 (-0.15)
#define NUI_TILT_HEAD_SPACE_MAX                 (0.5)

#define NUI_TILT_HEAD_SPACE_FULL_SKELETON       0.0
#define NUI_TILT_HEAD_SPACE_UPPER_BODY          0.15
#define NUI_TILT_HEAD_SPACE_HANDS_OVER_HEAD     0.3
                                    
#define NUI_TILT_DEFAULT_FAR_SPACE_DISTANCE     2.1336f
#define NUI_TILT_MAX_OBJECTS                    6

#define NUI_TILT_OBJECT_FLAG_INTERSECTS_TOP     (1<<0)
#define NUI_TILT_OBJECT_INFLAG_IGNORE           (1<<31)
#define NUI_TILT_OBJECT_INFLAG_PREFERRED        (1<<30)

typedef struct  _NUI_TILT_OBJECTS
{
    FLOAT CameraHeightMeters;
    DWORD Count;
    struct 
    {
        XMVECTOR Location;
        DWORD Flags;
    } Objects[NUI_TILT_MAX_OBJECTS];
} NUI_TILT_OBJECTS, *PNUI_TILT_OBJECTS;

NUIAPI
DWORD
NuiCameraAdjustTilt(
    _In_ NUI_TILT_FLAGS TiltFlags,
    _In_ FLOAT SpaceAboveHeadMeters,
    _In_ FLOAT FarSpaceDistanceMeters,
    _In_ FLOAT PreferredPlayspaceDistanceMeters,
    _Inout_opt_ NUI_TILT_OBJECTS* pTiltObjects,
    _Inout_opt_ XOVERLAPPED* pOverlapped
    );


typedef enum _NUI_CAMERA_TYPE
{
    NUI_CAMERA_TYPE_COLOR,
    NUI_CAMERA_TYPE_DEPTH
} NUI_CAMERA_TYPE;

typedef struct _NUI_IMAGE_VIEW_AREA
{
    NUI_IMAGE_DIGITALZOOM  eDigitalZoom;
    LONG                   lCenterX;
    LONG                   lCenterY;
} NUI_IMAGE_VIEW_AREA;

#define NUI_IMAGE_FRAME_FLAG_NONE              0x00000000
#define NUI_IMAGE_FRAME_FLAG_VIEW_AREA_UNKNOWN 0x00000001

typedef struct _NUI_IMAGE_FRAME
{
  LARGE_INTEGER             liTimeStamp;
  DWORD                     dwFrameNumber;
  NUI_IMAGE_TYPE            eImageType;
  NUI_IMAGE_RESOLUTION      eResolution;
  IDirect3DTexture9         *pFrameTexture;
  DWORD                     dwFrameFlags;  
  NUI_IMAGE_VIEW_AREA       ViewArea;
} NUI_IMAGE_FRAME;

#define NUI_IMAGE_STREAM_FLAG_SUPPRESS_NO_FRAME_DATA      0x00010000

#define NUI_IMAGE_STREAM_FRAME_LIMIT_MAXIMUM 48

#define NUI_FRAME_EVENT_FLAG_SUPPRESS_NO_FRAME_DATA   0x00000001


NUIAPI
HRESULT
NuiSetFrameEndEvent(
    _In_ HANDLE hEvent,
    _In_ DWORD dwFrameEventFlag
    );

NUIAPI
HRESULT
NuiImageStreamOpen(
    _In_ NUI_IMAGE_TYPE eImageType,
    _In_ NUI_IMAGE_RESOLUTION eResolution,
    _In_ DWORD dwImageFrameFlags,
    _In_ DWORD dwFrameLimit,
    _In_opt_ HANDLE hNextFrameEvent,
    _Out_ HANDLE *phStreamHandle
    );

NUIAPI
HRESULT
NuiImageStreamGetNextFrame(
    _In_  HANDLE hStream,
    _In_  DWORD dwMillisecondsToWait,
    _Deref_out_ CONST NUI_IMAGE_FRAME **ppcImageFrame
    );

NUIAPI
HRESULT
NuiImageStreamReleaseFrame(
    _In_ HANDLE hStream,
    CONST NUI_IMAGE_FRAME *pImageFrame
    );

HRESULT
NuiCameraGetColorImageViewArea(
    _Out_ NUI_IMAGE_VIEW_AREA *pViewArea
    );
    
HRESULT
NuiCameraSetColorImageViewArea(
    _In_ CONST NUI_IMAGE_VIEW_AREA *pcViewArea
    );
    
NUIAPI    
HRESULT
NuiImageGetColorPixelCoordinatesFromDepthPixel(
    _In_ NUI_IMAGE_RESOLUTION eColorResolution,
    _In_opt_ CONST NUI_IMAGE_VIEW_AREA *pcViewArea,
    _In_ LONG   lDepthX,
    _In_ LONG   lDepthY,
    _In_ USHORT usDepthValue,
    _Out_ LONG *plColorX,
    _Out_ LONG *plColorY
    );    

typedef enum _NUI_CAMERA_PROPERTY_AE_FACEBASED_STATUS_VALUES
{
    NUI_CAMERA_PROPERTY_AE_FACEBASED_STATUS_NOTSTARTED = 0,
    NUI_CAMERA_PROPERTY_AE_FACEBASED_STATUS_INPROGRESS,
    NUI_CAMERA_PROPERTY_AE_FACEBASED_STATUS_READY,
    NUI_CAMERA_PROPERTY_AE_FACEBASED_STATUS_INSUFFICIENT_LIGHT
} NUI_CAMERA_PROPERTY_AE_FACEBASED_STATUS_VALUES;

typedef enum _NUI_CAMERA_PROPERTY
{
    NUI_CAMERA_PROPERTY_AE_AWB_MODE              = 0,
    NUI_CAMERA_PROPERTY_FRAME_RATE_MAX           = 1,
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN        = 2,
    NUI_CAMERA_PROPERTY_QUALITY_PRESET           = 4,
    NUI_CAMERA_PROPERTY_AE_FACEBASED_TRACKING_ID = 5, 
    NUI_CAMERA_PROPERTY_AE_FACEBASED_STATUS      = 6,

    NUI_CAMERA_PROPERTY_MAXIMUM                  = NUI_CAMERA_PROPERTY_AE_FACEBASED_STATUS,

    //Deprecated names
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE = NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN
} NUI_CAMERA_PROPERTY;

#pragma deprecated(NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE)

typedef enum _NUI_CAMERA_PROPERTY_AE_AWB_MODE_VALUES
{
    NUI_CAMERA_PROPERTY_AE_AWB_MODE_STANDARD = 1,
    NUI_CAMERA_PROPERTY_AE_AWB_MODE_FACEBASED = 2,
    NUI_CAMERA_PROPERTY_AE_AWB_MODE_OFF      = 0,
    NUI_CAMERA_PROPERTY_AE_AWB_MODE_DEFAULT  = NUI_CAMERA_PROPERTY_AE_AWB_MODE_STANDARD
} NUI_CAMERA_PROPERTY_AE_AWB_MODE_VALUES;

typedef enum _NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_VALUES
{
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_30FPS        = 0,
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_15FPS        = 1,
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_10FPS        = 2,
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_LOWEST       = 3,
    
    // Deprecated names
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_FIXED_30FPS  = NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_30FPS,
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_NIGHT        = NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_LOWEST,
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_MIN_15FPS    = NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_15FPS,
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_MIN_10FPS    = NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_10FPS,
    NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_DEFAULT      = NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_30FPS,
    #pragma deprecated(NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_FIXED_30FPS)
    #pragma deprecated(NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_15FPS)
    #pragma deprecated(NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_10FPS)
    #pragma deprecated(NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_NIGHT)
    #pragma deprecated(NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_DEFAULT)
} NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MIN_VALUES,
  NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_VALUES;
#pragma deprecated(NUI_CAMERA_PROPERTY_AE_FRAME_RATE_MODE_VALUES)

typedef enum _NUI_CAMERA_PROPERTY_FRAME_RATE_MAX_VALUES
{
    NUI_CAMERA_PROPERTY_FRAME_RATE_MAX_30FPS = 0,
    NUI_CAMERA_PROPERTY_FRAME_RATE_MAX_15FPS = 1,
    NUI_CAMERA_PROPERTY_FRAME_RATE_MAX_10FPS = 2
} NUI_CAMERA_PROPERTY_FRAME_RATE_MAX_VALUES;


typedef enum _NUI_CAMERA_PROPERTY_QUALITY_PRESET_VALUES
{
    NUI_CAMERA_PROPERTY_QUALITY_PRESET_DYNAMIC                         = 0, // (default) Dynamically adjusts noise reduction based on gain.
    NUI_CAMERA_PROPERTY_QUALITY_PRESET_MOST_AGGRESSIVE_NOISE_REDUCTION = 1, 
    NUI_CAMERA_PROPERTY_QUALITY_PRESET_AGGRESSIVE_NOISE_REDUCTION      = 2,
    NUI_CAMERA_PROPERTY_QUALITY_PRESET_MODERATE_NOISE_REDUCTION        = 3,
    NUI_CAMERA_PROPERTY_QUALITY_PRESET_CONSERVATIVE_NOISE_REDUCTION    = 4,
    NUI_CAMERA_PROPERTY_QUALITY_PRESET_NO_NOISE_REDUCTION              = 5,
} NUI_CAMERA_PROPERTY_QUALITY_PRESET_VALUES;


typedef struct _NUI_CAMERA_AE_ROI
{
    FLOAT   Left;
    FLOAT   Top;
    FLOAT   Width;
    FLOAT   Height;
} NUI_CAMERA_AE_ROI;

#define NUI_CAMERA_AE_ROI_MINIMUM_WIDTH                     0.15f
#define NUI_CAMERA_AE_ROI_MINIMUM_HEIGHT                    0.15f

typedef enum _NUI_CAMERA_PROPERTYF
{
    NUI_CAMERA_PROPERTYF_EXPOSURE_TIME            = 0,
    NUI_CAMERA_PROPERTYF_AE_EXPOSURE_COMPENSATION = 5,
    NUI_CAMERA_PROPERTYF_COLOR_GAIN               = 7,
    NUI_CAMERA_PROPERTYF_AE_FACEBASED_MAX_GAIN    = 9,    
    NUI_CAMERA_PROPERTYF_MAXIMUM                  = NUI_CAMERA_PROPERTYF_AE_FACEBASED_MAX_GAIN
} NUI_CAMERA_PROPERTYF;

#define NUI_CAMERA_PROPERTYF_EXPOSURE_TIME_MINIMUM           0.0f
#define NUI_CAMERA_PROPERTYF_EXPOSURE_TIME_MAXIMUM        2000.0f
#define NUI_CAMERA_PROPERTYF_EXPOSURE_TIME_DEFAULT          33.0f

#define NUI_CAMERA_PROPERTYF_GAIN_MINIMUM                   (-6.0f)
#define NUI_CAMERA_PROPERTYF_GAIN_MAXIMUM                   6.0f
#define NUI_CAMERA_PROPERTYF_GAIN_DEFAULT                   0.0f

#define NUI_CAMERA_PROPERTYF_AE_COMPENSATION_MINIMUM        (-1.75f)
#define NUI_CAMERA_PROPERTYF_AE_COMPENSATION_MAXIMUM        1.75f
#define NUI_CAMERA_PROPERTYF_AE_COMPENSATION_DEFAULT        0.0f

NUIAPI
HRESULT
NuiCameraGetProperty(
    _In_  NUI_CAMERA_TYPE     CameraType,
    _In_  NUI_CAMERA_PROPERTY eCameraProperty,
    _Out_ LONG                *lValue
    );

NUIAPI
HRESULT
NuiCameraSetProperty(
    _In_ NUI_CAMERA_TYPE     CameraType,
    _In_ NUI_CAMERA_PROPERTY eCameraProperty,
    _In_ LONG                lValue
    );

NUIAPI
HRESULT
NuiCameraGetPropertyF(
    _In_  NUI_CAMERA_TYPE      CameraType,
    _In_  NUI_CAMERA_PROPERTYF eCameraProperty,
    _Out_ FLOAT               *pfValue
    );

NUIAPI
HRESULT
NuiCameraSetPropertyF(
    _In_ NUI_CAMERA_TYPE      CameraType,
    _In_ NUI_CAMERA_PROPERTYF eCameraProperty,
    _In_ FLOAT                fValue
    );

NUIAPI
HRESULT
NuiCameraSetExposureRegionOfInterest(
    _In_ NUI_CAMERA_TYPE CameraType,
    _In_ CONST NUI_CAMERA_AE_ROI* pRegionOfInterest
    );

NUIAPI
HRESULT
NuiCameraGetExposureRegionOfInterest(
    _In_ NUI_CAMERA_TYPE CameraType,
    _Out_ NUI_CAMERA_AE_ROI* pRegionOfInterest
    );

NUIAPI
HRESULT
NuiCameraSynchronizePropertyChangesBegin(
    );

NUIAPI
HRESULT
NuiCameraSynchronizePropertyChangesEnd(
    );

NUIAPI
HRESULT
NuiCameraGetNormalToGravity(
    _Out_ XMVECTOR* pxmvNormal
    );

#define NUI_CAMERA_ELEVATION_MAXIMUM  27
#define NUI_CAMERA_ELEVATION_MINIMUM (-27)

#define NUI_CAMERA_ELEVATION_MOTOR_MOVEMENT    (1<<0)
#define NUI_CAMERA_ELEVATION_DETECTED_MOVEMENT (1<<1)

NUIAPI
HRESULT
NuiCameraElevationGetAngle(
    _Out_ LONG* plAngleDegrees,
    _Out_ DWORD* pMovingFlags
    );

NUIAPI
HRESULT
NuiCameraElevationSetAngle(
    _In_ LONG lAngleDegrees
    );

NUIAPI
HRESULT
NuiCameraElevationStopMovement(
    );

    
#ifdef __cplusplus
} //close extern "C"
#endif

