/************************************************************************
*                                                                       *
*   NuiIdentity.h -- This module defines the APIs for the Natural       *
*                    User Interface(NUI) identity services.             *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/
#pragma once

#ifndef NUIAPI
#error "You must include nuiapi.h rather than including nuiidentity.h directly"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define NUI_IDENTITY_MAX_ENROLLMENT_COUNT     8


#define NUI_IDENTITY_ENROLLMENT_INDEX_CALL_IDENTIFY 0xFFFFFFFF
#define NUI_IDENTITY_ENROLLMENT_INDEX_UNKNOWN       0xFFFFFFFE
#define NUI_IDENTITY_ENROLLMENT_INDEX_BUSY          0xFFFFFFFC
#define NUI_IDENTITY_ENROLLMENT_INDEX_FAILURE       0xFFFFFFFB

#define NUI_IDENTITY_ENROLL_SKELETON          0x00000001
#define NUI_IDENTITY_FORCE_ENROLL             0x00000020
#define NUI_IDENTITY_ENROLLED_QUALITY_ISSUE   0x40000000

#define NUI_IDENTITY_QUALITY_ENVIRONMENT_FACE_DETECT_FAILURE 0x00000001
#define NUI_IDENTITY_QUALITY_USER_BODY_TURNED                0x00000002
#define NUI_IDENTITY_QUALITY_USER_NOT_UPRIGHT                0x00000004
#define NUI_IDENTITY_QUALITY_USER_OCCLUDED_FACE              0x00000008
#define NUI_IDENTITY_QUALITY_USER_OCCLUDED_BODY              0x00000010
#define NUI_IDENTITY_QUALITY_USER_FAR_AWAY                   0x00000020
#define NUI_IDENTITY_QUALITY_USER_CLOSE                      0x00000040
#define NUI_IDENTITY_QUALITY_USER_CLIPPED_AT_LEFT            0x00000080
#define NUI_IDENTITY_QUALITY_USER_CLIPPED_AT_RIGHT           0x00000100
#define NUI_IDENTITY_QUALITY_USER_CLIPPED_AT_TOP             0x00000200
#define NUI_IDENTITY_QUALITY_USER_CLIPPED_AT_BOTTOM          0x00000400
#define NUI_IDENTITY_QUALITY_ENVIRONMENT_TOO_DARK            0x00000800
#define NUI_IDENTITY_QUALITY_ENVIRONMENT_TOO_BRIGHT          0x00001000

#define NUI_IDENTITY_QUALITY_ENVIRONMENT_MASK  \
        (NUI_IDENTITY_QUALITY_ENVIRONMENT_FACE_DETECT_FAILURE   | \
         NUI_IDENTITY_QUALITY_ENVIRONMENT_TOO_DARK              | \
         NUI_IDENTITY_QUALITY_ENVIRONMENT_TOO_BRIGHT)

#define NUI_IDENTITY_QUALITY_USER_MASK  \
        (NUI_IDENTITY_QUALITY_USER_BODY_TURNED                  | \
         NUI_IDENTITY_QUALITY_USER_NOT_UPRIGHT                  | \
         NUI_IDENTITY_QUALITY_USER_OCCLUDED_FACE                | \
         NUI_IDENTITY_QUALITY_USER_OCCLUDED_BODY                | \
         NUI_IDENTITY_QUALITY_USER_FAR_AWAY                     | \
         NUI_IDENTITY_QUALITY_USER_CLOSE                        | \
         NUI_IDENTITY_QUALITY_USER_CLIPPED_AT_LEFT              | \
         NUI_IDENTITY_QUALITY_USER_CLIPPED_AT_RIGHT             | \
         NUI_IDENTITY_QUALITY_USER_CLIPPED_AT_TOP               | \
         NUI_IDENTITY_QUALITY_USER_CLIPPED_AT_BOTTOM)




typedef enum _NUI_IDENTITY_MESSAGE_ID
{
    NUI_IDENTITY_MESSAGE_ID_FRAME_PROCESSED,
    NUI_IDENTITY_MESSAGE_ID_COMPLETE,
}NUI_IDENTITY_MESSAGE_ID;

typedef enum _NUI_IDENTITY_OPERATION_ID
{
    NUI_IDENTITY_OPERATION_ID_NONE,
    NUI_IDENTITY_OPERATION_ID_IDENTIFY,
    NUI_IDENTITY_OPERATION_ID_ENROLL,
    NUI_IDENTITY_OPERATION_ID_TUNER,
}NUI_IDENTITY_OPERATION_ID;

typedef struct _NUI_IDENTITY_MESSAGE_FRAME_PROCESSED
{
    DWORD dwQualityFlags;
} NUI_IDENTITY_MESSAGE_FRAME_PROCESSED;

typedef struct _NUI_IDENTITY_MESSAGE_COMPLETE
{
    HRESULT   hrResult;
    DWORD     dwEnrollmentIndex;
    BOOL      bProfileMatched;
} NUI_IDENTITY_MESSAGE_COMPLETE;

typedef struct _NUI_IDENTITY_MESSAGE
{
    NUI_IDENTITY_MESSAGE_ID   MessageId;
    NUI_IDENTITY_OPERATION_ID OperationId;
    DWORD dwTrackingID;
    DWORD dwSkeletonFrameNumber;
    union
    {
        NUI_IDENTITY_MESSAGE_FRAME_PROCESSED   FrameProcessed;
        NUI_IDENTITY_MESSAGE_COMPLETE          Complete;
    } Data;
} NUI_IDENTITY_MESSAGE;

typedef BOOL 
(*PFN_NUI_IDENTITY_CALLBACK)(
    _In_ PVOID pvContext,
    _In_ NUI_IDENTITY_MESSAGE *pMessage
    );


HRESULT
NuiIdentityIdentify(
    _In_      DWORD dwTrackingID,
    _In_      DWORD dwFlags,
    _In_opt_  PFN_NUI_IDENTITY_CALLBACK pfnIdentityCallback,
    _In_opt_  VOID *pvContext
    );

HRESULT
NuiIdentityEnroll(
    _In_      DWORD dwTrackingID,
    _In_      DWORD dwForceEnrollmentIndex,
    _In_      DWORD dwFlags,
    _In_opt_  PFN_NUI_IDENTITY_CALLBACK pfnIdentityCallback,
    _In_opt_  VOID *pvContext
    );

HRESULT 
NuiIdentityUnenroll(    
    _In_ DWORD dwEnrollmentIndex
    );


HRESULT
NuiIdentityAbort();


HRESULT
NuiIdentityPause();

HRESULT
NuiIdentityResume();

typedef struct _NUI_ENROLLMENT_INFORMATION
{
    DWORD dwUserIndex;
    DWORD dwEnrollmentFlags;
} NUI_ENROLLMENT_INFORMATION;

HRESULT
NuiIdentityGetEnrollmentInformation(
    _In_  DWORD dwEnrollmentIndex,
    _Out_ NUI_ENROLLMENT_INFORMATION *pEnrollmentInformation
    );

HRESULT
NuiIdentityGetNextMessage(_Out_ NUI_IDENTITY_MESSAGE *pMessage);

HRESULT
NuiIdentityDetectedIntentToPlay(DWORD dwTrackingID);
#ifdef __cplusplus
}       //end of extern "C"
#endif
