/************************************************************************
*                                                                       *
*   NuiSkeleton.h -- This module aggregates all the Natural User        *
*                    Interface(NUI) API headers.                        *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/

#pragma once

#ifndef NUIAPI
#define NUIAPI

#include <pshpack8.h>

#ifdef __cplusplus
extern "C" {
#endif

//
// NUI Common Initialization Declarations
//

#define NUI_INITIALIZE_FLAG_USES_DEPTH_AND_PLAYER_INDEX 0x00000001
#define NUI_INITIALIZE_FLAG_USES_COLOR                  0x00000002
#define NUI_INITIALIZE_FLAG_USES_SKELETON               0x00000008  
#define NUI_INITIALIZE_FLAG_USES_DEPTH                  0x00000020
#define NUI_INITIALIZE_FLAG_USES_HIGH_QUALITY_COLOR     0x00000040

#define NUI_INITIALIZE_FLAG_AUTOMATIC_IDENTITY          0x00000400
#define NUI_INITIALIZE_FLAG_EXTRAPOLATE_FLOOR_PLANE     0x00000800


#define NUI_INITIALIZE_FLAG_USES_HEAD_POSITION          0x00002000
#define NUI_INITIALIZE_FLAG_USES_FITNESS                0x00004000
#define NUI_INITIALIZE_FLAG_USES_HEAD_ORIENTATION       0x00008000

#define NUI_INITIALIZE_FLAG_SUPPRESS_AUTOMATIC_UI       0x40000000
#define NUI_INITIALIZE_FLAG_NUI_GUIDE_DISABLED          0x80000000

// Passed to XMemAlloc in the dwObjectType field, when dwAllocatorId is eXALLOCAllocatorId_NuiApi.
// These give a recommendation of where in memory the object should go, in order to 
// obtain a moderate performance improvement.

#define NUI_XALLOC_OBJECTTYPE_DEFAULT               0
#define NUI_XALLOC_OBJECTTYPE_PREFER_UPPER_256MB    1
#define NUI_XALLOC_OBJECTTYPE_PREFER_LOWER_256MB    2


#define NUI_INITIALIZE_DEFAULT_HARDWARE_THREAD       0xFFFFFFFF


NUIAPI
HRESULT 
NuiInitialize(
    _In_ DWORD dwFlags,
    _In_ DWORD dwHardwareThreadSkeleton
    );


NUIAPI
VOID
NuiShutdown(
    );


//
// Define NUI error codes derived from win32 errors
//

#define E_NUI_DEVICE_NOT_CONNECTED  __HRESULT_FROM_WIN32(ERROR_DEVICE_NOT_CONNECTED)
#define E_NUI_DEVICE_NOT_READY      __HRESULT_FROM_WIN32(ERROR_NOT_READY)
#define E_NUI_ALREADY_INITIALIZED   __HRESULT_FROM_WIN32(ERROR_ALREADY_INITIALIZED)
#define E_NUI_IDENTITY_BUSY         __HRESULT_FROM_WIN32(ERROR_BUSY)
#define E_NUI_NO_MORE_ITEMS         __HRESULT_FROM_WIN32(ERROR_NO_MORE_ITEMS)
#define E_NUI_NOACCESS              __HRESULT_FROM_WIN32(ERROR_NOACCESS)
#define E_NUI_NOT_FOUND             __HRESULT_FROM_WIN32(ERROR_NOT_FOUND)

//
// Define NUI specific error codes
//

#define FACILITY_NUI 0x301
#define E_NUI_FRAME_NO_DATA                     MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 1)
#define E_NUI_STREAM_NOT_ENABLED                MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 2)
#define E_NUI_IMAGE_STREAM_IN_USE               MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 3)
#define E_NUI_FRAME_LIMIT_EXCEEDED              MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 4)
#define E_NUI_FEATURE_NOT_INITIALIZED           MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 5)
#define E_NUI_IDENTITY_ENROLLMENT_LIMIT         MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 6)
#define E_NUI_IDENTITY_UI_REQUIRED              MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 7)
#define E_NUI_IDENTITY_LOST_TRACK               MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 8)
#define E_NUI_IDENTITY_QUALITY_ISSUE            MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 9)
#define E_NUI_IDENTITY_NO_UNENROLL_SIGN_IN      MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 10)
#define E_NUI_SYSTEM_UI_PRESENT                 MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 11)
#define E_NUI_MUST_CALL_IDENTIFY                MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 12)
#define E_NUI_DATABASE_NOT_FOUND                MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 13)
#define E_NUI_DATABASE_VERSION_MISMATCH         MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 14)
#define E_NUI_DATABASE_CORRUPTED                MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 15)
#define E_NUI_TRACKED_PLAYER_LIMIT_EXCEEDED     MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 16)
#define E_NUI_TRACKING_ID_IN_USE                MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 17)
#define E_NUI_LOST_TRACKING                     MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 18)
#define E_NUI_GESTURE_NOT_INITIALIZED           MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 19)
#define E_NUI_GESTURE_ALREADY_INITIALIZED       MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 20)
#define E_NUI_NOT_LIVE_USER                     MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 21)
#define E_NUI_LIVE_USER_NOT_LOGGED_IN           MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NUI, 22)

// NOTE: Error codes in the range 100-199 associated with FACILITY_NUI are
// assigned to the NUI Handles API subsystem of NUI. See NuiHandles.h for
// details on those error codes.


#ifdef __cplusplus
} // end extern "C"
#endif

#include <NuiIdentity.h>
#include <NuiImageCamera.h>
#include <NuiSkeleton.h>
#include <NuiSpeech.h>
#include <NuiDiagnostics.h>
#include <NuiWave.h>
#include <NuiHeadPosition.h>
#include <NuiFitness.h>
#include <NuiHeadOrientation.h>
#include <NuiGesture.h>

#include <poppack.h>


#endif //NUIAPI

