
/**************************************************************************
 *
 * Copyright (c) Microsoft Corporation. All rights reserved.
 *
 * Defines the interface to the Quick Network library
 *
 **************************************************************************/

#ifndef __QNET_H__
#define __QNET_H__


#ifndef __cplusplus
#error "QNet requires C++!"
#endif // end if not defined __cplusplus

#include <sal.h>
#include <xaudio2.h>
#ifdef _XBOX
#include <xcam.h>
#endif


#define QNET_S_OK                                   S_OK                                      // 0x00000000
#define QNET_S_ALREADY_SET                          S_FALSE                                   // 0x00000001
#define QNET_E_ABORTED                              _HRESULT_TYPEDEF_(0x800704D3L)            // 0x800704D3 HRESULT_FROM_WIN32(ERROR_REQUEST_ABORTED)
#define QNET_E_ALREADY_EXISTS                       _HRESULT_TYPEDEF_(0x800700B7L)            // 0x800700B7 HRESULT_FROM_WIN32(ERROR_ALREADY_EXISTS)
#define QNET_E_BUFFER_TOO_SMALL                     _HRESULT_TYPEDEF_(0x8007007AL)            // 0x8007007A HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER)
#define QNET_E_CONNECTION_FAILED                    _HRESULT_TYPEDEF_(0x800704D0L)            // 0x800704D0 HRESULT_FROM_WIN32(ERROR_HOST_UNREACHABLE)
#define QNET_E_DENIED                               _HRESULT_TYPEDEF_(0x800710E0L)            // 0x800710E0 HRESULT_FROM_WIN32(ERROR_REQUEST_REFUSED)
#define QNET_E_DISJOINT_CONNECTIVITY                _HRESULT_TYPEDEF_(0x800713BBL)            // 0x800713BB HRESULT_FROM_WIN32(ERROR_CLUSTER_NODE_UNREACHABLE)
#define QNET_E_DOES_NOT_EXIST                       _HRESULT_TYPEDEF_(0x80070490L)            // 0x80070490 HRESULT_FROM_WIN32(ERROR_NOT_FOUND)
#define QNET_E_INCOMPATIBLE_SETTINGS                _HRESULT_TYPEDEF_(0x8007064AL)            // 0x8007064A HRESULT_FROM_WIN32(ERROR_BAD_CONFIGURATION)
#define QNET_E_INSUFFICIENT_PRIVILEGES              XONLINE_E_SESSION_INSUFFICIENT_PRIVILEGES // 0x80155201
#define QNET_E_INVALID_ARG                          E_INVALIDARG                              // 0x80070057
#define QNET_E_INVALID_SESSION_TYPE                 _HRESULT_TYPEDEF_(0x80070075L)            // 0x80070075 HRESULT_FROM_WIN32(ERROR_INVALID_CATEGORY)
#define QNET_E_INVALID_STATE                        _HRESULT_TYPEDEF_(0x80070649L)            // 0x80070649 HRESULT_FROM_WIN32(ERROR_INVALID_HANDLE_STATE)
#define QNET_E_NOT_HOST                             _HRESULT_TYPEDEF_(0x80070522L)            // 0x80070522 HRESULT_FROM_WIN32(ERROR_PRIVILEGE_NOT_HELD)
#define QNET_E_NOT_IMPLEMENTED                      E_NOTIMPL                                 // 0x80004001
#define QNET_E_NOT_SIGNED_IN                        _HRESULT_TYPEDEF_(0x800704DDL)            // 0x800704DD HRESULT_FROM_WIN32(ERROR_NOT_LOGGED_ON)
#define QNET_E_NOT_SIGNED_IN_TO_LIVE                XONLINE_E_SESSION_NOT_LOGGED_ON           // 0x80155209
#define QNET_E_OUT_OF_MEMORY                        E_OUTOFMEMORY                             // 0x8007000E
#define QNET_E_PARTIAL_CONNECTIVITY                 _HRESULT_TYPEDEF_(0x8007138DL)            // 0x8007138D HRESULT_FROM_WIN32(ERROR_HOST_NODE_NOT_AVAILABLE)
#define QNET_E_PENDING                              _HRESULT_TYPEDEF_(0x800703E5L)            // 0x800703E5 HRESULT_FROM_WIN32(ERROR_IO_PENDING)
#define QNET_E_REMOVED                              _HRESULT_TYPEDEF_(0x80070651L)            // 0x80070651 HRESULT_FROM_WIN32(ERROR_DEVICE_REMOVED)
#define QNET_E_SEND_QUEUE_FULL                      _HRESULT_TYPEDEF_(0x800710E8L)            // 0x800710E8 HRESULT_FROM_WIN32(ERROR_TRANSPORT_FULL)
#define QNET_E_SERVER_ERROR                         XONLINE_E_SERVER_ERROR                    // 0x8015000A
#define QNET_E_SESSION_FULL                         XONLINE_E_SESSION_FULL                    // 0x80155202
#define QNET_E_SESSION_ENDED_BY_HOST                _HRESULT_TYPEDEF_(0x800704CAL)            // 0x800704CA HRESULT_FROM_WIN32(ERROR_GRACEFUL_DISCONNECT)
#define QNET_E_SIGNED_IN_TO_LIVE                    _HRESULT_TYPEDEF_(0x80070555L)            // 0x80070555 HRESULT_FROM_WIN32(ERROR_BAD_LOGON_SESSION_STATE)
#define QNET_E_TASK_BUSY                            _HRESULT_TYPEDEF_(0x800700AAL)            // 0x800700AA HRESULT_FROM_WIN32(ERROR_BUSY)
#define QNET_E_TOO_LARGE                            _HRESULT_TYPEDEF_(0x800710F0L)            // 0x800710F0 HRESULT_FROM_WIN32(ERROR_MESSAGE_EXCEEDS_MAX_SIZE)
#define QNET_E_VERSION_MISMATCH                     _HRESULT_TYPEDEF_(0x8007051AL)            // 0x8007051A HRESULT_FROM_WIN32(ERROR_REVISION_MISMATCH)

#define QNET_MAX_NUM_PLAYERS                        31

#define QNET_USER_MASK_USER0                        0x1
#define QNET_USER_MASK_USER1                        0x2
#define QNET_USER_MASK_USER2                        0x4
#define QNET_USER_MASK_USER3                        0x8

#define QNET_SENDDATA_NONRELIABLE                   0x00000000 // send message as best-effort, may be dropped by the network (default)
#define QNET_SENDDATA_RELIABLE                      0x00000001 // ensure that the message arrives or else the connection is lost
#define QNET_SENDDATA_NONSEQUENTIAL                 0x00000000 // do not require any message ordering (default)
#define QNET_SENDDATA_SEQUENTIAL                    0x00000002 // require all previous QNET_SENDDATA_SEQUENTIAL sends to be received before this one
#define QNET_SENDDATA_DIRECT                        0x00000000 // sends data directly to target or targets (default)
#define QNET_SENDDATA_VIA_HOST                      0x00000004 // sends data through host rather than directly to target(s)
#define QNET_SENDDATA_STANDARD                      0x00000000 // indicates that the message is standard game data
#define QNET_SENDDATA_CHAT                          0x00000008 // indicates that the message is chat data
#define QNET_SENDDATA_SECONDARY                     0x00000010 // indicates that the message is game data to be placed in the secondary send queue (cannot be used with QNET_SENDDATA_CHAT)
#define QNET_SENDDATA_LOW_PRIORITY                  0x00000020 // treat the message as low priority when queuing
#define QNET_SENDDATA_NORMAL_PRIORITY               0x00000000 // treat the message as normal priority when queuing (default)
#define QNET_SENDDATA_HIGH_PRIORITY                 0x00000040 // treat the message as high priority when queuing

#define QNET_COMMFILTER_ALLOW_ALL                   0xFFFFFFFF
#define QNET_COMMFILTER_ALLOW_NONE                  0x00000000

#define QNET_GETSENDQUEUESIZE_BYTES                 0x00000000 // return the queue size in number of bytes (default)
#define QNET_GETSENDQUEUESIZE_MESSAGES              0x00000001 // return the queue size in number of messages
#define QNET_GETSENDQUEUESIZE_ALL_LOCAL_USERS       0x00000002 // include queues from all local users to the target, not just the invoked IQNetPlayer object
#define QNET_GETSENDQUEUESIZE_ALL_TARGET_USERS      0x00000004 // include queues to all users on the target system, not just pPlayerTarget
#define QNET_GETSENDQUEUESIZE_ALL_TYPES             0x00000000 // include all types of messages (default)
#define QNET_GETSENDQUEUESIZE_STANDARD_TYPE         0x00000008 // include standard messages
#define QNET_GETSENDQUEUESIZE_CHAT_TYPE             0x00000010 // include chat messages
#define QNET_GETSENDQUEUESIZE_SECONDARY_TYPE        0x00000020 // include secondary send queue messages

#define QNET_GETTRANSFERREDSIZE_BYTES               0x00000000 // return the transferred size in number of bytes (default)
#define QNET_GETTRANSFERREDSIZE_MESSAGES            0x00000001 // return the transferred size in number of messages
#define QNET_GETTRANSFERREDSIZE_SENT                0x00000000 // return the sent transferred size (default)
#define QNET_GETTRANSFERREDSIZE_RECEIVED            0x00000002 // return the received transferred size
#define QNET_GETTRANSFERREDSIZE_ALL_LOCAL_USERS     0x00000004 // include counts for all local users to the target, not just the invoked IQNetPlayer object
#define QNET_GETTRANSFERREDSIZE_ALL_TARGET_USERS    0x00000008 // include counts for all users on the specified target system, not just pPlayerTarget
#define QNET_GETTRANSFERREDSIZE_NETWORK             0x00000010 // return sends/receives transferred over the network, including internal QNet messages; requires QNET_GETTRANSFERREDSIZE_ALL_LOCAL_USERS and _ALL_TARGET_USERS

#define QNET_SMALL_ID_INDEX_MASK                    ((BYTE) 0x1F)
#define QNET_SMALL_ID_VERSION_MASK                  ((BYTE) 0xE0)
#define QNET_SMALL_ID_VERSION_SHIFT                 5

#define QNetGetSmallIdSlotIndex(SmallId)            ((SmallId) & QNET_SMALL_ID_INDEX_MASK)

#define QNET_CANCELQUEUEDSENDS_ALL_TYPES            0x00000000 // cancel all types of messages (default)
#define QNET_CANCELQUEUEDSENDS_STANDARD_TYPE        0x00000001 // cancel standard messages
#define QNET_CANCELQUEUEDSENDS_CHAT_TYPE            0x00000002 // cancel chat messages
#define QNET_CANCELQUEUEDSENDS_SECONDARY_TYPE       0x00000004 // cancel secondary send queue messages


typedef enum _QNET_SEARCHSTATE
{
    QNET_SEARCHSTATE_RETRIEVING_LIST, // the list of available sessions is being retrieved
    QNET_SEARCHSTATE_TESTING_QOS,     // QoS lookups are being performed on the available sessions
    QNET_SEARCHSTATE_DONE             // the search has completed
} QNET_SEARCHSTATE, * PQNET_SEARCHSTATE;

typedef enum _QNET_SESSIONTYPE
{
    QNET_SESSIONTYPE_LOCAL,         // local multiplayer only
    QNET_SESSIONTYPE_SYSTEMLINK,    // System Link multiplayer
    QNET_SESSIONTYPE_LIVE_STANDARD, // LIVE multiplayer using standard matchmaking
    QNET_SESSIONTYPE_LIVE_RANKED    // LIVE multiplayer using ranked matchmaking and arbitration
} QNET_SESSIONTYPE, * PQNET_SESSIONTYPE;

#define QNET_SESSIONTYPE_XBOXLIVE_STANDARD  QNET_SESSIONTYPE_LIVE_STANDARD
#define QNET_SESSIONTYPE_XBOXLIVE_RANKED    QNET_SESSIONTYPE_LIVE_RANKED

typedef enum _QNET_STATE
{
    QNET_STATE_IDLE,
    QNET_STATE_SESSION_HOSTING,
    QNET_STATE_SESSION_JOINING,
    QNET_STATE_GAME_LOBBY,
    QNET_STATE_SESSION_REGISTERING,
    QNET_STATE_SESSION_STARTING,
    QNET_STATE_GAME_PLAY,
    QNET_STATE_SESSION_ENDING,
    QNET_STATE_SESSION_LEAVING,
    QNET_STATE_SESSION_DELETING
} QNET_STATE, * PQNET_STATE;

typedef enum _QNET_CAMERA_STREAMS
{
    QNET_CAMERA_STREAMS_NONE,
    QNET_CAMERA_STREAMS_SEND,
    QNET_CAMERA_STREAMS_RECV,
    QNET_CAMERA_STREAMS_BOTH
} QNET_CAMERA_STREAMS, * PQNET_CAMERA_STREAMS;

typedef enum _QNET_PRESENCE_JOIN_MODE
{
    QNET_PRESENCE_JOIN_MODE_NOT_ALLOWED,
    QNET_PRESENCE_JOIN_MODE_EVERYONE,
    QNET_PRESENCE_JOIN_MODE_FRIENDS_ONLY,
} QNET_PRESENCE_JOIN_MODE, * PQNET_PRESENCE_JOIN_MODE;

typedef enum _QNET_LIVE_STATS_MODE
{
    QNET_LIVE_STATS_MODE_DISABLED,
    QNET_LIVE_STATS_MODE_ENABLED,
    QNET_LIVE_STATS_MODE_AUTO,
} QNET_LIVE_STATS_MODE, * PQNET_LIVE_STATS_MODE;

typedef enum _QNET_LIVE_PRESENCE_MODE
{
    QNET_LIVE_PRESENCE_MODE_NOT_ADVERTISED,
    QNET_LIVE_PRESENCE_MODE_ADVERTISED,
    QNET_LIVE_PRESENCE_MODE_AUTO,
} QNET_LIVE_PRESENCE_MODE, * PQNET_LIVE_PRESENCE_MODE;

typedef enum _QNET_OPTION
{
    QNET_OPTION_NOTIFY_LISTENER                = 0x00000000,
    QNET_OPTION_TOTAL_PUBLIC_SLOTS             = 0x00000001,
    QNET_OPTION_TOTAL_PRIVATE_SLOTS            = 0x00000002,
    QNET_OPTION_FILLED_PUBLIC_SLOTS            = 0x00000003,
    QNET_OPTION_FILLED_PRIVATE_SLOTS           = 0x00000004,
    QNET_OPTION_XSESSION_HANDLE                = 0x00000005,
    QNET_OPTION_SESSIONTYPE                    = 0x00000006,
    QNET_OPTION_JOIN_IN_PROGRESS_ALLOWED       = 0x00000007,
    QNET_OPTION_INVITES_ALLOWED                = 0x00000008,
    QNET_OPTION_PRESENCE_JOIN_MODE             = 0x00000009,
    QNET_OPTION_CALLBACK_INTERFACE             = 0x0000000A,
    QNET_OPTION_QOS_DATA_BUFFER                = 0x0000000B,
    QNET_OPTION_XHVENGINE_INTERFACE            = 0x0000000C,
    QNET_OPTION_XCAMSTREAMENGINE_INTERFACE     = 0x0000000D,
    QNET_OPTION_IMMEDIATE_SENDS_ALLOWED        = 0x0000000E,
    QNET_OPTION_NEVER_DROP_CONNECTIONS         = 0x0000000F,
    QNET_OPTION_NET_CALLBACKS_SUSPENDED        = 0x00000010,
    QNET_OPTION_LIVE_STATS_MODE                = 0x00000011,
    QNET_OPTION_LIVE_PRESENCE_MODE             = 0x00000012,
    QNET_OPTION_XAUDIO_VOICE_SETTINGS          = 0x00000018,
    QNET_OPTION_XAUDIO2_VOICE_SETTINGS         = 0x00000019,
    QNET_OPTION_XHV2ENGINE_INTERFACE           = 0x0000001A,
} QNET_OPTION, * PQNET_OPTION;

#define QNET_OPTION_TYPE_NOTIFY_LISTENER                QNET_OPTION_NOTIFY_LISTENER
#define QNET_OPTION_TYPE_TOTAL_PUBLIC_SLOTS             QNET_OPTION_TOTAL_PUBLIC_SLOTS
#define QNET_OPTION_TYPE_TOTAL_PRIVATE_SLOTS            QNET_OPTION_TOTAL_PRIVATE_SLOTS
#define QNET_OPTION_TYPE_FILLED_PUBLIC_SLOTS            QNET_OPTION_FILLED_PUBLIC_SLOTS
#define QNET_OPTION_TYPE_FILLED_PRIVATE_SLOTS           QNET_OPTION_FILLED_PRIVATE_SLOTS
#define QNET_OPTION_TYPE_XSESSION_HANDLE                QNET_OPTION_XSESSION_HANDLE
#define QNET_OPTION_TYPE_SESSIONTYPE                    QNET_OPTION_SESSIONTYPE
#define QNET_OPTION_TYPE_JOIN_IN_PROGRESS_ALLOWED       QNET_OPTION_JOIN_IN_PROGRESS_ALLOWED
#define QNET_OPTION_TYPE_INVITES_ALLOWED                QNET_OPTION_INVITES_ALLOWED
#define QNET_OPTION_TYPE_PRESENCE_JOIN_MODE             QNET_OPTION_PRESENCE_JOIN_MODE
#define QNET_OPTION_TYPE_CALLBACK_INTERFACE             QNET_OPTION_CALLBACK_INTERFACE
#define QNET_OPTION_TYPE_QOS_DATA_BUFFER                QNET_OPTION_QOS_DATA_BUFFER
#define QNET_OPTION_TYPE_XHVENGINE_INTERFACE            QNET_OPTION_XHVENGINE_INTERFACE
#define QNET_OPTION_TYPE_XCAMSTREAMENGINE_INTERFACE     QNET_OPTION_XCAMSTREAMENGINE_INTERFACE
#define QNET_OPTION_TYPE_IMMEDIATE_SENDS_ALLOWED        QNET_OPTION_IMMEDIATE_SENDS_ALLOWED
#define QNET_OPTION_TYPE_NEVER_DROP_CONNECTIONS         QNET_OPTION_NEVER_DROP_CONNECTIONS
#define QNET_OPTION_TYPE_NET_CALLBACKS_SUSPENDED        QNET_OPTION_NET_CALLBACKS_SUSPENDED
#define QNET_OPTION_TYPE_LIVE_STATS_MODE                QNET_OPTION_LIVE_STATS_MODE
#define QNET_OPTION_TYPE_LIVE_PRESENCE_MODE             QNET_OPTION_LIVE_PRESENCE_MODE
#define QNET_OPTION_TYPE                                QNET_OPTION
#define PQNET_OPTION_TYPE                               PQNET_OPTION


typedef enum _QNET_PLAYER_OPTION
{
    QNET_PLAYER_OPTION_XAUDIO_VOICE_SETTINGS         = 0x00000001,
    QNET_PLAYER_OPTION_XAUDIO2_VOICE_SETTINGS        = 0x00000002,
    QNET_PLAYER_OPTION_XAUDIO_VOICE_EFFECT_PARAM     = 0x00000003,
    QNET_PLAYER_OPTION_XAUDIO2_VOICE_EFFECT_PARAM    = 0x00000004,
} QNET_PLAYER_OPTION, * PQNET_PLAYER_OPTION;

typedef VOID (WINAPI *PFNQNETMICROPHONERAWDATAREADY)(
    __in  DWORD                 dwUserIndex,
    __in_bcount(dwSize) PVOID   pvData,
    __in  DWORD                 dwSize,
    __out PBOOL                 pfVoiceDetected
    );

#ifdef __XAUDIO_INCLUDED__
typedef struct _QNET_XAUDIO_GLOBAL_VOICE_SETTINGS
{
    PFNQNETMICROPHONERAWDATAREADY   pfnMicrophoneRawDataReady;
    BOOL                            fCustomVoiceActivationDetection;
    LPCXAUDIOVOICEFXCHAIN           pDefaultEffectChain;
    LPCXAUDIOVOICEFXCHAIN           pDefaultPerLocalUserPairedEffectChain;
    LPCXAUDIOVOICEFXCHAIN           pOutputEffectChain;
} QNET_XAUDIO_GLOBAL_VOICE_SETTINGS, * PQNET_XAUDIO_GLOBAL_VOICE_SETTINGS;

typedef struct _QNET_XAUDIO_PLAYER_VOICE_SETTINGS
{
    LPCXAUDIOVOICEFXCHAIN           pAllLocalUsersEffectChain;
    LPCXAUDIOVOICEFXCHAIN           pPerLocalUserPairedEffectChain;
    LPXAUDIOSUBMIXVOICE             pOutputVoice;
} QNET_XAUDIO_PLAYER_VOICE_SETTINGS, * PQNET_XAUDIO_PLAYER_VOICE_SETTINGS;

typedef struct _QNET_XAUDIO_VOICE_EFFECT_PARAM
{
    BOOL                            fPairEffect;
    XAUDIOVOICEFXINDEX              EffectIndex;
    XAUDIOFXPARAMID                 ParamId;
    XAUDIOFXPARAMTYPE               ParamType;
    XAUDIOFXPARAM                   Param;
} QNET_XAUDIO_VOICE_EFFECT_PARAM, * PQNET_XAUDIO_VOICE_EFFECT_PARAM;
#endif // end if defined __XAUDIO_INCLUDED__

#ifdef __XAUDIO2_INCLUDED__
typedef struct _QNET_XAUDIO2_GLOBAL_VOICE_SETTINGS
{
    PFNQNETMICROPHONERAWDATAREADY   pfnMicrophoneRawDataReady;
    BOOL                            fCustomVoiceActivationDetection;
    XAUDIO2_EFFECT_CHAIN **         ppDefaultEffectChains;
    XAUDIO2_EFFECT_CHAIN **         ppDefaultPerLocalUserPairedEffectChains;
    XAUDIO2_EFFECT_CHAIN *          pOutputEffectChain;
} QNET_XAUDIO2_GLOBAL_VOICE_SETTINGS, * PQNET_XAUDIO2_GLOBAL_VOICE_SETTINGS;

typedef struct _QNET_XAUDIO2_PLAYER_VOICE_SETTINGS
{
    XAUDIO2_EFFECT_CHAIN *          pAllLocalUsersEffectChain;
    XAUDIO2_EFFECT_CHAIN *          pPerLocalUserPairedEffectChain;
    IXAudio2SubmixVoice *           pOutputVoice;
} QNET_XAUDIO2_PLAYER_VOICE_SETTINGS, * PQNET_XAUDIO2_PLAYER_VOICE_SETTINGS;

typedef struct _QNET_XAUDIO2_VOICE_EFFECT_PARAM
{
    BOOL                            fPairEffect;
    UINT32                          uiEffectIndex;
    const void *                    pvParameters;
    UINT32                          uiParametersByteSize;
} QNET_XAUDIO2_VOICE_EFFECT_PARAM, * PQNET_XAUDIO2_VOICE_EFFECT_PARAM;
#endif // end if defined __XAUDIO2_INCLUDED__


//
// IQNetPlayer interface definition
//
class IQNetPlayer
{
public:
    virtual
    XUID
    GetXuid(
        ) = 0;

    virtual
    DWORD
    GetUserIndex(
        ) = 0;

    virtual
    DWORD
    GetSessionIndex(
        ) = 0;

    virtual
    ULONG_PTR
    GetCustomDataValue(
        ) = 0;

    virtual
    VOID
    SetCustomDataValue(
        __in_opt ULONG_PTR                      ulpCustomDataValue
        ) = 0;

    virtual
    LPCWSTR
    GetGamertag(
        ) = 0;

    virtual
    XONLINE_NAT_TYPE
    GetNatType(
        ) = 0;

    virtual
    HRESULT
    GetXnaddr(
        __out XNADDR *                          pxnaddr
        ) = 0;

    virtual
    DWORD
    GetCurrentRtt(
        ) = 0;

    virtual
    BOOL
    IsLocal(
        ) = 0;

    virtual
    BOOL
    IsHost(
        ) = 0;

    virtual
    BOOL
    IsPrivateSlot(
        ) = 0;

    virtual
    BOOL
    HasVoice(
        ) = 0;

    virtual
    BOOL
    HasCamera(
        ) = 0;

    virtual
    BOOL
    IsTalking(
        ) = 0;

    virtual
    BOOL
    IsReady(
        ) = 0;

    virtual
    BOOL
    IsMutedByLocalUser(
        __in DWORD                              dwUserIndex
        ) = 0;

    virtual
    BOOL
    IsSameSystem(
        __in IQNetPlayer *                      pPlayerOther
        ) = 0;

    virtual
    HRESULT
    SetReady(
        __in BOOL                               bReady
        ) = 0;

    virtual
    HRESULT
    SendData(
        __in_opt IQNetPlayer *                  pPlayerTarget,
        __in_bcount(dwDataSize) const void *    pvData,
        __in DWORD                              dwDataSize,
        __in DWORD                              dwFlags
        ) = 0;

    virtual
    HRESULT
    RemoveSystemFromGame(
        ) = 0;

    virtual
    IQNetPlayer *
    GetPlayerOnSameSystem(
        __in DWORD                              dwUserIndex
        ) = 0;

    virtual
    HRESULT
    WriteStats(
        __in DWORD                              dwNumViews,
        __in_ecount(dwNumViews) CONST XSESSION_VIEW_PROPERTIES * pViews
        ) = 0;

    virtual
    VOID
    GetCommFilter(
        __out_opt DWORD *                       pdwVoiceSendMask,
        __out_opt DWORD *                       pdwVoiceRecvMask,
        __out_opt DWORD *                       pdwCameraSendMask,
        __out_opt DWORD *                       pdwCameraRecvMask
        ) = 0;

    virtual
    HRESULT
    SetCommFilter(
        __in DWORD                              dwVoiceSendMask,
        __in DWORD                              dwVoiceRecvMask,
        __in DWORD                              dwCameraSendMask,
        __in DWORD                              dwCameraRecvMask
        ) = 0;

    virtual
    DWORD
    GetSendQueueSize(
        __in_opt IQNetPlayer *                  pPlayerTarget,
        __in DWORD                              dwFlags
        ) = 0;

    virtual
    ULONGLONG
    GetTransferredSize(
        __in_opt IQNetPlayer *                  pPlayerTarget,
        __in DWORD                              dwFlags
        ) = 0;

    virtual
    HRESULT
    GetPlayerOpt(
        __in QNET_PLAYER_OPTION                 Option,
        __out_bcount_opt(*pdwOptionDataSize) void * pvOptionData,
        __inout DWORD *                         pdwOptionDataSize
        ) = 0;

    virtual
    HRESULT
    SetPlayerOpt(
        __in QNET_PLAYER_OPTION                 Option,
        __in_bcount(dwOptionDataSize) void *    pvOptionData,
        __in DWORD                              dwOptionDataSize
        ) = 0;

    virtual
    BOOL
    IsGuest(
        ) = 0;

    virtual
    BYTE
    GetSmallId(
        ) = 0;
};


#ifdef _XBOX
//
// IQNetCameraManager interface definition
//
class IQNetCameraManager
{
public:
    virtual
    VOID
    Destroy(
        ) = 0;

    virtual
    VOID
    SetCameraStreams(
        __in QNET_CAMERA_STREAMS                CameraStreams
        ) = 0;

    virtual
    VOID
    SetCameraView(
        __in XCAMZOOMFACTOR                     ZoomFactor,
        __in int                                xCenter,
        __in int                                yCenter
        ) = 0;

    virtual
    BOOL
    IsSetCameraViewPending(
        ) = 0;

    virtual
    HRESULT
    SetRenderRectForSystem(
        __in_opt IQNetPlayer *                  pPlayerOnSystem,
        __in CONST D3DRECT *                    pRect
        ) = 0;

    virtual
    HRESULT
    SetRemoteSystemDefaultTexture(
        __in IQNetPlayer *                      pPlayerOnSystem,
        __in_opt D3DTexture *                   pDefaultTexture
        ) = 0;

    virtual
    HRESULT
    RenderSystem(
        __in_opt IQNetPlayer *                  pPlayerOnSystem
        ) = 0;

    virtual
    HRESULT
    GetNextFrameForRemoteSystem (
        __in IQNetPlayer *                      pPlayerOnSystem,
        __inout D3DLOCKED_RECT *                pYBuffer,
        __inout D3DLOCKED_RECT *                pUBuffer,
        __inout D3DLOCKED_RECT *                pVBuffer
        ) = 0;

    virtual
    HRESULT
    GetNextLocalPreviewFrame(
        __inout D3DLOCKED_RECT *                pBuffer
        ) = 0;

    virtual
    HRESULT
    GetNextFrameForSystemRGB(
        __in_opt IQNetPlayer *                  pPlayerOnSystem,
        __inout D3DLOCKED_RECT *                pBuffer
        ) = 0;
};
#endif


//
// IQNetGameSearch interface definition
//
class IQNetGameSearch
{
public:
    virtual
    VOID
    Destroy(
        ) = 0;

    virtual
    QNET_SEARCHSTATE
    GetState(
        ) = 0;

    virtual
    BOOL
    IsComplete(
        ) = 0;

    virtual
    HRESULT
    GetResultCode(
        ) = 0;

    virtual
    VOID
    Abort(
        ) = 0;

    virtual
    DWORD
    GetNumResults(
        ) = 0;

    virtual
    const XSESSION_SEARCHRESULT *
    GetSearchResultAtIndex(
        __in DWORD                              dwIndex
        ) = 0;

    virtual
    const XNQOSINFO *
    GetQosInfoAtIndex(
        __in DWORD                              dwIndex
        ) = 0;
};


//
// IQNetCallbacks interface definition
//
class IQNetCallbacks
{
public:
    virtual
    VOID
    NotifyStateChanged(
        __in QNET_STATE                 OldState,
        __in QNET_STATE                 NewState,
        __in HRESULT                    hrInfo
        ) = 0;

    virtual
    VOID
    NotifyPlayerJoined(
        __in IQNetPlayer *              pPlayer
        ) = 0;

    virtual
    VOID
    NotifyPlayerLeaving(
        __in IQNetPlayer *              pPlayer
        ) = 0;

    virtual
    VOID
    NotifyNewHost(
        __in IQNetPlayer *              pPlayer
        ) = 0;

    virtual
    VOID
    NotifyDataReceived(
        __in IQNetPlayer *              pPlayerFrom,
        __in DWORD                      dwNumPlayersTo,
        __in_ecount(dwNumPlayersTo) IQNetPlayer ** apPlayersTo,
        __in_bcount(dwDataSize) const BYTE * pbData,
        __in DWORD                      dwDataSize
        ) = 0;

    virtual
    VOID
    NotifyWriteStats(
        __in IQNetPlayer *              pPlayer
        ) = 0;

    virtual
    VOID
    NotifyReadinessChanged(
        __in IQNetPlayer *              pPlayer,
        __in BOOL                       bReady
        ) = 0;

    virtual
    VOID
    NotifyCommSettingsChanged(
        __in IQNetPlayer *              pPlayer
        ) = 0;

    virtual
    VOID
    NotifyGameSearchComplete(
        __in IQNetGameSearch *          pGameSearch,
        __in HRESULT                    hrComplete,
        __in DWORD                      dwNumResults
        ) = 0;

    virtual
    VOID
    NotifyGameInvite(
        __in DWORD                      dwUserIndex,
        __in const XINVITE_INFO *       pInviteInfo
        ) = 0;

    virtual
    VOID
    NotifyContextChanged(
        __in const XUSER_CONTEXT *      pContext
        ) = 0;

    virtual
    VOID
    NotifyPropertyChanged(
        __in const XUSER_PROPERTY *     pProperty
        ) = 0;
};


//
// IQNet interface definition
//
class IQNet
{
public:
    virtual
    VOID
    Destroy(
        ) = 0;

    virtual
    VOID
    DoWork(
        ) = 0;

    virtual
    HRESULT
    HostGame(
        __in DWORD                              dwUserIndexHost,
        __in DWORD                              dwUserMask,
        __in DWORD                              dwPublicSlots,
        __in DWORD                              dwPrivateSlots,
        __in DWORD                              cProperties,
        __in_ecount_opt(cProperties) const XUSER_PROPERTY * pProperties,
        __in DWORD                              cContexts,
        __in_ecount(cContexts) const XUSER_CONTEXT * pContexts
        ) = 0;

    virtual
    HRESULT
    JoinGameFromSearchResult(
        __in DWORD                              dwUserIndex,
        __in DWORD                              dwUserMask,
        __in const XSESSION_SEARCHRESULT *      pSearchResult
        ) = 0;

    virtual
    HRESULT
    JoinGameFromInviteInfo(
        __in DWORD                              dwUserIndex,
        __in DWORD                              dwUserMask,
        __in const XINVITE_INFO *               pInviteInfo
        ) = 0;

    virtual
    HRESULT
    LeaveGame(
        __in BOOL                               bMigrateHost
        ) = 0;

    virtual
    HRESULT
    StartGame(
        ) = 0;

    virtual
    HRESULT
    EndGame(
        ) = 0;

#ifdef _XBOX
    virtual
    HRESULT
    CreateCameraManager(
        __in XCAMRESOLUTION                     Resolution,
        __in XCAMFRAMERATE                      Framerate,
        __in DWORD                              dwBitRate,
        __in_opt IDirect3DDevice9 *             pd3dDevice,
        __deref_out IQNetCameraManager **       ppIQNetCameraManager
        ) = 0;
#else
    virtual
    HRESULT
    CreateCameraManager(
        __in DWORD                              dwReserved1,
        __in DWORD                              dwReserved2,
        __in DWORD                              dwReserved3,
        __in_opt void *                         pvReserved4,
        __deref_out void **                     ppvReserved5
        ) = 0;
#endif

    virtual
    HRESULT
    CreateGameSearch(
        __in DWORD                              dwUserIndex,
        __in DWORD                              dwUserMask,
        __in DWORD                              dwMatchQueryIndex,
        __in DWORD                              dwMaxResults,
        __in DWORD                              cProperties,
        __in_ecount_opt(cProperties) const XUSER_PROPERTY * pProperties,
        __in DWORD                              cContexts,
        __in_ecount(cContexts) const XUSER_CONTEXT * pContexts,
        __deref_out IQNetGameSearch **          ppGameSearch
        ) = 0;

    virtual
    HRESULT
    CreateGameSearchById(
        __in DWORD                              dwUserIndex,
        __in XNKID                              xnkidSessionId,
        __deref_out IQNetGameSearch **          ppGameSearch
        ) = 0;

    virtual
    BOOL
    IsHost(
        ) = 0;

    virtual
    IQNetPlayer *
    GetHostPlayer(
        ) = 0;

    virtual
    DWORD
    GetPlayerCount(
        ) = 0;

    virtual
    IQNetPlayer *
    GetPlayerByIndex(
        __in DWORD                              dwPlayerIndex
        ) = 0;

    virtual
    IQNetPlayer *
    GetPlayerByXuid(
        __in XUID                               xuid
        ) = 0;

    virtual
    IQNetPlayer *
    GetLocalPlayerByUserIndex(
        __in DWORD                              dwUserIndex
        ) = 0;

    virtual
    HRESULT
    AddLocalPlayerByUserIndex(
        __in DWORD                              dwUserIndex
        ) = 0;

    virtual
    HRESULT
    RemoveLocalPlayerByUserIndex(
        __in DWORD                              dwUserIndex
        ) = 0;

    virtual
    HRESULT
    ResetReady(
        ) = 0;

    virtual
    BOOL
    IsEveryoneReady(
        ) = 0;

    virtual
    HRESULT
    SetGameContext(
        __in const XUSER_CONTEXT *              pContext
        ) = 0;

    virtual
    const XUSER_CONTEXT *
    GetGameContext(
        __in DWORD                              dwContextId
        ) = 0;

    virtual
    HRESULT
    SetGameProperty(
        __in const XUSER_PROPERTY *             pProperty
        ) = 0;

    virtual
    const XUSER_PROPERTY *
    GetGameProperty(
        __in DWORD                              dwPropertyId
        ) = 0;

    virtual
    HRESULT
    FlushStats(
        __inout_opt XOVERLAPPED *               pXOverlapped
        ) = 0;

    virtual
    QNET_STATE
    GetState(
        ) = 0;

    virtual
    HRESULT
    GetLastStateResult(
        ) = 0;

    virtual
    HRESULT
    Notify(
        __in DWORD                              dwID,
        __in ULONG_PTR                          Param
        ) = 0;

    virtual
    HRESULT
    GetOpt(
        __in QNET_OPTION                        Option,
        __inout void *                          pvOptionData,
        __out_bcount_opt( *pvOptionData ) DWORD * pdwOptionDataSize
        ) = 0;

    virtual
    HRESULT
    SetOpt(
        __in QNET_OPTION                        Option,
        __in_bcount(dwOptionDataSize) void *    pvOptionData,
        __in DWORD                              dwOptionDataSize
        ) = 0;

    virtual
    IQNetPlayer *
    GetPlayerBySmallId(
        __in BYTE                               SmallId
        ) = 0;

    virtual
    HRESULT
    SendDataEnhanced(
        __in_opt IQNetPlayer *                  pPlayerSource,
        __in_ecount_opt(dwNumTargets) IQNetPlayer ** ppPlayerTargets,
        __in DWORD                              dwNumTargets,
        __in_bcount(dwDataSize) const void *    pvData,
        __in DWORD                              dwDataSize,
        __in WORD                               wCancelBitValue,
        __in DWORD                              dwQueueTimeoutMs,
        __in DWORD                              dwFlags
        ) = 0;

    virtual
    HRESULT
    CancelQueuedSends(
        __in_opt IQNetPlayer *                  pPlayerSource,
        __in_opt IQNetPlayer *                  pPlayerTarget,
        __in WORD                               wCancelBitMask,
        __in DWORD                              dwFlags,
        __out_opt DWORD *                       pdwNumSendsCancelled
        ) = 0;
};



    // The old XAudio1-dependent QNet API functions were removed in June 2009

    HRESULT
    WINAPI
    QNetCreate(
        __in QNET_SESSIONTYPE       SessionType,
        __in IQNetCallbacks *       pIQNetCallbacks,
        __deref_out IQNet **        ppIQNet
        );

    HRESULT
    WINAPI
    QNetCreateUsingWindow(
        __in QNET_SESSIONTYPE       SessionType,
        __in IQNetCallbacks *       pIQNetCallbacks,
        __in_opt HWND               hwndFocus,
        __deref_out IQNet **        ppIQNet
        );


HRESULT
WINAPI
QNetCreateUsingXAudio2(
    __in QNET_SESSIONTYPE           SessionType,
    __in IQNetCallbacks *           pIQNetCallbacks,
    __in_opt HWND                   hwndFocus,
    __in IXAudio2 *                 pXAudio2,
    __deref_out IQNet **            ppIQNet
    );


#endif // end if not defined __QNET_H__
