/************************************************************************
*                                                                       *
*   xauth.h -- This module defines the non-XLSP connectivity feature    *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/
#ifndef __XAUTH_H__
#define __XAUTH_H__

#ifdef __cplusplus
extern "C" {
#endif
//-----------------------------------------------------------------------------
// Non-XLSP third party connectivity constants
//-----------------------------------------------------------------------------

#define XAUTH_FLAG_BYPASS_SECURITY 0x00000001

//-----------------------------------------------------------------------------
// Non-XLSP third party connectivity structs
//-----------------------------------------------------------------------------

typedef struct _XAUTH_SETTINGS
{
    DWORD SizeOfStruct;
    DWORD Flags;
} XAUTH_SETTINGS, * PXAUTH_SETTINGS;

//-----------------------------------------------------------------------------
// Non-XLSP third party connectivity APIs
//-----------------------------------------------------------------------------

XBOXAPI
HRESULT
WINAPI
XAuthStartup(
    __in CONST XAUTH_SETTINGS * Settings
);

XBOXAPI
VOID
WINAPI
XAuthShutdown(
    VOID
);

//-----------------------------------------------------------------------------
// Xbox Live service single sign on tokens
//-----------------------------------------------------------------------------

typedef struct _RELYING_PARTY_TOKEN
{
    DWORD Reserved;
    DWORD Length;
    PBYTE pToken;
} RELYING_PARTY_TOKEN, *PRELYING_PARTY_TOKEN;

XBOXAPI
HRESULT
WINAPI
XAuthGetToken (
    __in DWORD                     UserIndex,
    __in CONST CHAR *              pUrn,
    __in DWORD                     UrnLength,
    __inout RELYING_PARTY_TOKEN ** pToken,
    __inout_opt XOVERLAPPED *      pXOverlapped
   );

XBOXAPI
VOID 
WINAPI
XAuthFreeToken (
   __inout RELYING_PARTY_TOKEN * pToken
   );

XBOXAPI
HRESULT
WINAPI
XAuthGetRelyingPartyId (
    __in CONST char * pUrn,
    __in DWORD        UrnLength,
    __inout DWORD *   pRelyPartyIDLength,
    __inout CHAR *    pRelyingPartyID
    );

XBOXAPI
BOOL
WINAPI
XAuthInsecureSocketsAllowed();


#ifdef __cplusplus
}
#endif // __cplusplus

#endif // __XAUTH_H__

