/*++

Copyright (c) Microsoft Corporation.  All rights reserved.

Module Name:

    Xbc.h

Abstract:

    Contains the Xbc (SmartGlass) title library API definitions

--*/
#pragma once

#include <xjson.h>

#define XBC_MAX_CLIENTS         4
#define XBC_CLIENT_BROADCAST    0xffffffff
#define XBC_MAX_MESSAGE_LENGTH  1024

#pragma pack(push, 1)

typedef enum _XBC_DELIVERY_METHOD
{
    XBC_DELIVERY_RELIABLE,
    XBC_DELIVERY_UNRELIABLE
} XBC_DELIVERY_METHOD;

typedef enum _XBC_PROTOCOL_ID
{
    XBC_PROTOCOL_NONE = 0,
    XBC_PROTOCOL_TCP = 1,
    XBC_PROTOCOL_CLOUD = 2,
} XBC_PROTOCOL_ID;

typedef struct _XBC_ACCELEROMETER_DATA
{
    float x;
    float y;
    float z;
} XBC_ACCELEROMETER_DATA, *PXBC_ACCELEROMETER_DATA;

typedef struct _XBC_GYROSCOPE_DATA
{
    float x;
    float y;
    float z;
} XBC_GYROSCOPE_DATA, *PXBC_GYROSCOPE_DATA;

typedef enum _XBC_TOUCH_ACTION
{
    XBC_TOUCH_ACTION_NONE = 0x00,
    XBC_TOUCH_ACTION_DOWN = 0x01,
    XBC_TOUCH_ACTION_MOVE = 0x02,
    XBC_TOUCH_ACTION_UP = 0x4,
} XBC_TOUCH_ACTION;

typedef struct _XBC_TOUCH_POINT
{
    DWORD nId;
    XBC_TOUCH_ACTION eAction;
    double x;
    double y;
} XBC_TOUCH_POINT, *PXBC_TOUCH_POINT;

typedef struct _XBC_TOUCH_FRAME
{
    DWORD nTouchPoints;
    XBC_TOUCH_POINT pTouchPoints[1];
} XBC_TOUCH_FRAME, *PXBC_TOUCH_FRAME;

typedef enum _XBC_EVENT_TYPE
{
    XBC_EVENT_CLIENT_CONNECTED = 1,
    XBC_EVENT_CLIENT_DISCONNECTED = 2,
    XBC_EVENT_JSON_SEND_COMPLETE = 3,
    XBC_EVENT_JSON_RECEIVE_COMPLETE = 4,
    XBC_EVENT_BINARY_RECEIVE_ACCELEROMETER_DATA = 5,
    XBC_EVENT_BINARY_RECEIVE_GYROSCOPE_DATA = 6,
    XBC_EVENT_BINARY_RECEIVE_TOUCH_FRAME = 7,
} XBC_EVENT_TYPE;

typedef struct _XBC_EVENT_PARAMS
{
    DWORD nClientId;
    DWORD nUserIndex;
    DWORD nTimeStamp;
    XBC_EVENT_TYPE Type;
    union
    {
        XBC_PROTOCOL_ID ProtocolId;
        HJSONREADER hReader;
        VOID* pvSendState;
        PXBC_ACCELEROMETER_DATA pAccelerometerData;
        PXBC_GYROSCOPE_DATA pGyroscopeData;
        PXBC_TOUCH_FRAME pTouchFrame;
    };
} XBC_EVENT_PARAMS, *PXBC_EVENT_PARAMS;

typedef VOID (CALLBACK *XBC_EVENT_CALLBACK)(__in HRESULT hr, __in PXBC_EVENT_PARAMS pParams, __in_opt VOID* pvState);

HRESULT WINAPI XbcInitialize(__in XBC_EVENT_CALLBACK pCallback, __in_opt VOID* pvState);
VOID WINAPI XbcShutdown();

HRESULT WINAPI XbcDoWork();

HRESULT WINAPI XbcSendJSON(__in XBC_DELIVERY_METHOD eDeliveryMethod, __in DWORD nClientId, __in HJSONWRITER hWriter, __in_opt VOID* pvSendState);

HRESULT WINAPI XbcDisconnectClient(__in DWORD nClientId);

#pragma pack(pop)

