/************************************************************************
*                                                                       *
*   Xbox.h -- This module defines the Xbox APIs                         *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/
#pragma once

#ifndef _XBOX_H_
#define _XBOX_H_

#include <sal.h>


//
// Define API decoration for direct importing of DLL references.
//

#define XBOXAPI

#ifdef __cplusplus
extern "C" {
#endif

#include <xconfig.h>
#include <xinputdefs.h>

#define XUSER_NAME_SIZE                 16
#define XUSER_MAX_NAME_LENGTH           (XUSER_NAME_SIZE - 1)

typedef ULONGLONG                       XUID;
typedef XUID                            *PXUID;

#define INVALID_XUID                    ((XUID) 0)


XBOXAPI
FORCEINLINE
BOOL
WINAPI
IsEqualXUID(
    __in    XUID                        xuid1,
    __in    XUID                        xuid2
    )
{
    return (xuid1 == xuid2);
}

XBOXAPI
FORCEINLINE
BOOL
WINAPI
IsTeamXUID(
    __in XUID xuid
    )
{
    return (xuid & 0xFF00000000000000) == 0xFE00000000000000;
}


XBOXAPI
DWORD
WINAPI
XGetLanguage(
    VOID
    );

#define XGetLanguage    XTLGetLanguageV3
#define XTLGetLanguage  XTLGetLanguageV2
#define XGetLocale      XTLGetLocaleV2

XBOXAPI
DWORD
WINAPI
XTLGetLanguageV3(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XTLGetLanguageV2(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XTLGetLocaleV2(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XTLGetLocaleV1(
    VOID
    );


typedef struct _XVIDEO_MODE {
    DWORD                               dwDisplayWidth;
    DWORD                               dwDisplayHeight;
    BOOL                                fIsInterlaced;
    BOOL                                fIsWideScreen;
    BOOL                                fIsHiDef;
    float                               RefreshRate;
    DWORD                               VideoStandard;
    DWORD                               Reserved[5];
} XVIDEO_MODE, *PXVIDEO_MODE;

#define XC_VIDEO_STANDARD_NTSC_M        1
#define XC_VIDEO_STANDARD_NTSC_J        2
#define XC_VIDEO_STANDARD_PAL_I         3

XBOXAPI
VOID
WINAPI
XGetVideoMode(
    __out   PXVIDEO_MODE                pVideoMode
    );


#define XC_VIDEO_FLAGS_WIDESCREEN       0x00000001
#define XC_VIDEO_FLAGS_HDTV_720p        0x00000002
#define XC_VIDEO_FLAGS_HDTV_480p        0x00000008
#define XC_VIDEO_FLAGS_HDTV_1080i       0x00000004

XBOXAPI
DWORD
WINAPI
XGetVideoCapabilities(
    VOID
    );

#define XC_VIDEO_STEREOSCOPIC_3D_ENABLED            0x00000001
#define XC_VIDEO_STEREOSCOPIC_3D_720P_60HZ          0x00000010

//
// Functions to control the current audio output configuration.
//


typedef DWORD XAUDIOSPEAKERCONFIG, *LPXAUDIOSPEAKERCONFIG;

#define XAUDIOSPEAKERCONFIG_ANALOG_SURROUND         0x00000001
#define XAUDIOSPEAKERCONFIG_ANALOG_MONO             0x00000002
#define XAUDIOSPEAKERCONFIG_ANALOG_STEREOBYPASS     0x00000003
#define XAUDIOSPEAKERCONFIG_ANALOG_MASK             0x0000FFFF

#define XAUDIOSPEAKERCONFIG_DIGITAL_DOLBYDIGITAL    0x00010000
#define XAUDIOSPEAKERCONFIG_DIGITAL_WMAPRO          0x00020000
#define XAUDIOSPEAKERCONFIG_ANALOG_LOW_LATENCY      0x80000000
#define XAUDIOSPEAKERCONFIG_DIGITAL_MASK            0xFFFF0000


// Provide new names for these APIs without actually changing the names
// of the kernel entry points
#define XGetSpeakerConfig XAudioGetSpeakerConfig

XBOXAPI
HRESULT
WINAPI
XAudioGetSpeakerConfig(
    __out LPXAUDIOSPEAKERCONFIG pConfig
    );

#define XOverrideSpeakerConfig XAudioOverrideSpeakerConfig

XBOXAPI
HRESULT
WINAPI
XAudioOverrideSpeakerConfig(
    __in XAUDIOSPEAKERCONFIG Config
    );


//
// Analog speaker configuration values are a enumeration while digital values
// are a bitmask.  0 is not a valid analog value, but 0 in the digital word
// indicates that no digital encoding or preencoded output is allowed on the
// digital channel.  In that case, the digital channel will output the same
// data as the analog.
//

#define XC_CONSOLE_REGION_NA            0x00
#define XC_CONSOLE_REGION_ASIA          0x01
#define XC_CONSOLE_REGION_EUROPE        0x02
#define XC_CONSOLE_REGION_RESTOFWORLD   0x03
#define XC_CONSOLE_REGION_MAXIMUM       0x04
#define XC_CONSOLE_REGION_DEVKIT        0x7F

#define XC_GAME_REGION(region, subregion) \
    ((XC_CONSOLE_REGION_##region << 8) | (subregion))

#define XC_GAME_REGION_NA_ALL           XC_GAME_REGION(NA, 0xFF)
#define XC_GAME_REGION_ASIA_ALL         XC_GAME_REGION(ASIA, 0xFF)
#define XC_GAME_REGION_ASIA_JAPAN       XC_GAME_REGION(ASIA, 0x01)
#define XC_GAME_REGION_ASIA_CHINA       XC_GAME_REGION(ASIA, 0x02)
#define XC_GAME_REGION_ASIA_REST        XC_GAME_REGION(ASIA, 0xFC)
#define XC_GAME_REGION_EUROPE_ALL       XC_GAME_REGION(EUROPE, 0xFF)
#define XC_GAME_REGION_EUROPE_AUNZ      XC_GAME_REGION(EUROPE, 0x01)
#define XC_GAME_REGION_EUROPE_REST      XC_GAME_REGION(EUROPE, 0xFE)
#define XC_GAME_REGION_RESTOFWORLD_ALL  XC_GAME_REGION(RESTOFWORLD, 0xFF)
#define XC_GAME_REGION_DEVKIT_ALL       XC_GAME_REGION(DEVKIT, 0xFF)
#define XC_GAME_REGION_MANUFACTURING    0x8000
#define XC_GAME_REGION_UNRESTRICTED     0xFFFF

#define XC_GAME_REGION_REGION(region) \
    (((region) & 0xFF00) >> 8)

XBOXAPI
USHORT
WINAPI
XGetDisplaySize(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XGetGameRegion(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XInputGetState(
    __in    DWORD                       dwUserIndex,
    __out   PXINPUT_STATE               pState
    );

XBOXAPI
DWORD
WINAPI
XInputGetStateEx(
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwFlags,
    __out   PXINPUT_STATE               pState
    );

XBOXAPI
DWORD
WINAPI
XInputGetCapabilities(
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwFlags,
    __out   PXINPUT_CAPABILITIES        pCapabilities
    );

XBOXAPI
DWORD
WINAPI
XInputSetState(
    __in    DWORD                       dwUserIndex,
    __in    PXINPUT_VIBRATION           pVibration
    );

XBOXAPI
DWORD
XInputRemap(
    __in    DWORD                       rgMapping[XUSER_MAX_COUNT]
    );



XBOXAPI
DWORD
WINAPI
XInputGetKeyboardState(
    __in    DWORD                       dwUserIndex,
    __out   PXINPUT_KEYBOARDSTATE       pKeyboardState
    );


#define XINPUT_LANGUAGE_ENGLISH         1
#define XINPUT_LANGUAGE_ENGLISH_UK      2
#define XINPUT_LANGUAGE_JAPANESE        3
#define XINPUT_LANGUAGE_GERMAN          4
#define XINPUT_LANGUAGE_FRENCH          5
#define XINPUT_LANGUAGE_FRENCH_CANADA   6
#define XINPUT_LANGUAGE_ITALIAN         7
#define XINPUT_LANGUAGE_SPANISH         8
#define XINPUT_LANGUAGE_CHINESE_BOPOMOFO    9
#define XINPUT_LANGUAGE_CHINESE_CHAJEI      10
#define XINPUT_LANGUAGE_KOREAN          11
#define XINPUT_LANGUAGE_SWEDISH         12
#define XINPUT_LANGUAGE_PORTUGUESE_PT   13
#define XINPUT_LANGUAGE_PORTUGUESE_BR   14
#define XINPUT_LANGUAGE_POLISH          15
#define XINPUT_LANGUAGE_RUSSIAN         16



XBOXAPI
DWORD
WINAPI
XInputGetKeyboardLanguage(
    __in    DWORD                       dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XInputGetKeystroke(
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwFlags,
    __out   PXINPUT_KEYSTROKE           pKeystroke
    );

XBOXAPI
DWORD
WINAPI
XInputGetKeystrokeEx(
    __inout PDWORD                      pdwUserIndex,
    __in    DWORD                       dwFlags,
    __out   PXINPUT_KEYSTROKE           pKeystroke
    );

#if _DEBUG
XBOXAPI
VOID
WINAPI
XInputEnableAutobind(
    __in    BOOL                        fEnable
    );
#else   // _DEBUG
#define XInputEnableAutobind(f)
#endif  // !_DEBUG


XBOXAPI
DWORD
WINAPI
XMountUtilityDrive(
    __in    BOOL                        fFormatClean,
    __in    DWORD                       dwBytesPerCluster,
    __in    SIZE_T                      dwFileCacheSize
    );

#define XMOUNTUTILITYDRIVE_FORMAT0      0x00000001
#define XMOUNTUTILITYDRIVE_FORMAT1      0x00000002
#define XMOUNTUTILITYDRIVE_FORMAT_ALL   0x0000000f


XBOXAPI
DWORD
WINAPI
XMountUtilityDriveEx(
    __in    DWORD                       dwFlags,
    __in    DWORD                       dwBytesPerCluster,
    __in    SIZE_T                      dwFileCacheSize
    );

XBOXAPI
DWORD
WINAPI
XFlushUtilityDrive(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XUnmountUtilityDrive(
    VOID
    );


#define MAX_LAUNCH_DATA_SIZE            1020
#define XLDEMO_RUNMODE_USERSELECTED     0x02

typedef struct _LD_DEMO  // Required for launching into demos
{
    DWORD dwID;
    DWORD dwRunmode;
    DWORD dwTimeout;
    CHAR  szLauncherXEX[128];
    CHAR  szLaunchedXEX[128];
} LD_DEMO, *PLD_DEMO;

// value of LD_DEMO.dwID
#define LAUNCH_DATA_DEMO_ID 'CDX1'




XBOXAPI
DWORD
WINAPI
XSetLaunchData(
    __in_bcount( dwLaunchDataSize ) PVOID pLaunchData,
    __in    DWORD                       dwLaunchDataSize
    );

XBOXAPI
DWORD
WINAPI
XGetLaunchDataSize(
    __out   LPDWORD                     pdwLaunchDataSize
    );

XBOXAPI
DWORD
WINAPI
XGetLaunchData(
    __out_bcount( dwBufferSize ) PVOID  pBuffer,
    __in    DWORD                       dwBufferSize
    );

DECLSPEC_NORETURN
XBOXAPI
VOID
WINAPI
XLaunchNewImage(
    __in_z  LPCSTR                      szImagePath,
    __in    DWORD                       dwFlags
    );


#define XLAUNCH_KEYWORD_DASH            NULL
#define XLAUNCH_KEYWORD_DEFAULT_APP     ""
#define XLAUNCH_KEYWORD_DASH_ARCADE     "XLNI_DASH_ARCADE"
#define XLAUNCH_KEYWORD_DASH_NUIHUB     "XLNI_DASH_NUIHUB"
#define XLAUNCH_FLAG_MATCH_MIN_VERSION  0x00010000
#define XLAUNCH_FLAG_CLEAR_LAUNCH_DATA  0x00020000

XBOXAPI
HMODULE
WINAPI
XLoadLibrary(
    __in_z  LPCSTR                      szLibFileName,
    __in    DWORD                       dwMinVersion
    );


typedef struct _XSWAPDISC_ERROR_TEXT {
    LPCWSTR wszTitle;
    LPCWSTR wszText;
    LPCWSTR wszButton;
} XSWAPDISC_ERROR_TEXT, *PXSWAPDISC_ERROR_TEXT;

XBOXAPI
DWORD
WINAPI
XSwapDisc(
    __in    UCHAR                       bDiscNum,
    __in    HANDLE                      hComplete,
    __in    CONST XSWAPDISC_ERROR_TEXT* pErrorStrings
    );

XBOXAPI
BOOL
WINAPI
XSwapCancel(
    VOID
    );


typedef struct _XOVERLAPPED             XOVERLAPPED, *PXOVERLAPPED;


XBOXAPI
BOOL
WINAPI
XGetModuleSection(
    __in    HANDLE                      ModuleHandle,
    __in_z  LPCSTR                      szSectionName,
    __deref_out_bcount( *pSectionSize ) PVOID *pSectionData,
    __out   ULONG                       *pSectionSize
    );


XBOXAPI
DWORD
WINAPI
XSetThreadProcessor(
    __in    HANDLE                      hThread,
    __in    DWORD                       dwHardwareThread
    );

typedef VOID (WINAPI *XTHREAD_NOTIFY_PROC)(BOOL fCreate);
typedef struct _XTHREAD_NOTIFICATION {
    LIST_ENTRY                          ListEntry;
    XTHREAD_NOTIFY_PROC                 pfnNotifyRoutine;
} XTHREAD_NOTIFICATION, *PXTHREAD_NOTIFICATION;

XBOXAPI
VOID
WINAPI
XRegisterThreadNotifyRoutine(
    __inout PXTHREAD_NOTIFICATION       pThreadNotification,
    __in    BOOL                        fRegister
    );

XBOXAPI
VOID
WINAPI
XSetProcessQuantumLength(
    __in    DWORD                       dwMilliseconds
    );

XBOXAPI
DWORD
WINAPI
XGetProcessQuantumLength(
    VOID
    );

XBOXAPI
BOOL
WINAPI
XSetFileCacheSize(
    __in    SIZE_T                      dwCacheSize
    );

XBOXAPI
SIZE_T
WINAPI
XGetFileCacheSize(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XGetFilePhysicalSortKey(
    __in    HANDLE                      hFile
    );


XBOXAPI
VOID
WINAPI
XSaveFloatingPointStateForDpc(
    VOID
    );

XBOXAPI
VOID
WINAPI
XRestoreFloatingPointStateForDpc(
    VOID
    );

#define XLOCKL2_INDEX_XPS                       0
#define XLOCKL2_INDEX_TITLE                     1

#define XLOCKL2_LOCK_SIZE_1_WAY                 (128*1024)
#define XLOCKL2_LOCK_SIZE_2_WAYS                (256*1024)

#define XLOCKL2_FLAG_SUSPEND_REPLACEMENT        0x00000001
#define XLOCKL2_FLAG_NONEXCLUSIVE_REPLACEMENT   0X00000002

XBOXAPI
BOOL
WINAPI
XLockL2(
    __in    DWORD                       dwIndex,
    __in_bcount( dwRangeSize ) CONST PVOID pRangeStart,
    __in    DWORD                       dwRangeSize,
    __in    DWORD                       dwLockSize,
    __in    DWORD                       dwFlags
    );

VOID
WINAPI
XUnlockL2(
    __in    DWORD                       dwIndex
    );

XBOXAPI
LPVOID
WINAPI
XPhysicalAlloc(
    __in     SIZE_T                     dwSize,
    __in_opt ULONG_PTR                  ulPhysicalAddress,
    __in_opt ULONG_PTR                  ulAlignment,
    __in     DWORD                      flProtect
    );

XBOXAPI
LPVOID
WINAPI
XPhysicalAllocEx(
    __in     SIZE_T                     dwSize,
    __in     ULONG_PTR                  ulLowestAcceptableAddress,
    __in     ULONG_PTR                  ulHighestAcceptableAddress,
    __in_opt ULONG_PTR                  ulAlignment,
    __in     DWORD                      flProtect
    );

XBOXAPI
SIZE_T
WINAPI
XPhysicalSize(
    __in    LPVOID                     lpAddress
    );

XBOXAPI
VOID
WINAPI
XPhysicalProtect(
    __in    LPVOID                      lpAddress,
    __in    SIZE_T                      dwSize,
    __in    DWORD                       flNewProtect
    );

XBOXAPI
VOID
WINAPI
XPhysicalFree(
    __in    LPVOID                      lpAddress
    );

XBOXAPI
DWORD
WINAPI
XQueryMemoryProtect(
    __in    LPVOID                      lpAddress
    );

XBOXAPI
VOID
WINAPI
XEnableSmallPagesOverride(
    __in    BOOL                        fEnable
    );

XBOXAPI
BOOL
WINAPI
XGetSmallPagesOverride(
    );

XBOXAPI
LPVOID
WINAPI
XEncryptedAlloc(
    __in    SIZE_T                      dwSize
    );

XBOXAPI
VOID
WINAPI
XEncryptedFree(
    __in    LPVOID                      lpAddress
    );



// allocation attribute definitions for XMemAlloc and XMemFree APIs

#define XALLOC_MEMTYPE_HEAP                         0
#define XALLOC_MEMTYPE_PHYSICAL                     1

#define XALLOC_MEMPROTECT_WRITECOMBINE_LARGE_PAGES  0
#define XALLOC_MEMPROTECT_NOCACHE                   1
#define XALLOC_MEMPROTECT_READWRITE                 2
#define XALLOC_MEMPROTECT_WRITECOMBINE              3

#define XALLOC_ALIGNMENT_DEFAULT                    0x0
#define XALLOC_ALIGNMENT_4                          0x1
#define XALLOC_ALIGNMENT_8                          0x2
#define XALLOC_ALIGNMENT_16                         0x4

#define XALLOC_PHYSICAL_ALIGNMENT_DEFAULT           0x0 // Default is 4K alignment
#define XALLOC_PHYSICAL_ALIGNMENT_4                 0x2
#define XALLOC_PHYSICAL_ALIGNMENT_8                 0x3
#define XALLOC_PHYSICAL_ALIGNMENT_16                0x4
#define XALLOC_PHYSICAL_ALIGNMENT_32                0x5
#define XALLOC_PHYSICAL_ALIGNMENT_64                0x6
#define XALLOC_PHYSICAL_ALIGNMENT_128               0x7
#define XALLOC_PHYSICAL_ALIGNMENT_256               0x8
#define XALLOC_PHYSICAL_ALIGNMENT_512               0x9
#define XALLOC_PHYSICAL_ALIGNMENT_1K                0xA
#define XALLOC_PHYSICAL_ALIGNMENT_2K                0xB
#define XALLOC_PHYSICAL_ALIGNMENT_4K                0xC
#define XALLOC_PHYSICAL_ALIGNMENT_8K                0xD
#define XALLOC_PHYSICAL_ALIGNMENT_16K               0xE
#define XALLOC_PHYSICAL_ALIGNMENT_32K               0xF

typedef enum _XALLOC_ALLOCATOR_IDS {
    eXALLOCAllocatorId_GameMin                  = 0,
    eXALLOCAllocatorId_GameMax                  = 127,
    eXALLOCAllocatorId_MsReservedMin            = 128,
    eXALLOCAllocatorId_D3D                      = 128,
    eXALLOCAllocatorId_D3DX,
    eXALLOCAllocatorId_XAUDIO,
    eXALLOCAllocatorId_XAPI,
    eXALLOCAllocatorId_XACT,
    eXALLOCAllocatorId_XBOXKERNEL,
    eXALLOCAllocatorId_XBDM,
    eXALLOCAllocatorId_XGRAPHICS,
    eXALLOCAllocatorId_XONLINE,
    eXALLOCAllocatorId_XVOICE,
    eXALLOCAllocatorId_XHV,
    eXALLOCAllocatorId_USB,
    eXALLOCAllocatorId_XMV,
    eXALLOCAllocatorId_SHADERCOMPILER,
    eXALLOCAllocatorId_XUI,
    eXALLOCAllocatorId_XASYNC,
    eXALLOCAllocatorId_XCAM,
    eXALLOCAllocatorId_XVIS,
    eXALLOCAllocatorId_XIME,
    eXALLOCAllocatorId_XFILECACHE,
    eXALLOCAllocatorId_XRN,
    eXALLOCAllocatorID_XMCORE,
    eXALLOCAllocatorID_XMASSIVE,
    eXALLOCAllocatorId_XAUDIO2,
    eXALLOCAllocatorId_XAVATAR,
    eXALLOCAllocatorId_XLSP,
    eXALLOCAllocatorId_D3DAlloc,
    eXALLOCAllocatorID_NUISPEECH,
    eXALLOCAllocatorId_NuiApi,
    eXALLOCAllocatorId_NuiIdentity,
    eXALLOCAllocatorId_XTweak,
    eXALLOCAllocatorId_XAMCOMMON,
    eXALLOCAllocatorId_NUIUI,
    eXALLOCAllocatorId_LUA,
    eXALLOCAllocatorId_DRM,
    eXALLOCAllocatorId_XAVPipeline,
    eXALLOCAllocatorId_XAVCodecs,
    eXALLOCAllocatorId_XAUTH,
    eXALLOCAllocatorId_WINUSB,
	eXALLOCAllocatorId_BLUETOOTH,
    eXALLOCAllocatorId_NuiApi_LargePageReadWrite = 190,
    eXALLOCAllocatorId_MsReservedMax            = 191,
    eXALLOCAllocatorId_MiddlewareReservedMin    = 192,
    eXALLOCAllocatorId_MiddlewareReservedMax    = 255
} XALLOC_ALLOCATOR_IDS;


// The PPC compiler defines bitfields to be ordered from the MSB to the LSB,
// which is opposite the convention on the X86 platform.  Use the 'bitfield_order'
// pragma to switch the ordering.  This does not affect endianness in any way.

#if defined(_M_PPCBE)
#pragma bitfield_order(push)
#pragma bitfield_order(lsb_to_msb)
#endif

typedef struct _XALLOC_ATTRIBUTES {
    DWORD                               dwObjectType:13;
    DWORD                               dwHeapTracksAttributes:1;
    DWORD                               dwMustSucceed:1;
    DWORD                               dwFixedSize:1;
    DWORD                               dwAllocatorId:8;
    DWORD                               dwAlignment:4;
    DWORD                               dwMemoryProtect:2;
    DWORD                               dwZeroInitialize:1;
    DWORD                               dwMemoryType:1;
} XALLOC_ATTRIBUTES, *PXALLOC_ATTRIBUTES;

#if defined(_M_PPCBE)
#pragma bitfield_order(pop)
#endif

#define MAKE_XALLOC_ATTRIBUTES(ObjectType,\
                               HeapTracksAttributes,\
                               MustSucceed,\
                               FixedSize,\
                               AllocatorId,\
                               Alignment,\
                               MemoryProtect,\
                               ZeroInitialize,\
                               MemoryType)\
    ((DWORD)( ObjectType | \
             (HeapTracksAttributes << 13) | \
             (MustSucceed << 14) | \
             (FixedSize << 15) | \
             (AllocatorId << 16) | \
             (Alignment << 24) | \
             (MemoryProtect << 28) | \
             (ZeroInitialize << 30) | \
             (MemoryType << 31)))

#define XALLOC_IS_PHYSICAL(Attributes)  ((BOOL)(Attributes & 0x80000000)!=0)

XBOXAPI
LPVOID
WINAPI
XMemAlloc(
    __in    SIZE_T                      dwSize,
    __in    DWORD                       dwAllocAttributes
    );

XBOXAPI
LPVOID
WINAPI
XMemAllocDefault(
    __in    SIZE_T                      dwSize,
    __in    DWORD                       dwAllocAttributes
    );

XBOXAPI
VOID
WINAPI
XMemFree(
    __in_opt PVOID                      pAddress,
    __in     DWORD                      dwAllocAttributes
    );

XBOXAPI
VOID
WINAPI
XMemFreeDefault(
    __in_opt PVOID                      pAddress,
    __in     DWORD                      dwAllocAttributes
    );

XBOXAPI
SIZE_T
WINAPI
XMemSize(
    __in    PVOID                       pAddress,
    __in    DWORD                       dwAllocAttributes
    );

XBOXAPI
SIZE_T
WINAPI
XMemSizeDefault(
    __in    PVOID                       pAddress,
    __in    DWORD                       dwAllocAttributes
    );

XBOXAPI
VOID
WINAPI
XSetAttributesOnHeapAlloc(
    __in    PVOID                       pBaseAddress,
    __in    DWORD                       dwAllocAttributes
    );

XBOXAPI
DWORD
WINAPI
XGetAttributesOnHeapAlloc(
    __in    PVOID                       pBaseAddress
    );

XBOXAPI
PVOID
WINAPI
XMemSet128(
    __out_bcount( count ) PVOID         pDest,
    __in    INT                         c,
    __in    SIZE_T                      count
    );

XBOXAPI
PVOID
WINAPI
XMemSet(
    __out_bcount( count ) PVOID         pDest,
    __in    INT                         c,
    __in    SIZE_T                      count
    );

XBOXAPI
PVOID
WINAPI
XMemCpy128(
    __out_bcount( count ) PVOID         pDest,
    __in_bcount( count ) CONST VOID*    pSrc,
    __in SIZE_T                         count
    );

XBOXAPI
PVOID
WINAPI
XMemCpy(
    __out_bcount( count ) PVOID         pDest,
    __in_bcount( count ) CONST VOID*    pSrc,
    __in SIZE_T                         count
    );

XBOXAPI
PVOID
WINAPI
XMemCpyStreaming(
    __out_bcount( count ) PVOID         pDest,
    __in_bcount( count ) CONST VOID*    pSrc,
    __in SIZE_T                         count
    );

XBOXAPI
PVOID
WINAPI
XMemCpyStreaming_Cached(
    __out_bcount( count ) PVOID         pDest,
    __in_bcount( count ) CONST VOID*    pSrc,
    __in SIZE_T                         count
    );

XBOXAPI
PVOID
WINAPI
XMemCpyStreaming_WriteCombined(
    __out_bcount( count ) PVOID         pDest,
    __in_bcount( count ) CONST VOID*    pSrc,
    __in SIZE_T                         count
    );

XBOXAPI
DWORD
WINAPI
XMemGetPageSize(
    __in const VOID* address
    );

// XMemCompress/XMemDecompress:

#include <xcompress.h>

#define XVER_DEVKIT                     1
#define XVER_RETAIL                     2

#ifdef _DEBUG

XBOXAPI
DWORD
WINAPI
XDebugGetSystemVersionA(
    __out_ecount( cchVersionString ) LPSTR szVersionString,
    __in    UINT                        cchVersionString
    );
#define XDebugGetSystemVersion          XDebugGetSystemVersionA

XBOXAPI
DWORD
WINAPI
XDebugGetXTLVersionA(
    __out_ecount( cchVersionString ) LPSTR szVersionString,
    __in    UINT                        cchVersionString
    );
#define XDebugGetXTLVersion             XDebugGetXTLVersionA

#endif  // _DEBUG


// Xbox-specific Overlapped

typedef
VOID
(WINAPI *PXOVERLAPPED_COMPLETION_ROUTINE)(
    __in    DWORD                       dwErrorCode,
    __in    DWORD                       dwNumberOfBytesTransfered,
    __inout PXOVERLAPPED                pOverlapped
    );


typedef struct _XOVERLAPPED {
    ULONG_PTR                           InternalLow;
    ULONG_PTR                           InternalHigh;
    ULONG_PTR                           InternalContext;
    HANDLE                              hEvent;
    PXOVERLAPPED_COMPLETION_ROUTINE     pCompletionRoutine;
    DWORD_PTR                           dwCompletionContext;
    DWORD                               dwExtendedError;
} XOVERLAPPED, *PXOVERLAPPED;

#define XHasOverlappedIoCompleted(lpOverlapped) \
    (*((volatile ULONG_PTR*)(&(lpOverlapped)->InternalLow)) != ERROR_IO_PENDING)

XBOXAPI
DWORD
WINAPI
XGetOverlappedResult(
    __in        PXOVERLAPPED    pOverlapped,
    __out_opt   LPDWORD         pdwResult,
    __in        BOOL            bWait
    );

XBOXAPI
DWORD
WINAPI
XGetOverlappedExtendedError(
    __in_opt    PXOVERLAPPED    pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XCancelOverlapped(
    __in        PXOVERLAPPED     pOverlapped
    );

// XContent

// Content types
#define XCONTENTTYPE_SAVEDGAME                      0x00000001
#define XCONTENTTYPE_MARKETPLACE                    0x00000002
#define XCONTENTTYPE_PUBLISHER                      0x00000003

#define XCONTENTTYPE_GAMEDEMO                       0x00080000
#define XCONTENTTYPE_ARCADE                         0x000D0000

//  Content creation/open flags
#define XCONTENTFLAG_NONE                           0x00000000
#define XCONTENTFLAG_CREATENEW                      CREATE_NEW
#define XCONTENTFLAG_CREATEALWAYS                   CREATE_ALWAYS
#define XCONTENTFLAG_OPENEXISTING                   OPEN_EXISTING
#define XCONTENTFLAG_OPENALWAYS                     OPEN_ALWAYS
#define XCONTENTFLAG_TRUNCATEEXISTING               TRUNCATE_EXISTING

//  Content attributes
#define XCONTENTFLAG_NOPROFILE_TRANSFER             0x00000010
#define XCONTENTFLAG_NODEVICE_TRANSFER              0x00000020
#define XCONTENTFLAG_STRONG_SIGNED                  0x00000040
#define XCONTENTFLAG_ALLOWPROFILE_TRANSFER          0x00000080
#define XCONTENTFLAG_MOVEONLY_TRANSFER              0x00000800

//  Device selector flags
#define XCONTENTFLAG_MANAGESTORAGE                  0x00000100
#define XCONTENTFLAG_FORCE_SHOW_UI                  0x00000200

//  Enumeration scoping
#define XCONTENTFLAG_ENUM_EXCLUDECOMMON             0x00001000


#define XCONTENT_MAX_DISPLAYNAME_LENGTH 128
#define XCONTENT_MAX_FILENAME_LENGTH    42
#define XCONTENTDEVICE_MAX_NAME_LENGTH  27

typedef DWORD                           XCONTENTDEVICEID, *PXCONTENTDEVICEID;

#define XCONTENTDEVICE_ANY              ((XCONTENTDEVICEID)0)

typedef struct _XCONTENT_DATA
{
    XCONTENTDEVICEID                    DeviceID;
    DWORD                               dwContentType;
    WCHAR                               szDisplayName[XCONTENT_MAX_DISPLAYNAME_LENGTH];
    CHAR                                szFileName[XCONTENT_MAX_FILENAME_LENGTH];
} XCONTENT_DATA, *PXCONTENT_DATA;

typedef struct _XCONTENT_CROSS_TITLE_DATA
{
    XCONTENTDEVICEID                    DeviceID;
    DWORD                               dwContentType;
    WCHAR                               szDisplayName[XCONTENT_MAX_DISPLAYNAME_LENGTH];
    CHAR                                szFileName[XCONTENT_MAX_FILENAME_LENGTH];
    DWORD                               dwTitleId;
} XCONTENT_CROSS_TITLE_DATA, *PXCONTENT_CROSS_TITLE_DATA;


typedef DWORD                           XCONTENTDEVICETYPE, *PXCONTENTDEVICETYPE;

//
// Defines the set of XContent storage Device Types.
//

#define XCONTENTDEVICETYPE_HDD              ((XCONTENTDEVICETYPE)1)
#define XCONTENTDEVICETYPE_MU               ((XCONTENTDEVICETYPE)2)
#define XCONTENTDEVICETYPE_ODD              ((XCONTENTDEVICETYPE)4)

typedef struct _XDEVICE_DATA
{
    XCONTENTDEVICEID                    DeviceID;
    XCONTENTDEVICETYPE                  DeviceType;
    ULONGLONG                           ulDeviceBytes;
    ULONGLONG                           ulDeviceFreeBytes;
    WCHAR                               wszFriendlyName[XCONTENTDEVICE_MAX_NAME_LENGTH];
} XDEVICE_DATA, *PXDEVICE_DATA;


#define XCONTENT_NONE                   0x00000000
#define XCONTENT_CREATED_NEW            0x00000001
#define XCONTENT_OPENED_EXISTING        0x00000002

XBOXAPI
DWORD
WINAPI
XContentCreate(
    __in    DWORD                       dwUserIndex,
    __in_z  LPCSTR                      szRootName,
    __in    CONST XCONTENT_DATA*        pContentData,
    __in    DWORD                       dwContentFlags,
    __out_opt PDWORD                    pdwDisposition,
    __out_opt PDWORD                    pdwLicenseMask,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentCreateEx(
    __in    DWORD                       dwUserIndex,
    __in_z  LPCSTR                      szRootName,
    __in    CONST XCONTENT_DATA*        pContentData,
    __in    DWORD                       dwContentFlags,
    __out_opt PDWORD                    pdwDisposition,
    __out_opt PDWORD                    pdwLicenseMask,
    __in    SIZE_T                      dwFileCacheSize,
    __in    ULARGE_INTEGER              uliContentSize,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentDelete(
    __in    DWORD                       dwUserIndex,
    __in    CONST XCONTENT_DATA*        pContentData,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentInstall(
    __in    DWORD                       dwUserIndex,
    __in    XCONTENTDEVICEID            DeviceID,
    __in_z  LPCSTR                      pszFileName,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentClose(
    __in_z  LPCSTR                      szRootName,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentFlush(
    __in_z  LPCSTR                      szRootName,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentSetThumbnail(
    __in    DWORD                       dwUserIndex,
    __in    CONST XCONTENT_DATA*        pContentData,
    __in_bcount( cbThumbnail ) CONST BYTE* pbThumbnail,
    __in    DWORD                       cbThumbnail,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentGetThumbnail(
    __in    DWORD                       dwUserIndex,
    __in    CONST XCONTENT_DATA*        pContentData,
    __out_bcount_opt( *pcbThumbnail ) PBYTE pbThumbnail,
    __inout PDWORD                      pcbThumbnail,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentGetCreator(
    __in    DWORD                       dwUserIndex,
    __in    CONST XCONTENT_DATA*        pContentData,
    __out   PBOOL                       pfUserIsCreator,
    __out_opt PXUID                     pxuid,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentGetLicenseMask(
    __out   PDWORD                      pdwLicenseMask,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentLaunchImage(
    __in    DWORD                       dwUserIndex,
    __in    CONST XCONTENT_DATA*        pContentData,
    __in_z  LPCSTR                      szImagePath
    );

XBOXAPI
ULONGLONG
WINAPI
XContentCalculateSize(
    __in    ULONGLONG                   cbData,
    __in    DWORD                       cDirectories
    );

XBOXAPI
DWORD
WINAPI
XContentGetDeviceState(
    __in    XCONTENTDEVICEID            DeviceID,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentGetDeviceStateByHandle(
    __in    CONST XCONTENT_DATA*        pContentData,
    __inout_opt PXOVERLAPPED            pOverlapped
    );



XBOXAPI
DWORD
WINAPI
XContentQueryVolumeDeviceType(
    __in_z  LPCSTR                      pszRootName,
    __out   XCONTENTDEVICETYPE*         pDeviceType,
    __inout_opt XOVERLAPPED*            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentOpenPackage(
    __in    DWORD                       dwUserIndex,
    __in    CONST XCONTENT_DATA*        pContentData,
    __out   HANDLE *                    phPackageFile
    );

XBOXAPI
DWORD
WINAPI
XContentCreatePackage(
    __in    DWORD                       dwUserIndex,
    __in    CONST XCONTENT_DATA*        pContentData,
    __in    BOOL                        fFailIfExists,
    __out   HANDLE*                     phPackageFile
    );

XBOXAPI
DWORD
WINAPI
XContentCrossTitleCreate(
    __in    DWORD                               dwUserIndex,
    __in_z  LPCSTR                              szRootName,
    __in    CONST XCONTENT_CROSS_TITLE_DATA*    pContentData,
    __in    DWORD                               dwContentFlags,
    __out_opt PDWORD                            pdwDisposition,
    __out_opt PDWORD                            pdwLicenseMask,
    __in    DWORD                               dwFileCacheSize,
    __in    ULARGE_INTEGER                      uliContentSize,
    __inout_opt PXOVERLAPPED                    pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentCrossTitleDelete(
    __in    DWORD                               dwUserIndex,
    __in    CONST XCONTENT_CROSS_TITLE_DATA*    pContentData,
    __inout_opt PXOVERLAPPED                    pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentCreateCrossTitleEnumerator(
    __in    DWORD                               dwUserIndex,
    __in    XCONTENTDEVICEID                    DeviceID,
    __in    DWORD                               dwContentType,
    __in    DWORD                               dwContentFlags,
    __in    DWORD                               cItem,
    __out_opt PDWORD                            pcbBuffer,
    __out   HANDLE*                             phEnum
    );

XBOXAPI
DWORD
WINAPI
XEnumerateCrossTitle(
    __in                        HANDLE              hEnum,
    __inout_bcount( cbBuffer )  PVOID               pvBuffer,
    __in                        DWORD               cbBuffer,
    __out_opt                   PDWORD              pcItemsReturned,
    __inout_opt                 PXOVERLAPPED        pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XContentLaunchImageFromFile(
    __in    LPCSTR                      pszFilePath,
    __in    LPCSTR                      pszImagePath
    );

XBOXAPI
DWORD
WINAPI
XContentCrossTitleLaunchImage(
    __in    DWORD                                   dwUserIndex,
    __in    CONST XCONTENT_CROSS_TITLE_DATA*        pContentData,
    __in_z  LPCSTR                                  szImagePath
    );


XBOXAPI
DWORD
WINAPI
XContentGetDeviceData(
    __in    XCONTENTDEVICEID            DeviceID,
    __out   PXDEVICE_DATA               pDeviceData
    );

XBOXAPI
DWORD
WINAPI
XContentGetDeviceName(
    __in    XCONTENTDEVICEID            DeviceID,
    __out_ecount( cchName ) LPWSTR      wszName,
    __in   DWORD                        cchName
    );


XBOXAPI
DWORD
WINAPI
XContentCreateEnumerator(
    __in    DWORD                       dwUserIndex,
    __in    XCONTENTDEVICEID            DeviceID,
    __in    DWORD                       dwContentType,
    __in    DWORD                       dwContentFlags,
    __in    DWORD                       cItem,
    __out_opt PDWORD                    pcbBuffer,
    __out   PHANDLE                     phEnum
    );

typedef struct _XOFFERING_CONTENTAVAILABLE_RESULT
{
    DWORD                               dwNewOffers;
    DWORD                               dwTotalOffers;
} XOFFERING_CONTENTAVAILABLE_RESULT;

XBOXAPI
DWORD
WINAPI
XContentGetMarketplaceCounts(
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwContentCategories,
    __in    DWORD                       cbResults,
    __inout XOFFERING_CONTENTAVAILABLE_RESULT *pResults,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

// Enumeration flags
#define ENUMERATEFLAG_NONE                      0x00000000
#define ENUMERATEFLAG_INCLUDEOPTICALMEDIA       0x00000020

XBOXAPI
DWORD
WINAPI
XEnumerate(
    __in                        HANDLE          hEnum,
    __inout_bcount( cbBuffer )  PVOID           pvBuffer,
    __in                        DWORD           cbBuffer,
    __out_opt                   PDWORD          pcItemsReturned,
    __inout_opt                 PXOVERLAPPED    pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XEnumerateBack(
    __in                        HANDLE          hEnum,
    __inout_bcount( cbBuffer )  PVOID           pvBuffer,
    __in                        DWORD           cbBuffer,
    __out_opt                   PDWORD          pcItemsReturned,
    __inout_opt                 PXOVERLAPPED    pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XEnumerateEx(
    __in                        HANDLE          hEnum,
    __in                        DWORD           dwFlags,
    __inout_bcount( cbBuffer )  PVOID           pvBuffer,
    __in                        DWORD           cbBuffer,
    __out_opt                   PDWORD          pcItemsReturned,
    __inout_opt                 PXOVERLAPPED    pOverlapped
    );


//-----------------------------------------------------------------------------
// Game instrumentation errors              = 0x8056F0XX
//-----------------------------------------------------------------------------

#define SPA_E_CORRUPT_FILE          _HRESULT_TYPEDEF_(0x8056F001)
#define SPA_E_NOT_LOADED            _HRESULT_TYPEDEF_(0x8056F002)
#define SPA_E_BAD_TITLEID           _HRESULT_TYPEDEF_(0x8056F003)


// User/Profile/Account APIs

typedef enum _XUSER_SIGNIN_STATE
{
    eXUserSigninState_NotSignedIn,
    eXUserSigninState_SignedInLocally,
    eXUserSigninState_SignedInToLive
} XUSER_SIGNIN_STATE;

XBOXAPI
DWORD
WINAPI
XUserGetXUID(
    __in    DWORD                       dwUserIndex,
    __out   PXUID                       pxuid
    );

XBOXAPI
DWORD
WINAPI
XUserGetName(
    __in                        DWORD   dwUserIndex,
    __out_ecount( cchUserName ) LPSTR   szUserName,
    __in                        DWORD   cchUserName
    );

XBOXAPI
XUSER_SIGNIN_STATE
WINAPI
XUserGetSigninState(
    __in    DWORD                       dwUserIndex
    );

XBOXAPI
FLOAT
WINAPI
XUserGetReputationStars(
    __in    FLOAT                       fGamerRating
    );

#define XUSER_INFO_FLAG_LIVE_ENABLED    0x00000001
#define XUSER_INFO_FLAG_GUEST           0x00000002

typedef struct _XUSER_SIGNIN_INFO
{
   XUID                 xuid;
   DWORD                dwInfoFlags;
   XUSER_SIGNIN_STATE   UserSigninState;
   DWORD                dwGuestNumber;
   DWORD                dwSponsorUserIndex;
   CHAR                 szUserName[XUSER_NAME_SIZE];
} XUSER_SIGNIN_INFO, * PXUSER_SIGNIN_INFO;

#define XUSER_GET_SIGNIN_INFO_ONLINE_XUID_ONLY      0x00000001
#define XUSER_GET_SIGNIN_INFO_OFFLINE_XUID_ONLY     0x00000002

XBOXAPI
DWORD
WINAPI
XUserGetSigninInfo(
    __in        DWORD                   dwUserIndex,
    __in_opt    DWORD                   dwFlags,
    __out       PXUSER_SIGNIN_INFO      pSigninInfo
    );


typedef enum _XPRIVILEGE_TYPE
{
    XPRIVILEGE_MULTIPLAYER_SESSIONS              = 254, // on|off

    XPRIVILEGE_COMMUNICATIONS                    = 252, // on (communicate w/everyone) | off (check _FO)
    XPRIVILEGE_COMMUNICATIONS_FRIENDS_ONLY       = 251, // on (communicate w/friends only) | off (blocked)

    XPRIVILEGE_PROFILE_VIEWING                   = 249, // on (viewing allowed) | off (check _FO)
    XPRIVILEGE_PROFILE_VIEWING_FRIENDS_ONLY      = 248, // on (view friends only) | off (no details)

    XPRIVILEGE_USER_CREATED_CONTENT              = 247, // on (allow viewing of UCC) | off (check _FO)
    XPRIVILEGE_USER_CREATED_CONTENT_FRIENDS_ONLY = 246, // on (view UCC from friends only) | off (blocked)

    XPRIVILEGE_PURCHASE_CONTENT                  = 245, // on (allow purchase) | off (blocked)

    XPRIVILEGE_PRESENCE                          = 244, // on (share presence info) | off (check _FO)
    XPRIVILEGE_PRESENCE_FRIENDS_ONLY             = 243, // on (share w/friends only | off (dont share)

    XPRIVILEGE_TRADE_CONTENT                     = 238, // on (allow trading) | off (blocked)

    XPRIVILEGE_VIDEO_COMMUNICATIONS              = 235, // on (communicate w/everyone) | off (check _FO)
    XPRIVILEGE_VIDEO_COMMUNICATIONS_FRIENDS_ONLY = 234, // on (communicate w/friends only) | off (blocked)

    XPRIVILEGE_SOCIAL_NETWORK_SHARING            = 220, // on (social network sharing allowed) | off (blocked)

    XPRIVILEGE_CONTENT_AUTHOR                    = 222, // on (allowed to publish content) | off (blocked)

    XPRIVILEGE_UNSAFE_PROGRAMMING                = 212, // on (regular programming) | off (family programming)

    XPRIVILEGE_SHARE_CONTENT_OUTSIDE_LIVE        = 211, // on (allow share kinect content) | off (blocked)

    XPRIVILEGE_INTERNET_BROWSING                 = 217, // on (allow web browsing using Internet Explorer) | off (blocked)
} XPRIVILEGE_TYPE;


XBOXAPI
DWORD
WINAPI
XUserAreUsersFriends(
    __in        DWORD                   dwUserIndex,
    __in        PXUID                   pXuids,
    __in        DWORD                   dwXuidCount,
    __out_opt   PBOOL                   pfResult,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XUserCheckPrivilege(
    __in    DWORD                       dwUserIndex,
    __in    XPRIVILEGE_TYPE             PrivilegeType,
    __out   PBOOL                       pfResult
    );

// Context and Property definitions

typedef struct _XUSER_DATA
{
    BYTE                                type;

    union
    {
        LONG                            nData;     // XUSER_DATA_TYPE_INT32
        LONGLONG                        i64Data;   // XUSER_DATA_TYPE_INT64
        double                          dblData;   // XUSER_DATA_TYPE_DOUBLE
        struct                                     // XUSER_DATA_TYPE_UNICODE
        {
            DWORD                       cbData;    // Includes null-terminator
            LPWSTR                      pwszData;
        } string;
        FLOAT                           fData;     // XUSER_DATA_TYPE_FLOAT
        struct                                     // XUSER_DATA_TYPE_BINARY
        {
            DWORD                       cbData;
            PBYTE                       pbData;
        } binary;
        FILETIME                        ftData;    // XUSER_DATA_TYPE_DATETIME
    };
} XUSER_DATA, *PXUSER_DATA;

typedef struct _XUSER_PROPERTY
{
    DWORD                               dwPropertyId;
    XUSER_DATA                          value;
} XUSER_PROPERTY, *PXUSER_PROPERTY;

typedef struct _XUSER_CONTEXT
{
    DWORD                               dwContextId;
    DWORD                               dwValue;
} XUSER_CONTEXT, *PXUSER_CONTEXT;

// Context and Property APIs

XBOXAPI
DWORD
WINAPI
XUserGetContext(
    __in        DWORD                   dwUserIndex,
    __inout     XUSER_CONTEXT*          pContext,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
VOID
WINAPI
XUserSetContext(
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwContextId,
    __in    DWORD                       dwContextValue
    );

XBOXAPI
DWORD
WINAPI
XUserSetContextEx(
    __in        DWORD                   dwUserIndex,
    __in        DWORD                   dwContextId,
    __in        DWORD                   dwContextValue,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XUserGetProperty(
    __in        DWORD                           dwUserIndex,
    __inout     DWORD*                          pcbActual,
    __inout_bcount( pcbActual ) XUSER_PROPERTY* pProperty,
    __inout_opt PXOVERLAPPED                    pOverlapped
    );

XBOXAPI
VOID
WINAPI
XUserSetProperty(
    __in                    DWORD               dwUserIndex,
    __in                    DWORD               dwPropertyId,
    __in                    DWORD               cbValue,
    __in_bcount( cbValue )  CONST VOID*         pvValue
    );

XBOXAPI
DWORD
WINAPI
XUserSetPropertyEx(
    __in                    DWORD               dwUserIndex,
    __in                    DWORD               dwPropertyId,
    __in                    DWORD               cbValue,
    __in_bcount( cbValue )  CONST VOID*         pvValue,
    __inout_opt             PXOVERLAPPED        pOverlapped
    );

// System-defined contexts and properties

#define X_PROPERTY_TYPE_MASK            0xF0000000
#define X_PROPERTY_SCOPE_MASK           0x00008000
#define X_PROPERTY_ID_MASK              0x00007FFF


#define XPROPERTYID(global, type, id)   ((global ? X_PROPERTY_SCOPE_MASK : 0) | ((type << 28) & X_PROPERTY_TYPE_MASK) | (id & X_PROPERTY_ID_MASK))
#define XCONTEXTID(global, id)          XPROPERTYID(global, XUSER_DATA_TYPE_CONTEXT, id)
#define XPROPERTYTYPEFROMID(id)         ((id >> 28) & 0xf)
#define XISSYSTEMPROPERTY(id)           (id & X_PROPERTY_SCOPE_MASK)

// Predefined contexts
#define X_CONTEXT_PRESENCE              XCONTEXTID(1, 0x1)
#define X_CONTEXT_GAME_TYPE             XCONTEXTID(1, 0xA)
#define X_CONTEXT_GAME_MODE             XCONTEXTID(1, 0xB)

#define X_PROPERTY_RANK                 XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0x1)
#define X_PROPERTY_GAMERNAME            XPROPERTYID(1, XUSER_DATA_TYPE_UNICODE, 0x2)
#define X_PROPERTY_SESSION_ID           XPROPERTYID(1, XUSER_DATA_TYPE_INT64,   0x3)

// System attributes used in matchmaking queries
#define X_PROPERTY_GAMER_ZONE           XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0x101)
#define X_PROPERTY_GAMER_COUNTRY        XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0x102)
#define X_PROPERTY_GAMER_LANGUAGE       XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0x103)
#define X_PROPERTY_GAMER_RATING         XPROPERTYID(1, XUSER_DATA_TYPE_FLOAT,   0x104)
#define X_PROPERTY_GAMER_MU             XPROPERTYID(1, XUSER_DATA_TYPE_DOUBLE,  0x105)
#define X_PROPERTY_GAMER_SIGMA          XPROPERTYID(1, XUSER_DATA_TYPE_DOUBLE,  0x106)
#define X_PROPERTY_GAMER_PUID           XPROPERTYID(1, XUSER_DATA_TYPE_INT64,   0x107)
#define X_PROPERTY_AFFILIATE_SCORE      XPROPERTYID(1, XUSER_DATA_TYPE_INT64,   0x108)
#define X_PROPERTY_GAMER_HOSTNAME       XPROPERTYID(1, XUSER_DATA_TYPE_UNICODE, 0x109)
#define X_PROPERTY_SESSION_SCORE        XPROPERTYID(1, XUSER_DATA_TYPE_FLOAT,   0x10A)

// Properties used to write to skill leaderboards
#define X_PROPERTY_RELATIVE_SCORE                   XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0xA)
#define X_PROPERTY_SESSION_TEAM                     XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0xB)

// Properties written at the session level to override TrueSkill parameters
#define X_PROPERTY_PLAYER_PARTIAL_PLAY_PERCENTAGE   XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0xC)
#define X_PROPERTY_PLAYER_SKILL_UPDATE_WEIGHTING_FACTOR XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0xD)
#define X_PROPERTY_SESSION_SKILL_BETA               XPROPERTYID(1, XUSER_DATA_TYPE_DOUBLE,  0xE)
#define X_PROPERTY_SESSION_SKILL_TAU                XPROPERTYID(1, XUSER_DATA_TYPE_DOUBLE,  0xF)
#define X_PROPERTY_SESSION_SKILL_DRAW_PROBABILITY   XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0x10)

// Attachment size is written to a leaderboard when the entry qualifies for
// a gamerclip.  The rating can be retrieved via XUserEstimateRankForRating.
#define X_PROPERTY_ATTACHMENT_SIZE                  XPROPERTYID(1, XUSER_DATA_TYPE_INT32,   0x11)


// Values for X_CONTEXT_GAME_TYPE
#define X_CONTEXT_GAME_TYPE_RANKED      0
#define X_CONTEXT_GAME_TYPE_STANDARD    1

// Achievement APIs

typedef struct
{
    DWORD                               dwUserIndex;
    DWORD                               dwAchievementId;
} XUSER_ACHIEVEMENT, *PXUSER_ACHIEVEMENT;

XBOXAPI
DWORD
WINAPI
XUserWriteAchievements(
    __in                            DWORD                       dwNumAchievements,
    __in_ecount( dwNumAchievements ) CONST XUSER_ACHIEVEMENT*   pAchievements,
    __inout_opt                     PXOVERLAPPED                pOverlapped
    );

typedef struct
{
    DWORD                               dwId;
    LPWSTR                              pwszLabel;
    LPWSTR                              pwszDescription;
    LPWSTR                              pwszUnachieved;
    DWORD                               dwImageId;
    DWORD                               dwCred;
    FILETIME                            ftAchieved;
    DWORD                               dwFlags;
} XACHIEVEMENT_DETAILS, *PXACHIEVEMENT_DETAILS;

// These lengths include the NULL-terminator
#define XACHIEVEMENT_MAX_LABEL_LENGTH   32
#define XACHIEVEMENT_MAX_DESC_LENGTH    100
#define XACHIEVEMENT_MAX_UNACH_LENGTH   100

#define XACHIEVEMENT_SIZE_BASE          (sizeof(XACHIEVEMENT_DETAILS))
#define XACHIEVEMENT_SIZE_STRINGS       (sizeof(WCHAR) * (XACHIEVEMENT_MAX_LABEL_LENGTH  + XACHIEVEMENT_MAX_DESC_LENGTH + XACHIEVEMENT_MAX_UNACH_LENGTH))
#define XACHIEVEMENT_SIZE_FULL          (XACHIEVEMENT_SIZE_BASE + XACHIEVEMENT_SIZE_STRINGS)

#define XACHIEVEMENT_INVALID_ID         ((DWORD)0xFFFFFFFF)

// XACHIEVEMENT_DETAILS::dwFlags can be manipulated with these defines and macros
#define XACHIEVEMENT_DETAILS_MASK_TYPE          0x00000007
#define XACHIEVEMENT_DETAILS_SHOWUNACHIEVED     0x00000008
#define XACHIEVEMENT_DETAILS_ACHIEVED_ONLINE    0x00010000
#define XACHIEVEMENT_DETAILS_ACHIEVED           0x00020000

#define AchievementType(dwFlags)           (dwFlags & XACHIEVEMENT_DETAILS_MASK_TYPE)
#define AchievementShowUnachieved(dwFlags) (dwFlags & XACHIEVEMENT_DETAILS_SHOWUNACHIEVED ? TRUE : FALSE)
#define AchievementEarnedOnline(dwFlags)   (dwFlags & XACHIEVEMENT_DETAILS_ACHIEVED_ONLINE ? TRUE : FALSE)
#define AchievementEarned(dwFlags)         (dwFlags & XACHIEVEMENT_DETAILS_ACHIEVED ? TRUE : FALSE)
#define AchievementRevoked(dwFlags)        (dwFlags & XACHIEVEMENT_DETAILS_NOT_ACHIEVABLE ? TRUE : FALSE)

// Types returned from AchievementType macro

#define XACHIEVEMENT_TYPE_COMPLETION            1
#define XACHIEVEMENT_TYPE_LEVELING              2
#define XACHIEVEMENT_TYPE_UNLOCK                3
#define XACHIEVEMENT_TYPE_EVENT                 4
#define XACHIEVEMENT_TYPE_TOURNAMENT            5
#define XACHIEVEMENT_TYPE_CHECKPOINT            6
#define XACHIEVEMENT_TYPE_OTHER                 7

#define XACHIEVEMENT_DETAILS_ALL                0xFFFFFFFF
#define XACHIEVEMENT_DETAILS_LABEL              0x00000001
#define XACHIEVEMENT_DETAILS_DESCRIPTION        0x00000002
#define XACHIEVEMENT_DETAILS_UNACHIEVED         0x00000004
#define XACHIEVEMENT_DETAILS_TFC                0x00000020

XBOXAPI
DWORD
WINAPI
XUserCreateAchievementEnumerator(
    __in    DWORD                       dwTitleId,
    __in    DWORD                       dwUserIndex,
    __in    XUID                        xuid,
    __in    DWORD                       dwDetailFlags,
    __in    DWORD                       dwStartingIndex,
    __in    DWORD                       cItem,
    __out   PDWORD                      pcbBuffer,
    __out   PHANDLE                     ph
    );

// Age groups defined by XOnline for use with XOnlineUserGetAgeGroup

typedef enum _XUSER_AGE_GROUP
{
    XAGEGROUP_CHILD     = 0,
    XAGEGROUP_TEEN      = 1,
    XAGEGROUP_ADULT     = 2
} XUSER_AGE_GROUP;

XBOXAPI
DWORD
WINAPI
XUserGetAge(
    IN      DWORD                       dwUserIndex,
    OUT     PBYTE                       pbAge,
    IN OUT  PXOVERLAPPED                pOverlapped         OPTIONAL
    );

XBOXAPI
DWORD
WINAPI
XUserGetAgeGroup(
    IN      DWORD                       dwUserIndex,
    OUT     XUSER_AGE_GROUP*            pAgeGroup,
    IN OUT  PXOVERLAPPED                pOverlapped         OPTIONAL
    );


XBOXAPI
DWORD
WINAPI
XUserContentRestrictionGetFlags(
    IN      DWORD               dwUserIndex,
    OUT     PDWORD              pFlags
    );

XBOXAPI
DWORD
WINAPI
XUserContentRestrictionGetRating(
    IN      DWORD               dwUserIndex,
    IN      DWORD               dwContentType,
    OUT     PDWORD              pRatingSystem,
    OUT     PDWORD              pRatingLevel
    );

XBOXAPI
DWORD
WINAPI
XUserContentRestrictionCheckAccess(
    IN      DWORD                   dwUserIndex,
    IN      DWORD                   dwAccessLevel,
    IN      DWORD                   dwContentType,
    IN      DWORD                   dwRatingSystem,
    IN      DWORD                   dwRatingLevel,
    OUT     PBOOL                   pfAllowAccess,
    IN OUT  PXOVERLAPPED            pOverlapped
    );

//
// Avatar Award APIs
//

typedef struct
{
    DWORD                               dwUserIndex;
    DWORD                               dwAwardId;
} XUSER_AVATARASSET, *PXUSER_AVATARASSET;

XBOXAPI
DWORD
WINAPI
XUserAwardAvatarAssets(
    __in                        DWORD                       dwNumAssets,
    __in_ecount( dwNumAssets )  CONST XUSER_AVATARASSET*    pAssets,
    __inout_opt                 PXOVERLAPPED                pOverlapped
    );


//
// Pictures
//

XBOXAPI
DWORD
WINAPI
XUserReadAchievementPicture(
    __in                                DWORD           dwUserIndex,
    __in                                DWORD           dwTitleId,
    __in                                DWORD           dwImageId,
    __inout_bcount( dwPitch*dwHeight )  PBYTE           pbTextureBuffer,
    __in                                DWORD           dwPitch,
    __in                                DWORD           dwHeight,
    __inout_opt                         PXOVERLAPPED    pOverlapped
    );



XBOXAPI
DWORD
WINAPI
XUserReadGamerPicture(
    __in                                DWORD           dwUserIndex,
    __in                                BOOL            fSmall,
    __inout_bcount( dwPitch*dwHeight )  PBYTE           pbTextureBuffer,
    __in                                DWORD           dwPitch,
    __in                                DWORD           dwHeight,
    __inout_opt                         PXOVERLAPPED    pOverlapped
    );


XBOXAPI
DWORD
WINAPI
XUserReadGamerPictureByKey(
    __in                                CONST PXUSER_DATA   pGamercardPictureKey,
    __in                                BOOL                fSmall,
    __inout_bcount( dwPitch*dwHeight )  PBYTE               pbTextureBuffer,
    __in                                DWORD               dwPitch,
    __in                                DWORD               dwHeight,
    __inout_opt                         PXOVERLAPPED        pOverlapped
    );


XBOXAPI
DWORD
WINAPI
XUserAwardGamerPicture(
    __in        DWORD                   dwUserIndex,
    __in        DWORD                   dwPictureId,
    __in        DWORD                   dwReserved,
    __inout_opt PXOVERLAPPED            pXOverlapped
    );


// Stats APIs

#define XUSER_STATS_ATTRS_IN_SPEC       64

typedef struct _XUSER_STATS_COLUMN
{
    WORD                                wColumnId;
    XUSER_DATA                          Value;
} XUSER_STATS_COLUMN, *PXUSER_STATS_COLUMN;

typedef struct _XUSER_STATS_ROW
{
    XUID                                xuid;
    DWORD                               dwRank;
    LONGLONG                            i64Rating;
    CHAR                                szGamertag[XUSER_NAME_SIZE];
    DWORD                               dwNumColumns;
    PXUSER_STATS_COLUMN                 pColumns;
} XUSER_STATS_ROW, *PXUSER_STATS_ROW;

typedef struct _XUSER_STATS_VIEW
{
    DWORD                               dwViewId;
    DWORD                               dwTotalViewRows;
    DWORD                               dwNumRows;
    PXUSER_STATS_ROW                    pRows;
} XUSER_STATS_VIEW, *PXUSER_STATS_VIEW;

typedef struct _XUSER_STATS_READ_RESULTS
{
    DWORD                               dwNumViews;
    PXUSER_STATS_VIEW                   pViews;
} XUSER_STATS_READ_RESULTS, *PXUSER_STATS_READ_RESULTS;

typedef struct _XUSER_STATS_SPEC
{
    DWORD                               dwViewId;
    DWORD                               dwNumColumnIds;
    WORD                                rgwColumnIds[XUSER_STATS_ATTRS_IN_SPEC];
} XUSER_STATS_SPEC, *PXUSER_STATS_SPEC;

XBOXAPI
DWORD
WINAPI
XUserResetStatsView(
    __in        DWORD                   dwUserIndex,
    __in        DWORD                   dwViewId,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XUserResetStatsViewAllUsers(
    __in        DWORD                   dwViewId,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XUserReadStats(
    __in                            DWORD                       dwTitleId,
    __in                            DWORD                       dwNumXuids,
    __in_ecount( dwNumXuids )       CONST XUID*                 pXuids,
    __in                            DWORD                       dwNumStatsSpecs,
    __in_ecount( dwNumStatSpecs )   CONST XUSER_STATS_SPEC*     pSpecs,
    __inout                         PDWORD                      pcbResults,
    __out_bcount_opt( pcbResults )  PXUSER_STATS_READ_RESULTS   pResults,
    __inout_opt                     PXOVERLAPPED                pOverlapped
    );


XBOXAPI
DWORD
WINAPI
XUserCreateStatsEnumeratorByRank(
    __in                            DWORD                   dwTitleId,
    __in                            DWORD                   dwRankStart,
    __in                            DWORD                   dwNumRows,
    __in                            DWORD                   dwNumStatsSpecs,
    __in_ecount( dwNumStatsSpecs )  CONST XUSER_STATS_SPEC* pSpecs,
    __out                           PDWORD                  pcbBuffer,
    __out                           PHANDLE                 ph
    );

XBOXAPI
DWORD
WINAPI
XUserCreateStatsEnumeratorByXuid(
    __in                            DWORD                   dwTitleId,
    __in                            XUID                    XuidPivot,
    __in                            DWORD                   dwNumRows,
    __in                            DWORD                   dwNumStatsSpecs,
    __in_ecount( dwNumStatsSpecs )  CONST XUSER_STATS_SPEC* pSpecs,
    __out                           PDWORD                  pcbBuffer,
    __out                           PHANDLE                 ph
    );


XBOXAPI
DWORD
WINAPI
XUserCreateStatsEnumeratorByRating(
    __in                            DWORD                   dwTitleId,
    __in                            LONGLONG                i64Rating,
    __in                            DWORD                   dwNumRows,
    __in                            DWORD                   dwNumStatsSpecs,
    __in_ecount( dwNumStatsSpecs )  CONST XUSER_STATS_SPEC* pSpecs,
    __out                           PDWORD                  pcbBuffer,
    __out                           PHANDLE                 ph
    );


// System defined leaderboard columns
#define X_STATS_COLUMN_ATTACHMENT_SIZE          ((WORD)0xFFFA)

// Column ids for skill leaderboards (STATS_VIEW_SKILL_* views)
#define X_STATS_COLUMN_SKILL_SKILL              61
#define X_STATS_COLUMN_SKILL_GAMESPLAYED        62
#define X_STATS_COLUMN_SKILL_MU                 63
#define X_STATS_COLUMN_SKILL_SIGMA              64

#define X_STATS_SKILL_SKILL_DEFAULT             1
#define X_STATS_SKILL_MU_DEFAULT                3.0
#define X_STATS_SKILL_SIGMA_DEFAULT             1.0


//------------------------------------------------------------------------------
// Strings stored in the SPA file
//------------------------------------------------------------------------------

#define X_STRINGID_TITLE                0x8000

#define X_STRINGID_GAMETYPE             0x7f00
#define X_STRINGID_RANKED               0x7f01
#define X_STRINGID_STANDARD             0x7f02
#define X_STRINGID_GAMERNAME            0x7f03
#define X_STRINGID_GAMERNAMECOL         0x7f04
#define X_STRINGID_RANK                 0x7f05
#define X_STRINGID_RANKCOL              0x7f06


XBOXAPI
DWORD
WINAPI
XResourceGetString(
    __in    DWORD                       dwStringId,
    __out_ecount( *pcch ) WCHAR*        pwszStringBuffer,
    __inout PDWORD                      pcch,
    __inout_opt PXOVERLAPPED            pOverlapped
    );



// Signin UI API

// XShowSigninUI flags.
#define XSSUI_FLAGS_LOCALSIGNINONLY                 0x00000001
#define XSSUI_FLAGS_SHOWONLYONLINEENABLED           0x00000002
#define XSSUI_FLAGS_ALLOW_SIGNOUT                   0x00000004
#define XSSUI_FLAGS_DISALLOW_PLAYAS                 0x00000010
#define XSSUI_FLAGS_ADDUSER                         0x00010000
#define XSSUI_FLAGS_ENABLE_GUEST                    0x00080000
#define XSSUI_FLAGS_CONVERTOFFLINETOGUEST           0x00400000
#define XSSUI_FLAGS_DISALLOW_GUEST                  0x01000000

// XShowCommunitySessionsUI flags.
#define XSHOWCOMMUNITYSESSION_SHOWPARTY             0x00000001

//XPlat constant for max users on platform
#define XSIGNIN_MAX_COUNT                           4


XBOXAPI
DWORD
WINAPI
XEnableGuestSignin(
    __in    BOOL                    fEnable
    );

XBOXAPI
DWORD
WINAPI
XShowNuiGuideUI(
    __in    DWORD                   dwTrackingID
    );

XBOXAPI
DWORD
WINAPI
XShowSigninUI(
    __in    DWORD                   cPanes,
    __in    DWORD                   dwFlags
    );


XBOXAPI
DWORD
WINAPI
XShowNuiSigninUI(
    __in        DWORD               dwTrackingID,
    __in        DWORD               dwFlags
    );

XBOXAPI
DWORD
WINAPI
XShowFriendsUI(
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XShowNuiFriendsUI(
    __in    DWORD                   dwTrackingID,
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XShowPlayersUI(
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XShowCommunitySessionsUI(
    __in        DWORD                   dwUserIndex,
    __in        DWORD                   dwSocialSessionsFlags
    );

XBOXAPI
DWORD
WINAPI
XShowNuiCommunitySessionsUI(
    __in        DWORD                   dwTrackingID,
    __in        DWORD                   dwUserIndex,
    __in        DWORD                   dwSocialSessionsFlags
    );

XBOXAPI
DWORD
WINAPI
XShowNuiControllerRequiredUI(
    __in    DWORD                   dwTrackingID,
    __in    DWORD                   dwUserIndex
    );

#define XSNHRUI_FLAGS_HIDEEXITTITLE   0x00000001

XBOXAPI
DWORD
WINAPI
XShowNuiHardwareRequiredUI(
    __in    DWORD                   dwFlags
    );


XBOXAPI
DWORD
WINAPI
XShowMessagesUI(
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XShowPartyUI(
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XShowNuiPartyUI(
    __in    DWORD                   dwTrackingID,
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XShowVoiceChannelUI(
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XShowGameVoiceChannelUI(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XShowRequiredGameVoiceChannelUI(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XShowMessageComposeUI(
    __in                            DWORD       dwUserIndex,
    __in_ecount_opt( cRecipients )  CONST XUID* pXuidRecipients,
    __in                            DWORD       cRecipients,
    __in_z_opt                      LPCWSTR     wszText
    );

// Deprecated. Use XCUSTOMACTION_FLAG_CLOSES_GUIDE instead.
#define CUSTOMACTION_FLAG_CLOSESUI      1

typedef enum _XMSG_FLAGS
{
    XMSG_FLAG_DISABLE_EDIT_RECIPIENTS   = 0x00000001
} XMSG_FLAGS;

#define XMSG_MAX_CUSTOM_IMAGE_SIZE      (36*1024)

typedef enum _XCUSTOMACTION_FLAGS
{
    XCUSTOMACTION_FLAG_CLOSES_GUIDE     = 0x00000001,
    XCUSTOMACTION_FLAG_DELETES_MESSAGE  = 0x00000002
} XCUSTOMACTION_FLAGS;

#define XMSG_MAX_CUSTOMACTION_TRANSLATIONS      11


typedef struct
{
    DWORD                               dwActionId;
    WCHAR                               wszEnActionText[23];
    WORD                                wReserved;
    DWORD                               dwFlags;
    struct
    {
        DWORD                           dwLanguageId;
        WCHAR                           wszActionText[23];
        WORD                            wReserved;
    } rgTranslations[XMSG_MAX_CUSTOMACTION_TRANSLATIONS];
} XMSG_CUSTOMACTION;

#define XCUSTOMACTION_MAX_PAYLOAD_SIZE  1024

XBOXAPI
DWORD
WINAPI
XShowCustomMessageComposeUI(
    __in    DWORD                       dwUserIndex,
    __in_ecount_opt( cRecipients ) CONST XUID* pXuidRecipients,
    __in    DWORD                       cRecipients,
    __in    DWORD                       dwFlags,
    __in_z  LPCWSTR                     wszTitle,
    __in_z  LPCWSTR                     wszGameMessage,
    __in_z_opt LPCWSTR                  wszPlayerMessage,
    __in_bcount_opt( cbImage ) CONST BYTE* pbImage,
    __in    DWORD                       cbImage,
    __in_ecount( cCustomActions ) CONST XMSG_CUSTOMACTION* pCustomActions,
    __in    DWORD                       cCustomActions,
    __in_bcount_opt( cbCustomPayload ) CONST BYTE* pbCustomPayload,
    __in    DWORD                       cbCustomPayload,
    __in    DWORD                       dwExpireMinutes,
    __inout_opt XOVERLAPPED*            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XShowGameInviteUI(
    __in                            DWORD       dwUserIndex,
    __in_ecount_opt( cRecipients )  CONST XUID* pXuidRecipients,
    __in                            DWORD       cRecipients,
    __in_z_opt                      LPCWSTR     wszUnused
    );

XBOXAPI
DWORD
WINAPI
XShowNuiGameInviteUI(
    __in                            DWORD       dwTrackingID,
    __in                            DWORD       dwUserIndex,
    __in_ecount( cRecipients )      CONST XUID* pXuidRecipients,
    __in                            DWORD       cRecipients,
    __in_z_opt                      LPCWSTR     wszUnused
    );

XBOXAPI
DWORD
WINAPI
XShowFriendRequestUI(
    __in    DWORD                   dwUserIndex,
    __in    XUID                    xuidUser
    );

XBOXAPI
DWORD
WINAPI
XShowNuiFriendRequestUI(
    __in    DWORD                   dwTrackingID,
    __in    DWORD                   dwUserIndex,
    __in    XUID                    xuidUser
    );

#define XPLAYERLIST_CUSTOMTEXT_MAX_LENGTH   31
#define XPLAYERLIST_TITLE_MAX_LENGTH        36
#define XPLAYERLIST_DESCRIPTION_MAX_LENGTH  83
#define XPLAYERLIST_IMAGE_MAX_SIZE          36864
#define XPLAYERLIST_MAX_PLAYERS             100
#define XPLAYERLIST_BUTTONTEXT_MAX_LENGTH   23

typedef struct
{
    XUID        xuid;
    WCHAR       wszCustomText[XPLAYERLIST_CUSTOMTEXT_MAX_LENGTH];
} XPLAYERLIST_USER;

typedef struct
{
    XUID        xuidSelected;
    DWORD       dwKeyCode;
} XPLAYERLIST_RESULT;

typedef enum _XPLAYERLIST_BUTTON_TYPE
{
    XPLAYERLIST_BUTTON_TYPE_TITLECUSTOM          = 0,
    XPLAYERLIST_BUTTON_TYPE_PLAYERREVIEW,
    XPLAYERLIST_BUTTON_TYPE_GAMEINVITE,
    XPLAYERLIST_BUTTON_TYPE_MESSAGE,
    XPLAYERLIST_BUTTON_TYPE_FRIENDREQUEST,
} XPLAYERLIST_BUTTON_TYPE;

typedef struct
{
    DWORD dwType;
    WCHAR wszCustomText[XPLAYERLIST_BUTTONTEXT_MAX_LENGTH];
} XPLAYERLIST_BUTTON;


typedef enum _XPLAYERLIST_FLAGS
{
    XPLAYERLIST_FLAG_CUSTOMTEXT             = 0x00000001,
} XPLAYERLIST_FLAGS;


XBOXAPI
DWORD
WINAPI
XShowCustomPlayerListUI(
    __in                        DWORD                       dwUserIndex,
    __in                        DWORD                       dwFlags,
    __in_z                      LPCWSTR                     pszTitle,
    __in_z                      LPCWSTR                     pszDescription,
    __in_bcount_opt( cbImage )  CONST BYTE*                 pbImage,
    __in                        DWORD                       cbImage,
    __in_ecount( cPlayers )     CONST XPLAYERLIST_USER*     rgPlayers,
    __in                        DWORD                       cPlayers,
    __in_opt                    CONST XPLAYERLIST_BUTTON*   pXButton,
    __in_opt                    CONST XPLAYERLIST_BUTTON*   pYButton,
    __out_opt                   XPLAYERLIST_RESULT*         pResult,
    __inout_opt                 XOVERLAPPED*                pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XShowKeyboardUI(
    __in                        DWORD           dwUserIndex,
    __in                        DWORD           dwFlags,
    __in_z_opt                  LPCWSTR         wseDefaultText,
    __in_z_opt                  LPCWSTR         wszTitleText,
    __in_z_opt                  LPCWSTR         wszDescriptionText,
    __out_ecount( cchResultText ) LPWSTR        wszResultText,
    __in                        DWORD           cchResultText,
    __inout_opt                 PXOVERLAPPED    pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XShowWordRegisterUI(
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XShowGamerCardUI(
    __in    DWORD                   dwUserIndex,
    __in    XUID                    XuidPlayer
    );

XBOXAPI
DWORD
WINAPI
XShowNuiGamerCardUI(
    __in    DWORD                   dwTrackingID,
    __in    DWORD                   dwUserIndex,
    __in    XUID                    XuidPlayer
    );

XBOXAPI
DWORD
WINAPI
XShowAchievementsUI(
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
DWORD
WINAPI
XShowNuiAchievementsUI(
    __in    DWORD                   dwTrackingID,
    __in    DWORD                   dwUserIndex
    );

typedef enum _XSHOWMARKETPLACEUI_ENTRYPOINTS {
    XSHOWMARKETPLACEUI_ENTRYPOINT_CONTENTLIST,
    XSHOWMARKETPLACEUI_ENTRYPOINT_CONTENTITEM,
    XSHOWMARKETPLACEUI_ENTRYPOINT_MEMBERSHIPLIST,
    XSHOWMARKETPLACEUI_ENTRYPOINT_MEMBERSHIPITEM,
    XSHOWMARKETPLACEUI_ENTRYPOINT_CONTENTLIST_BACKGROUND,
    XSHOWMARKETPLACEUI_ENTRYPOINT_CONTENTITEM_BACKGROUND,
    XSHOWMARKETPLACEUI_ENTRYPOINT_MAX
} XSHOWMARKETPLACEUI_ENTRYPOINTS;


XBOXAPI
DWORD
WINAPI
XShowMarketplaceUI(
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwEntryPoint,
    __in_opt ULONGLONG                  qwOfferID,
    __in    DWORD                       dwContentCategories
    );

XBOXAPI
DWORD
WINAPI
XShowNuiMarketplaceUI(
    __in    DWORD                       dwTrackingID,
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwEntryPoint,
    __in_opt ULONGLONG                  qwOfferID,
    __in    DWORD                       dwContentCategories
    );

XBOXAPI
DWORD
WINAPI
XShowTokenRedemptionUI(
    __in    DWORD                       dwUserIndex
    );

//-----------------------------------------------------------------------------
// XShowMarketplaceDownloadItemsUI Result errors = 0x8057F0XX
//-----------------------------------------------------------------------------

#define MPDI_E_CANCELLED            _HRESULT_TYPEDEF_(0x8057F001)
#define MPDI_E_INVALIDARG           _HRESULT_TYPEDEF_(0x8057F002)
#define MPDI_E_OPERATION_FAILED     _HRESULT_TYPEDEF_(0x8057F003)

typedef enum _XSHOWMARKETPLACEDOWNLOADITEMSUI_ENTRYPOINTS {
    XSHOWMARKETPLACEDOWNLOADITEMS_ENTRYPOINT_FREEITEMS=1000,
    XSHOWMARKETPLACEDOWNLOADITEMS_ENTRYPOINT_PAIDITEMS,
    XSHOWMARKETPLACEDOWNLOADITEMS_ENTRYPOINT_MAX
} XSHOWMARKETPLACEDOWNLOADITEMSUI_ENTRYPOINTS;

XBOXAPI
DWORD
WINAPI
XShowMarketplaceDownloadItemsUI(
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwEntryPoint,
    __in_ecount( dwOfferIdCount ) CONST ULONGLONG* pOfferIDs,
    __in    DWORD                       dwOfferIdCount,
    __out   HRESULT*                    phrResult,
    __inout_opt PXOVERLAPPED            pOverlapped
    );

#define XMARKETPLACE_IMAGE_URL_MINIMUM_WCHARCOUNT   55
#define XMARKETPLACE_MAX_OFFERIDS                   6

XBOXAPI
VOID
APIENTRY
XMarketplaceGetImageUrl(
  __in   DWORD     dwTitleID,
  __in   ULONGLONG qwOfferID,
  __in   DWORD     cchStringBuffer,
  __out_ecount( cchStringBuffer ) WCHAR* pwszStringBuffer
  );

XBOXAPI
DWORD
APIENTRY
XMarketplaceGetDownloadStatus(
  __in  DWORD       dwUserIndex,
  __in  ULONGLONG   qwOfferID,
  __out LPDWORD     pdwResult
  );

XBOXAPI
BOOL
APIENTRY
XMarketplaceDoesContentIdMatch(
    __in_bcount( XMARKETPLACE_CONTEXT_ID_LEN ) CONST BYTE* pbContentId,
    __in CONST XCONTENT_DATA*    pContentData
    );


typedef enum _XBACKGROUND_DOWNLOAD_MODE {
    XBACKGROUND_DOWNLOAD_MODE_ALWAYS_ALLOW = 1,
    XBACKGROUND_DOWNLOAD_MODE_AUTO         = 2
} XBACKGROUND_DOWNLOAD_MODE, * PXBACKGROUND_DOWNLOAD_MODE;

XBOXAPI
DWORD
WINAPI
XBackgroundDownloadSetMode(
    __in   XBACKGROUND_DOWNLOAD_MODE    Mode
    );

XBOXAPI
XBACKGROUND_DOWNLOAD_MODE
WINAPI
XBackgroundDownloadGetMode(
    VOID
    );

#ifdef XONLINE_CONTENT_ID_LEN
#define ACQUIRE_CONTENT_ID_LEN XONLINE_CONTENT_ID_LEN
#else
#define ACQUIRE_CONTENT_ID_LEN 20 // XONLINE_CONTENT_ID_LEN
#endif

typedef struct
{
    ULONGLONG       legacyOfferId;
    GUID            offerId;
    GUID            mediaId;
    GUID            mediaInstanceId;
    BYTE            contentId[ACQUIRE_CONTENT_ID_LEN];
    WCHAR           displayName[XCONTENT_MAX_DISPLAYNAME_LENGTH];
    LARGE_INTEGER   installSize;
} XMARKETPLACE_ACQUIRE_CONTENT_INFO;

XBOXAPI
DWORD
WINAPI
XMarketplaceAcquireFreeContent(
    __in    DWORD                       dwUserIndex,
    __in    CONST XMARKETPLACE_ACQUIRE_CONTENT_INFO*  pAcquireInfo,
    __inout PXOVERLAPPED                pOverlapped
    );


XBOXAPI
DWORD
WINAPI
XShowDeviceSelectorUI(
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwContentType,
    __in    DWORD                       dwContentFlags,
    __in    ULARGE_INTEGER              uliBytesRequested,
    __out   PXCONTENTDEVICEID           pDeviceID,
    __inout PXOVERLAPPED                pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XShowNuiDeviceSelectorUI(
    __in    DWORD                       dwTrackingID,
    __in    DWORD                       dwUserIndex,
    __in    DWORD                       dwContentType,
    __in    DWORD                       dwContentFlags,
    __in    ULARGE_INTEGER              uliBytesRequested,
    __out   PXCONTENTDEVICEID           pDeviceID,
    __inout PXOVERLAPPED                pOverlapped
    );

DECLSPEC_NORETURN
XBOXAPI
VOID
WINAPI
XShowDirtyDiscErrorUI(
    __in    DWORD                       dwUserIndex
    );

DECLSPEC_NORETURN
XBOXAPI
VOID
WINAPI
XShowNuiDirtyDiscErrorUI(
    __in    DWORD                       dwTrackingID,
    __in    DWORD                       dwUserIndex
    );

// API to show error and informational messages
#define XMB_NOICON                      0x00000000
#define XMB_ERRORICON                   0x00000001
#define XMB_WARNINGICON                 0x00000002
#define XMB_ALERTICON                   0x00000003

#define XMB_PASSCODEMODE                0x00010000
#define XMB_VERIFYPASSCODEMODE          0x00020000

#define XMB_MAXBUTTONS                  3
#define XMB_CANCELID                    -1

typedef struct _MESSAGEBOX_RESULT
{
    union
    {
        DWORD                           dwButtonPressed;
        WORD                            rgwPasscode[4];
    };
} MESSAGEBOX_RESULT, *PMESSAGEBOX_RESULT;

XBOXAPI
DWORD
WINAPI
XShowMessageBoxUI(
    __in                        DWORD               dwUserIndex,
    __in_z                      LPCWSTR             wszTitle,
    __in_z                      LPCWSTR             wszText,
    __in                        DWORD               cButtons,
    __in_opt                    LPCWSTR*            pwszButtons,
    __in                        DWORD               dwFocusButton,
    __in                        DWORD               dwFlags,
    __out                       PMESSAGEBOX_RESULT  pResult,
    __inout                     PXOVERLAPPED        pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XShowNuiMessageBoxUI(
    __in                        DWORD               dwTrackingID,
    __in                        DWORD               dwUserIndex,
    __in_z                      LPCWSTR             wszTitle,
    __in_z                      LPCWSTR             wszText,
    __in                        DWORD               cButtons,
    __in_opt                    LPCWSTR*            pwszButtons,
    __in                        DWORD               dwFocusButton,
    __in                        DWORD               dwFlags,
    __out                       PMESSAGEBOX_RESULT  pResult,
    __inout                     PXOVERLAPPED        pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XShowPlayerReviewUI(
    __in    DWORD                   dwUserIndex,
    __in    XUID                    XuidFeedbackTarget
    );

XBOXAPI
DWORD
WINAPI
XShowMultiplayerUpgradeUI(
    __in    DWORD                   dwUserIndex
    );

XBOXAPI
VOID
WINAPI
XCustomSetAction(
    __in        DWORD               dwActionIndex,
    __in_z_opt  LPCWSTR             szActionText,
    __in        DWORD               dwFlags
    );

typedef struct
{
    WORD                                wActionId;
    WCHAR                               wszActionText[23];
    DWORD                               dwFlags;
} XCUSTOMACTION;

XBOXAPI
VOID
WINAPI
XCustomRegisterDynamicActions(
    VOID
    );

XBOXAPI
VOID
WINAPI
XCustomUnregisterDynamicActions(
    VOID
    );

XBOXAPI
BOOL
WINAPI
XCustomGetCurrentGamercard(
    __out   DWORD*                  pdwUserIndex,
    __out   XUID*                   pXuid
    );

XBOXAPI
DWORD
WINAPI
XCustomSetDynamicActions(
    __in                        DWORD                   dwUserIndex,
    __in                        XUID                    xuid,
    __in_ecount( cCustomActions ) CONST XCUSTOMACTION*  pCustomActions,
    __in                        WORD                    cCustomActions
    );

XBOXAPI
BOOL
WINAPI
XCustomGetLastActionPress(
    __out_opt   DWORD*              pdwUserIndex,
    __out_opt   DWORD*              pdwActionIndex,
    __out_opt   XUID*               pXuid
    );

XBOXAPI
DWORD
WINAPI
XCustomGetLastActionPressEx(
    __out_opt                       DWORD*  pdwUserIndex,
    __out_opt                       DWORD*  pdwActionId,
    __out_opt                       XUID*   pXuid,
    __out_bcount_opt( pcbPayload )  BYTE*   pbPayload,
    __inout_opt                     WORD*   pcbPayload
    );


XBOXAPI
DWORD
WINAPI
XEnableScreenSaver(
    __in    BOOL                        fEnable
    );

//-----------------------------------------------------------------------------
// Closed Captioning
//-----------------------------------------------------------------------------

XBOXAPI
BYTE
WINAPI
XGetClosedCaptioningState(
    VOID
    );

XBOXAPI
DWORD
WINAPI
XGetClosedCaptioningSettings(
    __out PXCLOSEDCAPTIONING_SETTINGS pClosedCaptioningSettings
    );


typedef enum _XTITLE_DEPLOYMENT_TYPE {
    XTITLE_DEPLOYMENT_OPTICAL_DISC,
    XTITLE_DEPLOYMENT_INSTALLED_TO_HDD,
    XTITLE_DEPLOYMENT_DOWNLOAD,
    XTITLE_DEPLOYMENT_OTHER,
} XTITLE_DEPLOYMENT_TYPE, *PXTITLE_DEPLOYMENT_TYPE;

XBOXAPI
DWORD
WINAPI
XTitleGetDeploymentType(
    __out PXTITLE_DEPLOYMENT_TYPE       pTitleDeploymentType,
    __inout_opt PXOVERLAPPED            pOverlapped
    );


typedef struct _XHASH_STATE_SHA1 {
    BYTE Internal[88];
} XHASH_STATE_SHA1, *PXHASH_STATE_SHA1;

#define XHASHMEMORY_DIGEST_SIZE_SHA1 (20)

XBOXAPI
VOID
WINAPI
XHashMemoryBeginSHA1Deprecated(
    __out   XHASH_STATE_SHA1*                pState
    );

XBOXAPI
VOID
WINAPI
XHashMemoryUpdateSHA1Deprecated(
    __inout XHASH_STATE_SHA1*                pState,
    __in_bcount( dwSize ) CONST VOID*        pvMemory,
    __in    DWORD                            dwSize
    );

XBOXAPI
VOID
WINAPI
XHashMemoryEndSHA1Deprecated(
    __in    CONST XHASH_STATE_SHA1*          pState,
    __out_bcount( XHASHMEMORY_DIGEST_SIZE_SHA1 ) BYTE* pbDigest
    );

#pragma deprecated(XHashMemoryBeginSHA1Deprecated)
#pragma deprecated(XHashMemoryUpdateSHA1Deprecated)
#pragma deprecated(XHashMemoryEndSHA1Deprecated)


typedef PVOID XHASH_CONTEXT;

typedef enum _XHASHTYPE {
    eXHashSHA1Deprecated,
    eXHashSHA256,
    eXHashSHA384,
    eXHashSHA512
} XHASHTYPE;

#define XHASH_DIGEST_SIZE_SHA1_DEPRECATED 20
#define XHASH_DIGEST_SIZE_SHA256 32
#define XHASH_DIGEST_SIZE_SHA384 48
#define XHASH_DIGEST_SIZE_SHA512 64

XBOXAPI
HRESULT
WINAPI
XHashMemoryBegin(
    __in    XHASHTYPE                   HashType,
    __out   XHASH_CONTEXT*              pHashContext
    );

XBOXAPI
HRESULT
WINAPI
XHashMemoryUpdate(
    __in    XHASH_CONTEXT               pHashContext,
    __in_bcount( dwSize ) CONST VOID*   pvMemory,
    __in    DWORD                       dwSize
    );

XBOXAPI
HRESULT
WINAPI
XHashMemoryEnd(
    __in    XHASH_CONTEXT               pHashContext,
    __out_bcount_opt( cbDigest ) BYTE*  pbDigest,
    __in    DWORD                       cbDigest
    );

XBOXAPI
BOOL
WINAPI
XCloseHandle(
    __in     HANDLE                     hObject
    );



//-----------------------------------------------------------------------------
// Security Status.
//-----------------------------------------------------------------------------
typedef enum _XSECURITY_STATUS_CHECKS {
    XSECURITY_STATUS_ANY,
    XSECURITY_STATUS_VERSION_CHECK,
    XSECURITY_STATUS_DISC_CHECK,
    XSECURITY_STATUS_MAXIMUM
} XSECURITY_STATUS_CHECK;

//
// Returns:
//   ERROR_INVALID_PARAMETER - unknown security check
//   ERROR_ACCESS_DENIED - security check failed
//   ERROR_SUCCESS - security check succeeded.
XBOXAPI
DWORD
WINAPI
XGetSecurityStatus(
    __in DWORD dwSecurityCheck
    );

XBOXAPI
DWORD
WINAPI
XSetSecurityStatus(
    __in DWORD dwSecurityCheck,
    __in DWORD dwStatus
    );



//-----------------------------------------------------------------------------
// Nui
//-----------------------------------------------------------------------------


XBOXAPI
HRESULT
WINAPI
XNuiDelayUI(
    __in    ULONG   ulMilliSeconds
    );

XBOXAPI
DWORD
WINAPI
XNuiGetHardwareStatus();

#if !defined NUIAPI
typedef struct _NUI_SKELETON_FRAME NUI_SKELETON_FRAME;
typedef struct _NUI_IMAGE_FRAME    NUI_IMAGE_FRAME;
#endif


XBOXAPI
BOOL
WINAPI
XNuiFirstUseCompleted();

XBOXAPI
BOOL
WINAPI
XNuiIsChatMicrophoneEnabled();


//-----------------------------------------------------------------------------
// Title hashing.
//-----------------------------------------------------------------------------

#define XTITLEHASH_MINIMUM_KEY_SIZE    (8)
#define XTITLEHASH_DIGEST_SIZE         (20)


XBOXAPI
HRESULT
WINAPI
XTitleHash(
    __in_bcount( KeySize) const VOID*  KeyData,
    __in  ULONG                        KeySize,
    __out_bcount( DigestSize ) VOID*   DigestData,
    __in  ULONG                        DigestSize
    );

//-----------------------------------------------------------------------------
// Media Anti Piracy checks
//-----------------------------------------------------------------------------

#define XBOX_CIV_ANTI_PIRACY_VERSION 2

#define CIV_DEFAULT_THREAD 5

XBOXAPI
DWORD
WINAPI
XSecurityCreateProcess(
    __in DWORD dwHardwareThread
    );

XBOXAPI
VOID
WINAPI
XSecurityCloseProcess();

XBOXAPI
DWORD
WINAPI
XSecurityVerify(
    __in DWORD dwMilliseconds,
    __in_opt LPOVERLAPPED lpOverlapped,
    __in_opt LPOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine
    );


typedef struct _XSECURITY_FAILURE_INFORMATION
{
    DWORD dwSize;
    DWORD dwFailedReads;
    DWORD dwFailedHashes;
    DWORD dwBlocksChecked;
    DWORD dwTotalBlocks;
    BOOL  fComplete;
} XSECURITY_FAILURE_INFORMATION, *PXSECURITY_FAILURE_INFORMATION;


XBOXAPI
DWORD
WINAPI
XSecurityGetFailureInfo(
    __out PXSECURITY_FAILURE_INFORMATION pFailureInformation
    );

XBOXAPI
DWORD
WINAPI
XShowAntiPiracyUI(
    __in BOOL fTerminate
    );

XBOXAPI
VOID
WINAPI
XSecurityInjectErrors( __in DWORD dwNumberOfHashErrors,
                       __in DWORD dwNumberOfReadErrors
    );


#ifdef __cplusplus
}
#endif


#endif // _XBOX_H_

