
/************************************************************************
*                                                                       *
*   XCam.h -- This module defines the Xbox Camera APIs                  *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/

#pragma once
#ifndef __XCAM_H__
#define __XCAM_H__

#include <xonline.h>

#ifdef __cplusplus
extern "C" {
#endif


//------------------------------------------------------------------------------
// XCam Interfaces
//------------------------------------------------------------------------------
typedef struct IXCamVideo          *PIXCAMVIDEO;
typedef struct IXCamStreamEngine   *PIXCAMSTREAMENGINE;
typedef struct IXCamMotionEngine   *PIXCAMMOTIONENGINE;

//------------------------------------------------------------------------------
// XCam Enum Typedefs
//------------------------------------------------------------------------------
typedef enum _XCAMDEVICESTATE {
    XCAMDEVICESTATE_DISCONNECTED    = 0,
    XCAMDEVICESTATE_CONNECTED       = 1,
    XCAMDEVICESTATE_INITIALIZED     = 2,
    XCAMDEVICESTATE_IN_ERROR        = 3,
    XCAMDEVICESTATE_ERROR_FULLSPEED = 4,
    XCAMDEVICESTATE_FORCE_DWORD  = 0x7fffffff
} XCAMDEVICESTATE;

typedef enum _XCAMRESOLUTION {
    XCAMRESOLUTION_160x120      = 0x00A00078,
    XCAMRESOLUTION_176x144      = 0x00B00090,
    XCAMRESOLUTION_320x240      = 0x014000F0,
    XCAMRESOLUTION_352x288      = 0x01600120,
    XCAMRESOLUTION_640x480      = 0x028001E0,
    XCAMRESOLUTION_960x720      = 0x03C002D0,
    XCAMRESOLUTION_1280x960     = 0x050003C0,
    XCAMRESOLUTION_FORCE_DWORD  = 0x7fffffff
} XCAMRESOLUTION;

typedef enum _XCAMFRAMERATE {
    XCAMFRAMERATE_FASTEST       = 0,
    XCAMFRAMERATE_5             = 5,
    XCAMFRAMERATE_10            = 10,
    XCAMFRAMERATE_15            = 15,
    XCAMFRAMERATE_20            = 20,
    XCAMFRAMERATE_25            = 25,
    XCAMFRAMERATE_30            = 30,
    XCAMFRAMERATE_60            = 60,
    XCAMFRAMERATE_FORCE_DWORD   = 0x7fffffff
} XCAMFRAMERATE;

typedef enum _XCAMCONFIGIDS {
    XCAMCONFIGID_AE_MODE         = 2,
    XCAMCONFIGID_AE_PRIORITY     = 3,
    XCAMCONFIGID_EXPOSURE_TIME   = 4,
    XCAMCONFIGID_BACKLIGHT_COMP  = 101,
    XCAMCONFIGID_BRIGHTNESS      = 102,
    XCAMCONFIGID_CONTRAST        = 103,
    XCAMCONFIGID_GAIN            = 104,
    XCAMCONFIGID_FLICKER         = 105,
    XCAMCONFIGID_HUE             = 106,
    XCAMCONFIGID_SATURATION      = 107,
    XCAMCONFIGID_SHARPNESS       = 108,
    XCAMCONFIGID_GAMMA           = 109,
    XCAMCONFIGID_WHITE_BALANCE   = 110,
    XCAMCONFIGID_AUTO_WHITE_BAL  = 111,
    XCAMCONFIGID_LOWLIGHT        = 202,
    XCAMCONFIGID_LIGHTSOURCE     = 204,
    XCAMCONFIGID_COMP_ROOM_LIGHT = 300,
    XCAMCONFIGID_COMP_CORRECTED_LIGHTSOURCE = 301,
    XCAMCONFIGCONTROLID_FORCE_DWORD  = 0x7fffffff
} XCAMCONFIGCONTROLID;

typedef enum _XCAMZOOMFACTOR {
    XCAMZOOMFACTOR_1X            = 1,
    XCAMZOOMFACTOR_2X            = 2,
    XCAMZOOMFACTOR_4X            = 4,
    XCAMZOOMFACTOR_FORCE_DWORD   = 0x7fffffff
} XCAMZOOMFACTOR;


static const LONG XCAMCONFIG_LIGHTSOURCE_INCANDESCENT = 0;
static const LONG XCAMCONFIG_LIGHTSOURCE_FLOURESCENT = 1;
static const LONG XCAMCONFIG_LIGHTSOURCE_DAYLIGHT = 2;


//------------------------------------------------------------------------------
// XCam Callback Functions
//------------------------------------------------------------------------------
typedef VOID (*PFNCAMERADATAREADY)(DWORD UserData, PBYTE pVideoFrame, DWORD dwSize);

//------------------------------------------------------------------------------
// XCam Structures
//------------------------------------------------------------------------------
typedef struct _XCAM_STREAM_ENGINE_INIT_PARAMS {
    XCAMRESOLUTION                 VideoResolution;
    DWORD                          MaximumBitrate;
    DWORD                          MinimumBitrate;
    DWORD                          InitialTargetBitrate;
    XCAMFRAMERATE                  InitialTargetFramerate;
    DWORD                          MaxRemoteConsoles;
    DWORD                          NetworkDataPacketSize;
    DWORD                          ThreadProcessorID;
    BOOL                           GenerateLocalPreview;
    BOOL                           GetDataDirectFromCamera;
    PFNCAMERADATAREADY             CameraDataReadyCallback;
    DWORD                          CameraDataReadyCallbackUserdata;
    IDirect3DDevice9              *pD3DDevice;
} XCAM_STREAM_ENGINE_INIT_PARAMS, *PXCAM_STREAM_ENGINE_INIT_PARAMS;

typedef struct _XCAM_MOTION_ENGINE_INIT_PARAMS {
    DWORD                          ThreadProcessorID;
    XCAMRESOLUTION                 VideoResolution;
    IDirect3DDevice9*              pD3DDevice;
    BOOL                           GetDataDirectFromCamera;
    FLOAT                          PersistenceValue;
    BYTE                           MovementThreshold;
} XCAM_MOTION_ENGINE_INIT_PARAMS, *PXCAM_MOTION_ENGINE_INIT_PARAMS;


//------------------------------------------------------------------------------
// XCam Initialization and Shutdown APIs
//------------------------------------------------------------------------------

XBOXAPI
DWORD
WINAPI
XCamInitialize();

XBOXAPI
VOID
WINAPI
XCamShutdown();


//------------------------------------------------------------------------------
// XCam Basic Camera Access API's
//------------------------------------------------------------------------------


XBOXAPI
XCAMDEVICESTATE
WINAPI
XCamGetStatus();

XBOXAPI
DWORD
WINAPI
XCamGetConfig(
    __in     XCAMCONFIGCONTROLID     ControlID,
    __out    LONG                   *pValue,
    __inout_opt PXOVERLAPPED         pOverlapped );

XBOXAPI
DWORD
WINAPI
XCamSetConfig(
    __in     XCAMCONFIGCONTROLID     ControlID,
    __in     LONG                    Value,
    __inout_opt PXOVERLAPPED         pOverlapped);

XBOXAPI
DWORD
WINAPI
XCamSetView(
    __in     XCAMZOOMFACTOR          ZoomFactor, 
    __in     LONG                    XCenter,
    __in     LONG                    YCenter,
    __inout_opt PXOVERLAPPED         pOverlapped);

XBOXAPI
DWORD
WINAPI
XCamGetView(
    __out_opt XCAMZOOMFACTOR        *pZoomFactor, 
    __out_opt LONG                  *pXCenter,
    __out_opt LONG                  *pYCenter);

XBOXAPI
DWORD
WINAPI
XCamSnapshot(
    __in      XCAMRESOLUTION        Resolution,
    __inout   D3DLOCKED_RECT       *pBuffer,
    __inout   PXOVERLAPPED          pOverlapped);



//------------------------------------------------------------------------------
// XCam Interface Creation Functions
//------------------------------------------------------------------------------
XBOXAPI
DWORD
WINAPI
XCamCreateVideo(
    __deref_out PIXCAMVIDEO                   *ppVideo
    );

XBOXAPI
DWORD
WINAPI
XCamCreateStreamEngine(
    __in CONST XCAM_STREAM_ENGINE_INIT_PARAMS *pParams,
    __deref_out PIXCAMSTREAMENGINE            *ppEngine
    );

XBOXAPI
DWORD
WINAPI
XCamCreateMotionEngine(
    __in CONST XCAM_MOTION_ENGINE_INIT_PARAMS *pParams,
    __deref_out PIXCAMMOTIONENGINE            *ppEngine
    );


//------------------------------------------------------------------------------
// IXCamVideo Interface
//------------------------------------------------------------------------------

#undef  INTERFACE
#define INTERFACE   IXCamVideo

DECLARE_INTERFACE(IXCamVideo)
{
    STDMETHOD_(ULONG, AddRef)( __in THIS ) PURE;
    STDMETHOD_(ULONG, Release)( __in THIS ) PURE;

    STDMETHOD_(DWORD, SetCaptureMode)(
        __in                        THIS_
        __in    XCAMRESOLUTION      Resolution,
        __in    XCAMFRAMERATE       Framerate,
        __inout_opt PXOVERLAPPED    pOverlapped 
        ) PURE;

    STDMETHOD_(DWORD, ReadFrame)(
        __in                        THIS_
        __inout D3DLOCKED_RECT     *pBuffer,
        __inout PXOVERLAPPED        pOverlapped
        ) PURE;
};


//------------------------------------------------------------------------------
// IXCamStreamEngine Interface
//------------------------------------------------------------------------------

#undef  INTERFACE
#define INTERFACE   IXCamStreamEngine

DECLARE_INTERFACE(IXCamStreamEngine)
{
    STDMETHOD_(ULONG, AddRef)( __in THIS ) PURE;
    STDMETHOD_(ULONG, Release)( __in THIS ) PURE;

    //
    // Finds a free remote console object and registers it with the specified XNADDR.
    // Use the "SendVideoToConsole" and "ReceiveVideoFromConsole" flags to specify
    // whether video data should flow in a single direction or bi-directionally
    // between the local machine and the remote machine.  Subsequent calls to
    // this function with the same XNADDR can be used to enable or disable
    // sending/receiving of video data, and/or update the list of XUIDs signed
    // on to that remote machine.
    //
    // If, because of parental control privileges, a video connection cannot be
    // established with this RemoteConsole, this function will return an error code.
    //
    // NOTE: At least one of the two flags must be TRUE - you cannot register a
    // remote console and neither send video to nor receive video from them!
    //
    STDMETHOD_(DWORD, RegisterRemoteConsole)(
        __in                        THIS_
        __in  XNADDR                RemoteAddress,
        __in_ecount(RemoteXUIDListLength) CONST XUID *RemoteXUIDList,
        __in  DWORD                 RemoteXUIDListLength,
        __in  BOOL                  SendVideoToConsole,
        __in  BOOL                  ReceiveVideoFromConsole
        ) PURE;

    //
    // De-registers a previously registered remote XNADDR.  This will terminate
    // the outgoing/incoming video stream(s) for that XNADDR.
    //
    STDMETHOD_(DWORD, UnregisterRemoteConsole)(
        __in                        THIS_
        __in  XNADDR                RemoteAddress
        ) PURE;

    //
    // Retrieve the current list of registered remote consoles.
    //
    STDMETHOD_(VOID, GetRemoteConsoles)(
        __in                        THIS_
        __out_ecount(*pNumAddresses) XNADDR *pRemoteAddresses,
        __inout PDWORD              pNumAddresses
        ) PURE;

    //
    // Check to see whether the video streams to and from the specified remote
    // console are disabled as a result of family settings. After calling 
    // RegisterRemoteConsole(), this function should be used to check and see
    // whether the console is in a disabled state. Similarly, following a local
    // user sign-in event, this function should be called for each registered
    // RemoteConsole to check and see if any of them have become disabled (or
    // potentially re-enabled). Internally, as soon as the sign-in has occurred,
    // the XCamStreamEngine will re-examine the parental control privileges and
    // determine if video communication with each RemoteConsole can continue.
    // RemoteConsoles that are disabled as a result of family settings will not
    // produce any new outbound video packets, nor will they generate new frames
    // of video.
    //
    STDMETHOD_(BOOL, IsRemoteConsoleDisallowed)(
        __in                        THIS_
        __in    XNADDR              RemoteAddress
        ) PURE;

    //
    // Get the address of the RemoteConsole which is having the hardest time
    // receiving and processing the stream that we are sending.  This is the
    // console that is forcing the bitrate of the local encoder to drop the 
    // farthest.  By terminating the video feed to this console, the engine
    // should be able to raise the encoder bitrate up somewhat (assuming that
    // the other RemoteConsoles are not having an equally hard time receiving
    // and processing the stream we are sending).  If you want to try and
    // improve the video experience for all other recipients (ie: if the actual
    // bitrate has fallen lower than you find acceptable), then use this
    // function to determine which RemoteConsole you should stop sending the
    // local video stream to.
    //
    STDMETHOD_(DWORD, GetSlowestConnection)(
        __in                        THIS_
        __out XNADDR               *pRemoteAddress
        ) PURE;

    //
    // Push a frame of video into the "encode queue".  XOVERLAPPED is signaled once
    // the encode is complete
    //
    STDMETHOD_(DWORD, SubmitLocalVideoFrame)(
        __in                        THIS_
        __in  PBYTE                 pVideoFrame,
        __inout PXOVERLAPPED        pOverlapped
        ) PURE;

    //
    // Retrieves a chunk of data to be sent to the specified XNADDR (which was
    // previously registered via "RegisterRemoteConsole").  This blob can contain
    // both encoded video data, as well as QOS data and necessary feedback 
    // communication related to an incoming video stream.  This should be called
    // for _ALL_ registered remote consoles, regardless of whether they are sending
    // video, receiving video, or both.
    //
    STDMETHOD_(DWORD, GetNetworkData)(
        __in                        THIS_
        __in    XNADDR              RemoteAddress,
        __out_bcount(*pBufferSize) PBYTE pBuffer,
        __inout PDWORD              pBufferSize
        ) PURE;

    //
    // Submits a chunk of data that has been received on the network from the 
    // specified XNADDR (which was previously registered via "RegisterRemoteConsole").
    // See "GetNetworkData" for more information.
    //
    STDMETHOD_(DWORD, SubmitNetworkData)(
        __in                        THIS_
        __in  XNADDR                RemoteAddress,
        __in_bcount(BufferSize) PBYTE pBuffer,
        __in  DWORD                 BufferSize,
        __inout PXOVERLAPPED        pOverlapped
        ) PURE;

    //
    // Specifies the screenspace rectangle in which the incoming video stream
    // corresponding with the RemoteAddress should be rendered.  This function should
    // be used in conjunction with the "RenderRemoteConsole" function.
    //
    STDMETHOD_(DWORD, SetRemoteConsoleRect)(
        __in                        THIS_
        __in      XNADDR            RemoteAddress,
        __in CONST D3DRECT          *pRect
        ) PURE;

    //
    // Specifies the default texture that should be used as the "fallback" texture for
    // this remote console during calls to "RenderRemoteConsole".  If no new frames have
    // been decoded in the last 5 seconds, the engine will switch over to rendering this
    // fallback texture (if one has been provided).  To get rid of a previously set
    // default texture, simply calls this function and pass in NULL for the
    // pDefaultTexture.  This fallback texture has no effect on the data returned by 
    // "GetNextRemoteConsoleFrame", and should only be used in conjunction with the
    // "RenderRemoteConsole" function.
    //
    STDMETHOD_(DWORD, SetRemoteConsoleDefaultTexture)(
        __in                        THIS_
        __in  XNADDR                RemoteAddress,
        __in  D3DTexture           *pDefaultTexture
        ) PURE;

    //
    // Renders the current frame of the incoming video stream corresponding
    // with the specified RemoteAddress in the screenspace rectangle previously
    // specified in "SetRemoteConsoleRect".
    //
    STDMETHOD_(DWORD, RenderRemoteConsole)(
        __in                        THIS_
        __in  XNADDR                RemoteAddress
        ) PURE;

    //
    // Obtains the next frame of video for the specified RemoteAddress. Use
    // this function if you want to handle displaying the incoming stream
    // yourself, rather than use the "RenderRemoteConsole" function. If no
    // new frames are available, this will return ERROR_IO_PENDING.
    //
    STDMETHOD_(DWORD, GetNextRemoteConsoleFrame)(
        __in                        THIS_
        __in    XNADDR              RemoteAddress,
        __inout D3DLOCKED_RECT     *pYBuffer,
        __inout D3DLOCKED_RECT     *pUBuffer,
        __inout D3DLOCKED_RECT     *pVBuffer
        ) PURE;

    //
    // Obtains the next frame of video for the specified RemoteAddress, and
    // converts it to D3DFMT_LIN_X8R8G8B8 directly within this function using
    // the CPU.  This function is _MUCH_ more CPU intensive than 
    // GetNextRemoteConsoleFrame(), and should only be used if you actually want
    // the color conversion to occur on the CPU instead of the GPU. If no new
    // frames are available, this will return ERROR_IO_PENDING.
    //
    STDMETHOD_(DWORD, GetNextRemoteConsoleFrameRGB)(
        __in                        THIS_
        __in    XNADDR              RemoteAddress,
        __inout D3DLOCKED_RECT     *pBuffer
        ) PURE;


    //
    // Does for the local preview video what "SetRemoteConsoleRect"
    // does for a remote console's incoming video stream.
    //
    STDMETHOD_(VOID, SetLocalPreviewRect)(
        __in                        THIS_
        __in CONST D3DRECT          *pRect
        ) PURE;

    //
    // Does for the local preview video what "RenderRemoteConsole" does
    // for a remote console's incoming video stream.
    //
    STDMETHOD_(VOID, RenderLocalPreview)(
        __in                        THIS
        ) PURE;

    //
    // Does for the local preview video what "GetNextRemoteConsoleFrame" does
    // for a remote console's incoming video stream.
    //
    STDMETHOD_(DWORD, GetNextLocalPreviewFrame)(
        __in                        THIS_
        __inout D3DLOCKED_RECT     *pBuffer
        ) PURE;

    //
    // Does for the local preview video what "GetNextRemoteConsoleFrameRGB"
    // does for a remote console's incoming video stream
    //
    STDMETHOD_(DWORD, GetNextLocalPreviewFrameRGB)(
        __in                        THIS_
        __inout D3DLOCKED_RECT     *pBuffer
        ) PURE;

    //
    // Encoder bitrate and framerate controls
    //
    STDMETHOD_(VOID, SetEncoderTargetBitrate)(
        __in                        THIS_
        __in  DWORD                 TargetBitrate
        ) PURE;

    STDMETHOD_(DWORD, GetEncoderTargetBitrate)(
        __in                        THIS
        ) PURE;

    STDMETHOD_(DWORD, GetEncoderActualBitrate)(
        __in                        THIS
        ) PURE;

    STDMETHOD_(VOID, SetEncoderTargetFramerate)(
        __in                        THIS_
        __in  XCAMFRAMERATE         TargetFramerate
        ) PURE;

    STDMETHOD_(XCAMFRAMERATE, GetEncoderTargetFramerate)(
        __in                        THIS
        ) PURE;

    STDMETHOD_(XCAMFRAMERATE, GetEncoderActualFramerate)(
        __in                        THIS
        ) PURE;

};


//------------------------------------------------------------------------------
// IXCamMotionEngine Interface
//------------------------------------------------------------------------------

#undef  INTERFACE
#define INTERFACE   IXCamMotionEngine

DECLARE_INTERFACE(IXCamMotionEngine)
{
    STDMETHOD_(ULONG, AddRef)( __in THIS ) PURE;
    STDMETHOD_(ULONG, Release)( __in THIS ) PURE;

    //
    // Main worker function. Generates motion map by differencing the current
    // and previous frames.
    //
    STDMETHOD_(VOID, GenerateMotionMap)(
        __in                        THIS_
        __inout D3DLOCKED_RECT*     pBufferA, 
        __inout D3DLOCKED_RECT*     pBufferB
        ) PURE;

    //
    // Returns the motion map in a D3DFMT_LIN_L8 locked texture.
    //
    STDMETHOD_(VOID, GetMotionMapData)(
        __in                        THIS_
        __inout D3DLOCKED_RECT*     pBuffer
        ) PURE;
  
    //
    // Tests motion against a filled rectangle. Returns TRUE if motion
    // was detected.
    //
    STDMETHOD_(BOOL, RectFilledTouch)(
        __in                        THIS_
        __inout D3DRECT*            pRect
        ) PURE;

    //
    // Tests motion against a filled rectangle, returning a percentage
    // of motion inside the rectangle (0..1).
    //
    STDMETHOD_(FLOAT, RectFilledTouchPercent)(
        __in                        THIS_
        __inout D3DRECT*            pRect
        ) PURE;

    //
    // Tests motion against a rectange outline. Returns TRUE if motion
    // was detected.
    //
    STDMETHOD_(BOOL, RectOutlineTouch)(
        __in                        THIS_
        __inout D3DRECT*            pRect
        ) PURE;

    //
    // Tests motion against an ellipse outline. Returns TRUE if motion
    // was detected. If the pvecDirection vector is not NULL, on
    // return it will contain the direction of motion.
    //
    STDMETHOD_(BOOL, EllipseOutlineTouch)(
        __in                        THIS_
        __in INT                    nCenterX,
        __in INT                    nCenterY, 
        __in FLOAT                  fRadiusX,
        __in FLOAT                  fRadiusY, 
        __inout D3DVECTOR*          pvecDirection
        ) PURE;

    //
    // Tests motion against a filled ellipse. Returns TRUE if motion
    // was detected. If the pvecDirection vector is not NULL, on
    // return it will contain the direction of motion.
    //
    STDMETHOD_(BOOL, EllipseFilledTouch)(
        __in                        THIS_
        __in INT                    nCenterX,
        __in INT                    nCenterY, 
        __in FLOAT                  fRadiusX,
        __in FLOAT                  fRadiusY, 
        __inout D3DVECTOR*          pvecDirection
        ) PURE;

    //
    // Tests motion against a filled ellipse. Returns a percentage of
    // motion inside the ellipse (0..1). If the pvecDirection vector 
    // is not NULL, on return it will contain the direction of motion.
    //
    STDMETHOD_(FLOAT, EllipseFilledTouchPercent)(
        __in                        THIS_
        __in INT                    nCenterX,
        __in INT                    nCenterY, 
        __in FLOAT                  fRadiusX,
        __in FLOAT                  fRadiusY, 
        __inout D3DVECTOR*          pvecDirection
        ) PURE;

    //
    // Tests motion against an aribtrary bitmap. The bitmap must be a texture
    // in the D3DFMT_LIN_L8 format that represents a touch region. Pixels
    // with value of 255 represent areas to be tested. Returns TRUE if motion
    // detected.
    //
    STDMETHOD_(BOOL, RegionFilledTouch)(
        __in                        THIS_
        __inout BYTE*               pbyRegionMap,
        __inout D3DRECT*            prectRegion
        ) PURE;

    //
    // Tests motion against an aribtrary bitmap. The bitmap must be a texture
    // in the D3DFMT_LIN_L8 format that represents a touch region. Pixels
    // with value of 255 represent areas to be tested. Returns a percentage
    // of motion inside tested areas (0..1).
    //
    STDMETHOD_(FLOAT, RegionFilledTouchPercent)(
        __in                        THIS_
        __inout BYTE*               pbyRegionMap,
        __inout D3DRECT*            prectRegion
        ) PURE;

    //
    // Sets motion threshold (0..255). The lower the threshold the more motion
    // and potentially noise is detected.
    //
    STDMETHOD_(VOID, SetThreshold)(
        __in                        THIS_
        __in BYTE                   byThreshold
        ) PURE;

    //
    // Returns motion threshold
    //
    STDMETHOD_(BYTE, GetThreshold)(
        __in                        THIS
        ) PURE;

    //
    // Gets the frame retreived from the camera in D3DFMT_LE_LIN_YUY2 format.
    //
    STDMETHOD_(DWORD, GetNextLocalPreviewFrame)(
        __in                        THIS_
        __inout D3DLOCKED_RECT*     pBuffer
        ) PURE;

    //
    // Sets rectangle where live video will be rendered.
    //
    STDMETHOD_(VOID, SetLocalPreviewRect)(
        __in                        THIS_
        __inout D3DRECT*            pRect
        ) PURE;

    //
    // Render live video in local preview rectangle.
    //
    STDMETHOD_(VOID, RenderLocalPreview)(
        __in                        THIS
        ) PURE;


};

//------------------------------------------------------------------------------
// C Function Wrappers for XCAM Interfaces
//------------------------------------------------------------------------------

#pragma push_macro("VTBL")
#pragma push_macro("THIS")
#pragma push_macro("THIS_")

#undef  VTBL
#undef  THIS
#undef  THIS_

#if defined(__cplusplus) && !defined(CINTERFACE)
#define VTBL(p) (p)
#define THIS
#define THIS_
#else // defined(__cplusplus) && !defined(CINTERFACE)
#define VTBL(p) ((p)->lpVtbl)
#define THIS    pThis
#define THIS_   pThis,
#endif // defined(__cplusplus) && !defined(CINTERFACE)

FORCEINLINE
STDMETHODIMP_(ULONG)
IXCamVideo_AddRef(
    __in  PIXCAMVIDEO               pThis
    )
{
    return VTBL(pThis)->AddRef(THIS);
}

FORCEINLINE
STDMETHODIMP_(ULONG)
IXCamVideo_Release(
    __in  PIXCAMVIDEO               pThis
    )
{
    return VTBL(pThis)->Release(THIS);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamVideo_SetCaptureMode(
    __in  PIXCAMVIDEO               pThis,
    __in  XCAMRESOLUTION            Resolution,
    __in  XCAMFRAMERATE             Framerate,
    __inout_opt PXOVERLAPPED        pOverlapped
    )
{
    return VTBL(pThis)->SetCaptureMode(THIS_ Resolution, Framerate, pOverlapped);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamVideo_ReadFrame(
    __in  PIXCAMVIDEO               pThis,
    __inout D3DLOCKED_RECT         *pBuffer,
    __inout PXOVERLAPPED            pOverlapped
    )
{
    return VTBL(pThis)->ReadFrame(THIS_ pBuffer, pOverlapped);
}




FORCEINLINE
STDMETHODIMP_(ULONG)
IXCamStreamEngine_AddRef(
    __in  PIXCAMSTREAMENGINE        pThis
    )
{
    return VTBL(pThis)->AddRef(THIS);
}

FORCEINLINE
STDMETHODIMP_(ULONG)
IXCamStreamEngine_Release(
    __in  PIXCAMSTREAMENGINE        pThis
    )
{
    return VTBL(pThis)->Release(THIS);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_RegisterRemoteConsole(
    __in  PIXCAMSTREAMENGINE        pThis,
    __in  XNADDR                    RemoteAddress,
    __in_ecount(RemoteXUIDListLength) CONST XUID *RemoteXUIDList,
    __in  DWORD                     RemoteXUIDListLength,
    __in  BOOL                      SendVideoToConsole,
    __in  BOOL                      ReceiveVideoFromConsole
    )
{
    return VTBL(pThis)->RegisterRemoteConsole(THIS_ RemoteAddress, RemoteXUIDList, RemoteXUIDListLength, SendVideoToConsole, ReceiveVideoFromConsole);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_UnregisterRemoteConsole(
    __in  PIXCAMSTREAMENGINE        pThis,
    __in  XNADDR                    RemoteAddress
    )
{
    return VTBL(pThis)->UnregisterRemoteConsole(THIS_ RemoteAddress);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamStreamEngine_GetRemoteConsoles(
    __in     PIXCAMSTREAMENGINE     pThis,
    __out_ecount(*pNumAddresses) XNADDR *pRemoteAddresses,
    __inout PDWORD                  pNumAddresses
    )
{
    return VTBL(pThis)->GetRemoteConsoles(THIS_ pRemoteAddresses, pNumAddresses);
}

FORCEINLINE
STDMETHODIMP_(BOOL)
IXCamStreamEngine_IsRemoteConsoleDisallowed(
    __in     PIXCAMSTREAMENGINE     pThis,
    __in     XNADDR                 RemoteAddress
    )
{
    return VTBL(pThis)->IsRemoteConsoleDisallowed(THIS_ RemoteAddress);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_GetSlowestConnection(
    __in  PIXCAMSTREAMENGINE        pThis,
    __out  XNADDR                  *pRemoteAddress
    )
{
    return VTBL(pThis)->GetSlowestConnection(THIS_ pRemoteAddress);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_SubmitLocalVideoFrame(
    __in  PIXCAMSTREAMENGINE         pThis,
    __in  PBYTE                      pVideoFrame,
    __inout PXOVERLAPPED             pOverlapped
    )
{
    return VTBL(pThis)->SubmitLocalVideoFrame(THIS_ pVideoFrame, pOverlapped);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_GetNetworkData(
    __in     PIXCAMSTREAMENGINE     pThis,
    __in     XNADDR                 RemoteAddress,
    __out_bcount(*pBufferSize) PBYTE pBuffer,
    __inout PDWORD                  pBufferSize
    )
{
    return VTBL(pThis)->GetNetworkData(THIS_ RemoteAddress, pBuffer, pBufferSize);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_SubmitNetworkData(
    __in  PIXCAMSTREAMENGINE        pThis,
    __in  XNADDR                    RemoteAddress,
    __in_bcount(BufferSize) PBYTE   pBuffer,
    __in  DWORD                     BufferSize,
    __inout PXOVERLAPPED            pOverlapped
    )
{
    return VTBL(pThis)->SubmitNetworkData(THIS_ RemoteAddress, pBuffer, BufferSize, pOverlapped);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_SetRemoteConsoleRect(
    __in      PIXCAMSTREAMENGINE    pThis,
    __in      XNADDR                RemoteAddress,
    __in CONST D3DRECT             *pRect
    )
{
    return VTBL(pThis)->SetRemoteConsoleRect(THIS_ RemoteAddress, pRect);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_SetRemoteConsoleDefaultTexture(
    __in  PIXCAMSTREAMENGINE        pThis,
    __in  XNADDR                    RemoteAddress,
    __in  D3DTexture               *pDefaultTexture
    )
{
    return VTBL(pThis)->SetRemoteConsoleDefaultTexture(THIS_ RemoteAddress, pDefaultTexture);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_RenderRemoteConsole(
    __in  PIXCAMSTREAMENGINE        pThis,
    __in  XNADDR                    RemoteAddress
    )
{
    return VTBL(pThis)->RenderRemoteConsole(THIS_ RemoteAddress);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_GetNextRemoteConsoleFrame(
    __in     PIXCAMSTREAMENGINE     pThis,
    __in     XNADDR                 RemoteAddress,
    __inout D3DLOCKED_RECT         *pYBuffer,
    __inout D3DLOCKED_RECT         *pUBuffer,
    __inout D3DLOCKED_RECT         *pVBuffer
    )
{
    return VTBL(pThis)->GetNextRemoteConsoleFrame(THIS_ RemoteAddress, pYBuffer, pUBuffer, pVBuffer);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_GetNextRemoteConsoleFrameRGB(
    __in     PIXCAMSTREAMENGINE     pThis,
    __in     XNADDR                 RemoteAddress,
    __inout D3DLOCKED_RECT         *pBuffer
    )
{
    return VTBL(pThis)->GetNextRemoteConsoleFrameRGB(THIS_ RemoteAddress, pBuffer);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamStreamEngine_SetLocalPreviewRect(
    __in      PIXCAMSTREAMENGINE    pThis,
    __in CONST D3DRECT             *pRect
    )
{
    return VTBL(pThis)->SetLocalPreviewRect(THIS_ pRect);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamStreamEngine_RenderLocalPreview(
    __in  PIXCAMSTREAMENGINE        pThis
    )
{
    return VTBL(pThis)->RenderLocalPreview(THIS);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_GetNextLocalPreviewFrame(
    __in     PIXCAMSTREAMENGINE     pThis,
    __inout D3DLOCKED_RECT         *pBuffer
    )
{
    return VTBL(pThis)->GetNextLocalPreviewFrame(THIS_ pBuffer);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_GetNextLocalPreviewFrameRGB(
    __in     PIXCAMSTREAMENGINE     pThis,
    __inout D3DLOCKED_RECT         *pBuffer
    )
{
    return VTBL(pThis)->GetNextLocalPreviewFrameRGB(THIS_ pBuffer);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamStreamEngine_SetEncoderTargetBitrate(
    __in  PIXCAMSTREAMENGINE        pThis,
    __in  DWORD                     TargetBitrate
    )
{
    return VTBL(pThis)->SetEncoderTargetBitrate(THIS_ TargetBitrate);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_GetEncoderTargetBitrate(
    __in  PIXCAMSTREAMENGINE        pThis
    )
{
    return VTBL(pThis)->GetEncoderTargetBitrate(THIS);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamStreamEngine_GetEncoderActualBitrate(
    __in  PIXCAMSTREAMENGINE        pThis
    )
{
    return VTBL(pThis)->GetEncoderActualBitrate(THIS);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamStreamEngine_SetEncoderTargetFramerate(
    __in  PIXCAMSTREAMENGINE        pThis,
    __in  XCAMFRAMERATE             TargetFramerate
    )
{
    return VTBL(pThis)->SetEncoderTargetFramerate(THIS_ TargetFramerate);
}

FORCEINLINE
STDMETHODIMP_(XCAMFRAMERATE)
IXCamStreamEngine_GetEncoderTargetFramerate(
    __in  PIXCAMSTREAMENGINE        pThis
    )
{
    return VTBL(pThis)->GetEncoderTargetFramerate(THIS);
}

FORCEINLINE
STDMETHODIMP_(XCAMFRAMERATE)
IXCamStreamEngine_GetEncoderActualFramerate(
    __in  PIXCAMSTREAMENGINE        pThis
    )
{
    return VTBL(pThis)->GetEncoderActualFramerate(THIS);
}


FORCEINLINE
STDMETHODIMP_(ULONG)
IXCamMotionEngine_AddRef( 
    __in PIXCAMMOTIONENGINE         pThis
    )
{
    return VTBL(pThis)->AddRef(THIS);
}

FORCEINLINE
STDMETHODIMP_(ULONG)
IXCamMotionEngine_Release(
    __in PIXCAMMOTIONENGINE         pThis
    )
{
    return VTBL(pThis)->Release(THIS);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamMotionEngine_GenerateMotionMap(
    __in PIXCAMMOTIONENGINE         pThis,
    __inout D3DLOCKED_RECT*         pBufferA, 
    __inout D3DLOCKED_RECT*         pBufferB
    )
{
    return VTBL(pThis)->GenerateMotionMap(THIS_ pBufferA, pBufferB);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamMotionEngine_GetMotionMapData(
    __in PIXCAMMOTIONENGINE         pThis,
    __inout D3DLOCKED_RECT*         pBuffer
    )
{
    return VTBL(pThis)->GetMotionMapData(THIS_ pBuffer);
}

FORCEINLINE
STDMETHODIMP_(BOOL)
IXCamMotionEngine_RectFilledTouch(
    __in PIXCAMMOTIONENGINE         pThis,
    __inout D3DRECT*                pRect
    )
{
    return VTBL(pThis)->RectFilledTouch(THIS_ pRect);
}

FORCEINLINE
STDMETHODIMP_(FLOAT)
IXCamMotionEngine_RectFilledTouchPercent(
    __in PIXCAMMOTIONENGINE         pThis,
    __inout D3DRECT*                pRect
    )
{
    return VTBL(pThis)->RectFilledTouchPercent(THIS_ pRect);
}

FORCEINLINE
STDMETHODIMP_(BOOL)
IXCamMotionEngine_RectOutlineTouch(
    __in PIXCAMMOTIONENGINE         pThis,
    __inout D3DRECT*                pRect
    )
{
    return VTBL(pThis)->RectOutlineTouch(THIS_ pRect);
}

FORCEINLINE
STDMETHODIMP_(BOOL)
IXCamMotionEngine_EllipseOutlineTouch(
    __in PIXCAMMOTIONENGINE         pThis,
    __in INT                        nCenterX,
    __in INT                        nCenterY, 
    __in FLOAT                      fRadiusX,
    __in FLOAT                      fRadiusY, 
    __inout D3DVECTOR*              pvecDirection
    )
{
    return VTBL(pThis)->EllipseOutlineTouch(THIS_ nCenterX, nCenterY, fRadiusX, fRadiusY, pvecDirection);
}

FORCEINLINE
STDMETHODIMP_(BOOL)
IXCamMotionEngine_EllipseFilledTouch(
    __in PIXCAMMOTIONENGINE         pThis,
    __in INT                        nCenterX,
    __in INT                        nCenterY, 
    __in FLOAT                      fRadiusX,
    __in FLOAT                      fRadiusY, 
    __inout D3DVECTOR*              pvecDirection
    )
{
    return VTBL(pThis)->EllipseFilledTouch(THIS_ nCenterX, nCenterY, fRadiusX, fRadiusY, pvecDirection);
}

FORCEINLINE
STDMETHODIMP_(FLOAT)
IXCamMotionEngine_EllipseFilledTouchPercent(
    __in PIXCAMMOTIONENGINE         pThis,
    __in INT                        nCenterX,
    __in INT                        nCenterY, 
    __in FLOAT                      fRadiusX,
    __in FLOAT                      fRadiusY, 
    __inout D3DVECTOR*              pvecDirection
    )
{
    return VTBL(pThis)->EllipseFilledTouchPercent(THIS_ nCenterX, nCenterY, fRadiusX, fRadiusY, pvecDirection);
}

FORCEINLINE
STDMETHODIMP_(BOOL)
IXCamMotionEngine_RegionFilledTouch(
    __in PIXCAMMOTIONENGINE         pThis,
    __inout BYTE*                   pbyRegionMap,
    __inout D3DRECT*                prectRegion
    )
{
    return VTBL(pThis)->RegionFilledTouch(THIS_ pbyRegionMap, prectRegion);
}

FORCEINLINE
STDMETHODIMP_(FLOAT)
IXCamMotionEngine_RegionFilledTouchPercent(
    __in PIXCAMMOTIONENGINE         pThis,
    __inout BYTE*                   pbyRegionMap,
    __inout D3DRECT*                prectRegion
    )
{
    return VTBL(pThis)->RegionFilledTouchPercent(THIS_ pbyRegionMap, prectRegion);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamMotionEngine_SetThreshold(
    __in PIXCAMMOTIONENGINE         pThis,
    __in BYTE                       byThreshold
    )
{
    return VTBL(pThis)->SetThreshold(THIS_ byThreshold);
}

FORCEINLINE
STDMETHODIMP_(BYTE)
IXCamMotionEngine_GetThreshold(
    __in PIXCAMMOTIONENGINE         pThis
    )
{
    return VTBL(pThis)->GetThreshold(THIS);
}

FORCEINLINE
STDMETHODIMP_(DWORD)
IXCamMotionEngine_GetNextLocalPreviewFrame(
    __in PIXCAMMOTIONENGINE         pThis,
    __inout D3DLOCKED_RECT*         pBuffer
    )
{
    return VTBL(pThis)->GetNextLocalPreviewFrame(THIS_ pBuffer);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamMotionEngine_SetLocalPreviewRect(
    __in PIXCAMMOTIONENGINE         pThis,
    __inout D3DRECT*                pRect
    )
{
    return VTBL(pThis)->SetLocalPreviewRect(THIS_ pRect);
}

FORCEINLINE
STDMETHODIMP_(VOID)
IXCamMotionEngine_RenderLocalPreview(
    __in PIXCAMMOTIONENGINE         pThis
    )
{
    return VTBL(pThis)->RenderLocalPreview(THIS);
}



#pragma pop_macro("VTBL")
#pragma pop_macro("THIS")
#pragma pop_macro("THIS_")

#ifdef __cplusplus
}
#endif

#endif // __XCAM_H__
