/************************************************************************
*                                                                       *
*   Xconfig.h -- Public Configuration enums                             *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/
#ifndef _XCONFIG_H_
#define _XCONFIG_H_



#ifdef __cplusplus
extern "C" {
#endif



//
// Public closed captioning settings
//

// On state
//
typedef enum _XCLOSEDCAPTIONING_STATE
{
    XCLOSEDCAPTIONING_DISABLED,
    XCLOSEDCAPTIONING_ENABLED
} XCLOSEDCAPTIONING_STATE;


// Opacity
//
typedef enum _XCLOSEDCAPTIONING_OPACITY
{
    XCLOSEDCAPTIONING_OPACITY_OPAQUE,
    XCLOSEDCAPTIONING_OPACITY_SEMI,
    XCLOSEDCAPTIONING_OPACITY_TRANSPARENT
} XCLOSEDCAPTIONING_OPACITY;

// Font size
//
typedef enum _XCLOSEDCAPTIONING_FONTSIZE
{
    XCLOSEDCAPTIONING_FONTSIZE_SMALL,
    XCLOSEDCAPTIONING_FONTSIZE_MEDIUM,
    XCLOSEDCAPTIONING_FONTSIZE_LARGE
} XCLOSEDCAPTIONING_FONTSIZE;

// Font style
//
typedef enum _XCLOSEDCAPTIONING_FONTSTYLE
{
    XCLOSEDCAPTIONING_FONTSTYLE_DEFAULT,
    XCLOSEDCAPTIONING_FONTSTYLE_MONOSERIF,
    XCLOSEDCAPTIONING_FONTSTYLE_PROPSERIF,
    XCLOSEDCAPTIONING_FONTSTYLE_MONOSANSSERIF,
    XCLOSEDCAPTIONING_FONTSTYLE_PROPSANSSERIF,
    XCLOSEDCAPTIONING_FONTSTYLE_CASUAL,
    XCLOSEDCAPTIONING_FONTSTYLE_CURSIVE,
    XCLOSEDCAPTIONING_FONTSTYLE_SMALLCAPS
} XCLOSEDCAPTIONING_FONTSTYLE;

// Character edge attributes
//
typedef enum _XCLOSEDCAPTIONING_EDGE_ATTRIBUTE
{
    XCLOSEDCAPTIONING_EDGE_RAISED,
    XCLOSEDCAPTIONING_EDGE_DEPRESSED,
    XCLOSEDCAPTIONING_EDGE_DROPSHADOW,
    XCLOSEDCAPTIONING_EDGE_UNIFORM,
    XCLOSEDCAPTIONING_EDGE_NONE
} XCLOSEDCAPTIONING_EDGE_ATTRIBUTE;

// Preferred language
//
typedef enum _XCLOSEDCAPTIONING_LANGUAGE
{
    XCLOSEDCAPTIONING_LANGUAGE_ENGLISH,
    XCLOSEDCAPTIONING_LANGUAGE_SPANISH,
    XCLOSEDCAPTIONING_LANGUAGE_EASYREADER
} XCLOSEDCAPTIONING_LANGUAGE;

// Font, background and window color
//
typedef enum _XCLOSEDCAPTIONING_COLOR
{
    XCLOSEDCAPTIONING_COLOR_BLACK,
    XCLOSEDCAPTIONING_COLOR_WHITE,
    XCLOSEDCAPTIONING_COLOR_RED,
    XCLOSEDCAPTIONING_COLOR_GREEN,
    XCLOSEDCAPTIONING_COLOR_BLUE,
    XCLOSEDCAPTIONING_COLOR_YELLOW,
    XCLOSEDCAPTIONING_COLOR_MAGENTA,
    XCLOSEDCAPTIONING_COLOR_CYAN
} XCLOSEDCAPTIONING_COLOR;

#if defined(_M_PPCBE)
#pragma bitfield_order(push, lsb_to_msb)
#endif

typedef struct _XCLOSEDCAPTIONING_SETTINGS
{
    DWORD   UseDefaults         : 1; // 0.1 (Yes/No)
    DWORD   EdgeAttribute       : 3; // 0.4 (none, raised, depressed, uniform, drop shadow)
    DWORD   FontColor           : 4; // 1.0 (8 required colors, added buffer for possible increase)
    DWORD   FontSize            : 2; // 1.2 (small, medium, large)
    DWORD   FontStyle           : 4; // 1.6 (8 required styles, added buffer for possible increase)
    DWORD   FontOpacity         : 2; // 2.0 (opaque, semi, transparent)
    DWORD   BackgroundColor     : 4; // 2.4 (8 required colors, added buffer for possible increase)
    DWORD   BackgroundOpacity   : 2; // 2.6 (opaque, semi, transparent)
    DWORD   WindowOpacity       : 2; // 3.0 (opaque, semi, transparent)
    DWORD   WindowColor         : 4; // 3.4 (8 required colors, added buffer for possible increase)
    DWORD   PreferredLanguage   : 4; // 4.0 (Currently - English, Spanish, Easy Reader)
} XCLOSEDCAPTIONING_SETTINGS, *PXCLOSEDCAPTIONING_SETTINGS;    // 4

#if defined(_M_PPCBE)
#pragma bitfield_order(pop)
#endif


#define XC_LANGUAGE_ENGLISH             0x01
#define XC_LANGUAGE_JAPANESE            0x02
#define XC_LANGUAGE_GERMAN              0x03
#define XC_LANGUAGE_FRENCH              0x04
#define XC_LANGUAGE_SPANISH             0x05
#define XC_LANGUAGE_ITALIAN             0x06
#define XC_LANGUAGE_KOREAN              0x07
#define XC_LANGUAGE_TCHINESE            0x08
#define XC_LANGUAGE_PORTUGUESE          0x09
#define XC_LANGUAGE_SCHINESE_DEPRECATED 0x0A    // Not a supported language before 2012. This constant is deprecated but remains here for backward compatibility.
#define XC_LANGUAGE_POLISH              0x0B
#define XC_LANGUAGE_RUSSIAN             0x0C
#define XC_LANGUAGE_SWEDISH             0x0D
#define XC_LANGUAGE_TURKISH             0x0E
#define XC_LANGUAGE_BNORWEGIAN          0x0F
#define XC_LANGUAGE_DUTCH               0x10
#define XC_LANGUAGE_SCHINESE            0x11


#define XC_LOCALE_AUSTRALIA             1
#define XC_LOCALE_AUSTRIA               2
#define XC_LOCALE_BELGIUM               3
#define XC_LOCALE_BRAZIL                4
#define XC_LOCALE_CANADA                5
#define XC_LOCALE_CHILE                 6
#define XC_LOCALE_CHINA                 7
#define XC_LOCALE_COLOMBIA              8
#define XC_LOCALE_CZECH_REPUBLIC        9
#define XC_LOCALE_DENMARK               10
#define XC_LOCALE_FINLAND               11
#define XC_LOCALE_FRANCE                12
#define XC_LOCALE_GERMANY               13
#define XC_LOCALE_GREECE                14
#define XC_LOCALE_HONG_KONG             15
#define XC_LOCALE_HUNGARY               16
#define XC_LOCALE_INDIA                 17
#define XC_LOCALE_IRELAND               18
#define XC_LOCALE_ITALY                 19
#define XC_LOCALE_JAPAN                 20
#define XC_LOCALE_KOREA                 21
#define XC_LOCALE_MEXICO                22
#define XC_LOCALE_NETHERLANDS           23
#define XC_LOCALE_NEW_ZEALAND           24
#define XC_LOCALE_NORWAY                25
#define XC_LOCALE_POLAND                26
#define XC_LOCALE_PORTUGAL              27
#define XC_LOCALE_SINGAPORE             28
#define XC_LOCALE_SLOVAK_REPUBLIC       29
#define XC_LOCALE_SOUTH_AFRICA          30
#define XC_LOCALE_SPAIN                 31
#define XC_LOCALE_SWEDEN                32
#define XC_LOCALE_SWITZERLAND           33
#define XC_LOCALE_TAIWAN                34
#define XC_LOCALE_GREAT_BRITAIN         35
#define XC_LOCALE_UNITED_STATES         36
#define XC_LOCALE_RUSSIAN_FEDERATION    37
#define XC_LOCALE_WORLD_WIDE            38
#define XC_LOCALE_TURKEY                39
#define XC_LOCALE_ARGENTINA             40
#define XC_LOCALE_SAUDI_ARABIA          41
#define XC_LOCALE_ISRAEL                42
#define XC_LOCALE_UNITED_ARAB_EMIRATES  43



// Game systems - 63 possible rating systems + 1 unrated
//
#define XC_PC_GAME_RATING_SYS_DEFAULT   XC_PC_GAME_RATING_SYS_ESRB
#define XC_PC_GAME_RATING_SYS_NONE      0x3f

//
// These defines need to be kept in sync with the server.
// Do not change their values without making sure the server
// has been updated.
//
#define XC_PC_GAME_RATING_SYS_ESRB      0x00
#define XC_PC_GAME_RATING_SYS_PEGI      0x01
#define XC_PC_GAME_RATING_SYS_PEGI_FI   0x02
#define XC_PC_GAME_RATING_SYS_PEGI_PT   0x03
#define XC_PC_GAME_RATING_SYS_PEGIBBFC  0x04
#define XC_PC_GAME_RATING_SYS_CERO      0x05
#define XC_PC_GAME_RATING_SYS_USK       0x06
#define XC_PC_GAME_RATING_SYS_OFLC_AU   0x07
#define XC_PC_GAME_RATING_SYS_OFLC_NZ   0x08
#define XC_PC_GAME_RATING_SYS_KMRB      0x09
#define XC_PC_GAME_RATING_SYS_BRAZIL    0x0a
#define XC_PC_GAME_RATING_SYS_FPB       0x0b
#define XC_PC_GAME_RATING_SYS_TAIWAN    0x0c
#define XC_PC_GAME_RATING_SYS_SINGAPORE 0x0d

//
// Movie rating systems
//
#define XC_PC_MOVIE_RATING_SYS_DEFAULT      XC_PC_MOVIE_RATING_SYS_MPAA
#define XC_PC_MOVIE_RATING_SYS_NONE         7

#define XC_PC_MOVIE_RATING_SYS_MPAA         0   // USA and Canada
#define XC_PC_MOVIE_RATING_SYS_JAPAN        1
#define XC_PC_MOVIE_RATING_SYS_UK           2
#define XC_PC_MOVIE_RATING_SYS_IRELAND      3
#define XC_PC_MOVIE_RATING_SYS_FRANCE       4
#define XC_PC_MOVIE_RATING_SYS_GERMANY      5
#define XC_PC_MOVIE_RATING_SYS_AUSTRALIA    6
#define XC_PC_MOVIE_RATING_SYS_NETHERLANDS  8
#define XC_PC_MOVIE_RATING_SYS_AUSTRIA      9
#define XC_PC_MOVIE_RATING_SYS_NEW_ZEALAND  10
#define XC_PC_MOVIE_RATING_SYS_SWEDEN       11
#define XC_PC_MOVIE_RATING_SYS_NORWAY       12
#define XC_PC_MOVIE_RATING_SYS_DENMARK      13
#define XC_PC_MOVIE_RATING_SYS_FINLAND      14
#define XC_PC_MOVIE_RATING_SYS_HONG_KONG    15
#define XC_PC_MOVIE_RATING_SYS_SINGAPORE    16
#define XC_PC_MOVIE_RATING_SYS_CANADA       17
#define XC_PC_MOVIE_RATING_SYS_SPAIN        18
#define XC_PC_MOVIE_RATING_SYS_ITALY        19
#define XC_PC_MOVIE_RATING_SYS_MEXICO       20
#define XC_PC_MOVIE_RATING_SYS_TURKEY       21
#define XC_PC_MOVIE_RATING_SYS_SLOVAK_REPUBLIC     22
#define XC_PC_MOVIE_RATING_SYS_BRAZIL       23

// TV rating system - 13 possible rating systems + 1 unrated
//
//
#define XC_PC_TV_RATING_SYS_USA         0
#define XC_PC_TV_RATING_SYS_CANADA      1
#define XC_PC_TV_RATING_SYS_MEXICO      2
#define XC_PC_TV_RATING_SYS_FRANCE      3
#define XC_PC_TV_RATING_SYS_AUSTRALIA   4
#define XC_PC_TV_RATING_SYS_BRAZIL      5
#define XC_PC_TV_RATING_SYS_SPAIN       6
#define XC_PC_TV_RATING_SYS_NONE        7
#define XC_PC_TV_RATING_SYS_ITALY       8
#define XC_PC_TV_RATING_SYS_FINLAND     9
#define XC_PC_TV_RATING_SYS_NEW_ZEALAND 10
#define XC_PC_TV_RATING_SYS_TURKEY      11
#define XC_PC_TV_RATING_SYS_SLOVAK_REPUBLIC    12
#define XC_PC_TV_RATING_SYS_NETHERLANDS    13

//
// Parental controls ratings
//

// Content types
//
#define XC_PC_TYPE_GAME      0x00
#define XC_PC_TYPE_MOVIE     0x01
#define XC_PC_TYPE_TV        0x02
#define XC_PC_TYPE_EXPLICIT  0x03
#define XC_PC_TYPE_UNRATED   0x04


// Default unrated value (upper limit for content rating)
//
#define XC_PC_UNRATED                   0xff

// Access levels used by titles when querying restrictions for content
//
#define XC_PC_ACCESS_PREVIEW               1
#define XC_PC_ACCESS_PLAY                  2
#define XC_PC_ACCESS_BROWSE                3

// Public content definitions (replicates the internal video definitions)
//

// Explicit content rating system - 7 possible rating systems + 1 unrated
//
#define XC_PC_EXPLICIT_CONTENT_RATING_SYS_STD         0
#define XC_PC_EXPLICIT_CONTENT_RATING_SYS_NONE        7

// Unrated content rating system - 7 possible rating systems + 1 unrated
//
#define XC_PC_UNRATED_CONTENT_RATING_SYS_STD         0
#define XC_PC_UNRATED_CONTENT_RATING_SYS_NONE        7

// US Explicit Content
//
#define XC_PC_STD_EXPLICIT_CONTENT_DISALLOW       0x00                          // Disallow videos with the explicit tag
#define XC_PC_STD_EXPLICIT_CONTENT_ALLOW          XC_PC_UNRATED                 // Allow - Binary settings should use XC_PC_UNRATED as least restrictive

// US Unrated Content
//
#define XC_PC_STD_UNRATED_CONTENT_DISALLOW        0x00                          // Disallow unrated videos
#define XC_PC_STD_UNRATED_CONTENT_ALLOW           XC_PC_UNRATED                 // Allow - Binary settings should use XC_PC_UNRATED as least restrictive


// Console restriction
//
#define XC_PC_RESTRICTION_FLAG_ENABLED      0x80

// US TV Ratings
//
#define XC_PC_USTV_Y                    0x00            // TV-Y
#define XC_PC_USTV_Y7                   0x02            // TV-Y7
#define XC_PC_USTV_Y7FV                 0x04            // TV-YFV
#define XC_PC_USTV_G                    0x06            // TV-G
#define XC_PC_USTV_PG                   0x08            // TV-PG
#define XC_PC_USTV_14                   0x0a            // TV-14
#define XC_PC_USTV_MATURE               0x0c            // TV-M

// Canadian TV Ratings
//
#define XC_PC_CANADATV_E                0x00            // E
#define XC_PC_CANADATV_C                0x02            // C
#define XC_PC_CANADATV_C8               0x06            // C8+
#define XC_PC_CANADATV_G                0x08            // G
#define XC_PC_CANADATV_PG               0x0a            // PG
#define XC_PC_CANADATV_14               0x0c            // 14+
#define XC_PC_CANADATV_18               0x0e            // 18+

// Mexico TV Ratings
//
#define XC_PC_MEXICOTV_AA            0x00
#define XC_PC_MEXICOTV_A             0x02
#define XC_PC_MEXICOTV_B             0x06
#define XC_PC_MEXICOTV_B15           0x08
#define XC_PC_MEXICOTV_C             0x0a
#define XC_PC_MEXICOTV_D             0x0e

// France TV Ratings
//
#define XC_PC_FRANCETV_10            0x00
#define XC_PC_FRANCETV_12            0x02
#define XC_PC_FRANCETV_16            0x06
#define XC_PC_FRANCETV_18            0x08

// Australia TV Ratings
//
#define XC_PC_AUSTRALIATV_P            0x00
#define XC_PC_AUSTRALIATV_C            0x02
#define XC_PC_AUSTRALIATV_G            0x06
#define XC_PC_AUSTRALIATV_PG           0x08
#define XC_PC_AUSTRALIATV_M            0x0a
#define XC_PC_AUSTRALIATV_MA15         0x0c
#define XC_PC_AUSTRALIATV_AV15         0x0e
#define XC_PC_AUSTRALIATV_R18          0x10

// Brazil TV Ratings
//
#define XC_PC_BRAZILTV_ER                  0x00
#define XC_PC_BRAZILTV_L                   0x10
#define XC_PC_BRAZILTV_10                  0x20
#define XC_PC_BRAZILTV_12                  0x30
#define XC_PC_BRAZILTV_14                  0x40
#define XC_PC_BRAZILTV_16                  0x50
#define XC_PC_BRAZILTV_18                  0x60

// Spain TV Ratings
//
#define XC_PC_SPAINTV_MINOR         0
#define XC_PC_SPAINTV_GP            10
#define XC_PC_SPAINTV_7             20
#define XC_PC_SPAINTV_12            25
#define XC_PC_SPAINTV_16            35
#define XC_PC_SPAINTV_18            40
#define XC_PC_SPAINTV_X             50

// Italy TV Ratings
//
#define XC_PC_ITALYTV_MINOR             0
#define XC_PC_ITALYTV_14                10
#define XC_PC_ITALYTV_ADULT             20

// Finland TV Ratings
//
#define XC_PC_FINLANDTV_RATING_S        0
#define XC_PC_FINLANDTV_RATING_7        10
#define XC_PC_FINLANDTV_RATING_12       20
#define XC_PC_FINLANDTV_RATING_16       30
#define XC_PC_FINLANDTV_RATING_18       40

// New Zealand TV Ratings
//
#define XC_PC_NEW_ZEALANDTV_RATING_G    0
#define XC_PC_NEW_ZEALANDTV_RATING_PG   10
#define XC_PC_NEW_ZEALANDTV_RATING_M    20
#define XC_PC_NEW_ZEALANDTV_RATING_R16  30
#define XC_PC_NEW_ZEALANDTV_RATING_R18  40

// Turkey TV Ratings
//
#define XC_PC_TURKEYTV_RATING_G         0
#define XC_PC_TURKEYTV_RATING_7PLUS     10
#define XC_PC_TURKEYTV_RATING_7A        20      // Obsolete
#define XC_PC_TURKEYTV_RATING_13PLUS    30
#define XC_PC_TURKEYTV_RATING_13A       40      // Obsolete
#define XC_PC_TURKEYTV_RATING_15PLUS    50      // Obsolete
#define XC_PC_TURKEYTV_RATING_15A       60      // Obsolete
#define XC_PC_TURKEYTV_RATING_18PLUS    70

// Slovak Republic TV Ratings
//
#define XC_PC_SLOVAK_REPUBLICTV_RATING_U        0
#define XC_PC_SLOVAK_REPUBLICTV_RATING_7        10
#define XC_PC_SLOVAK_REPUBLICTV_RATING_7PLUS    20
#define XC_PC_SLOVAK_REPUBLICTV_RATING_12       30
#define XC_PC_SLOVAK_REPUBLICTV_RATING_12PLUS   40
#define XC_PC_SLOVAK_REPUBLICTV_RATING_15       50
#define XC_PC_SLOVAK_REPUBLICTV_RATING_18       60

// Netherlands TV Ratings
//
#define XC_PC_NETHERLANDSTV_RATING_AL   0
#define XC_PC_NETHERLANDSTV_RATING_6    10
#define XC_PC_NETHERLANDSTV_RATING_9    20
#define XC_PC_NETHERLANDSTV_RATING_12   30
#define XC_PC_NETHERLANDSTV_RATING_16   40

// ESRB
//
#define XC_PC_ESRB_EARLY_CHILDHOOD      0x00            // 3+
#define XC_PC_ESRB_EVERYONE             0x02            // 6+
#define XC_PC_ESRB_E10                  0x04            // 10+
#define XC_PC_ESRB_TEEN                 0x06            // 13+
#define XC_PC_ESRB_MATURE               0x08            // 17+
#define XC_PC_ESRB_ADULTS_ONLY          0x0A            // Adults only
#define XC_PC_ESRB_RP                   XC_PC_UNRATED   // Rating pending

// PEGI
//
#define XC_PC_PEGI_3_YEARS_PLUS         0x00
#define XC_PC_PEGI_4_YEARS_PLUS         0x01
#define XC_PC_PEGI_6_YEARS_PLUS         0x03
#define XC_PC_PEGI_7_YEARS_PLUS         0x04
#define XC_PC_PEGI_11_YEARS_PLUS        0x08
#define XC_PC_PEGI_12_YEARS_PLUS        0x09
#define XC_PC_PEGI_15_YEARS_PLUS        0x0c
#define XC_PC_PEGI_16_YEARS_PLUS        0x0d
#define XC_PC_PEGI_18_YEARS_PLUS        0x0e

// PEGI/BBFC
//
#define XC_PC_PEGIBBFC_3_YEARS_PLUS     0x00
#define XC_PC_PEGIBBFC_4_YEARS_PLUS     0x01
#define XC_PC_PEGIBBFC_7_YEARS_PLUS     0x04
#define XC_PC_PEGIBBFC_8_YEARS_PLUS     0x05
#define XC_PC_PEGIBBFC_12_YEARS_PLUS    0x09
#define XC_PC_PEGIBBFC_15_YEARS_PLUS    0x0c
#define XC_PC_PEGIBBFC_16_YEARS_PLUS    0x0d
#define XC_PC_PEGIBBFC_18_YEARS_PLUS    0x0e

// CERO
//
#define XC_PC_CERO_CHILDREN             0x00
#define XC_PC_CERO_12_YEARS_PLUS        0x02
#define XC_PC_CERO_15_YEARS_PLUS        0x04
#define XC_PC_CERO_18_YEARS_PLUS        0x06
#define XC_PC_CERO_RP                   XC_PC_UNRATED   // Rating pending

// CERO amended. Adopted in Japan on 3/1/2006
//
#define XC_PC_CERO_A_PLUS               XC_PC_CERO_CHILDREN         // 0+
#define XC_PC_CERO_B_PLUS               XC_PC_CERO_12_YEARS_PLUS    // 12+
#define XC_PC_CERO_C_PLUS               XC_PC_CERO_15_YEARS_PLUS    // 15+
#define XC_PC_CERO_D_PLUS               XC_PC_CERO_18_YEARS_PLUS    // 17+ NB: previous system 18+ maps to new system 17+
#define XC_PC_CERO_Z_PLUS               0x08                        // 18+

// USK
//
#define XC_PC_USK_ALL_AGES              0x00
#define XC_PC_USK_6_YEARS_PLUS          0x02
#define XC_PC_USK_12_YEARS_PLUS         0x04
#define XC_PC_USK_16_YEARS_PLUS         0x06
#define XC_PC_USK_NO_YOUTH              0x08
#define XC_PC_USK_RP                    XC_PC_UNRATED   // Rating pending

// KMRB
//
#define XC_PC_KMRB_ALL                  0x00
#define XC_PC_KMRB_12_YEARS_PLUS        0x02
#define XC_PC_KMRB_15_YEARS_PLUS        0x04
#define XC_PC_KMRB_18_YEARS_PLUS        0x06

// OFLC Australia
//
#define XC_PC_OFLC_AU_EXEMPT            0x00    // Same as All/G
#define XC_PC_OFLC_AU_ALL               0x00
#define XC_PC_OFLC_AU_G8_PLUS           0x02    // Obsolete, use PG
#define XC_PC_OFLC_AU_PG                0x03
#define XC_PC_OFLC_AU_M15_PLUS          0x04    // Obsolete, use M
#define XC_PC_OFLC_AU_M                 0x05
#define XC_PC_OFLC_AU_MA15_PLUS         0x06
#define XC_PC_OFLC_AU_REFUSED           0x08    // Obsolete, use R18_PLUS
#define XC_PC_OFLC_AU_R18_PLUS          0x09
#define XC_PC_OFLC_AU_X18_PLUS          0x0f    // Not used

// OFLC New Zealand
//
#define XC_PC_OFLC_NZ_ALL               0x00
#define XC_PC_OFLC_NZ_G8_PLUS           0x02    // Obsolete, use PG
#define XC_PC_OFLC_NZ_PG                0x02
#define XC_PC_OFLC_NZ_M15_PLUS          0x04    // Obsolete, use M
#define XC_PC_OFLC_NZ_R13               0x04
#define XC_PC_OFLC_NZ_MA15_PLUS         0x06    // Obsolete, use R15
#define XC_PC_OFLC_NZ_R15               0x06
#define XC_PC_OFLC_NZ_REFUSED           0x08    // Obsolete, use R
#define XC_PC_OFLC_NZ_M                 0x10
#define XC_PC_OFLC_NZ_R16               0x20
#define XC_PC_OFLC_NZ_R18               0x30
#define XC_PC_OFLC_NZ_R                 0x40

// FPB South Africa
//
#define XC_PC_FPB_A                     0x00
#define XC_PC_FPB_PG                    0x06    // Don't use - this is a movie rating level incorrectly listed as a game rating.
#define XC_PC_FPB_10                    0x07
#define XC_PC_FPB_PG_CURRENT            0x08    // This is the PG rating that should be used going forward (instead of XC_PC_FPB_PG or XC_PC_FPB_10)
#define XC_PC_FPB_13                    0x0a
#define XC_PC_FPB_16                    0x0d
#define XC_PC_FPB_18                    0x0e

// DJCTQ Brazil
//
#define XC_PC_BRAZIL_ER                  0x00
#define XC_PC_BRAZIL_L                   0x10
#define XC_PC_BRAZIL_10                  0x20
#define XC_PC_BRAZIL_12                  0x30
#define XC_PC_BRAZIL_14                  0x40
#define XC_PC_BRAZIL_16                  0x50
#define XC_PC_BRAZIL_18                  0x60

// Taiwan
//
#define XC_PC_TAIWAN_GENERAL             0x01
#define XC_PC_TAIWAN_PROTECTED           0x02
#define XC_PC_TAIWAN_PG                  0x03   // Obsolete, replaced by PG12 in Fall 2012
#define XC_PC_TAIWAN_PG12                0x03
#define XC_PC_TAIWAN_PG15                0x05
#define XC_PC_TAIWAN_RESTRICTED          0x04

// Singapore
//
#define XC_PC_SINGAPORE_GENERAL          0x01
#define XC_PC_SINGAPORE_AGE_ADVISORY     0x02
#define XC_PC_SINGAPORE_MATURE_18        0x03

// MPAA
//
#define XC_PC_MPAA_G                    0x01
#define XC_PC_MPAA_PG                   0x03
#define XC_PC_MPAA_PG13                 0x04
#define XC_PC_MPAA_R                    0x06
#define XC_PC_MPAA_NC17                 0x07

// Japan Movie Ratings
//
#define XC_PC_JAPAN_MOVIE_RATING_1      0x0
#define XC_PC_JAPAN_MOVIE_RATING_2      0x1
#define XC_PC_JAPAN_MOVIE_RATING_3      0x2
#define XC_PC_JAPAN_MOVIE_RATING_4      0x3
#define XC_PC_JAPAN_MOVIE_RATING_5      0x4
#define XC_PC_JAPAN_MOVIE_RATING_6      0x5
#define XC_PC_JAPAN_MOVIE_RATING_7      0x6
#define XC_PC_JAPAN_MOVIE_RATING_8      0x7

// UK Movie Ratings
//
#define XC_PC_UK_MOVIE_RATING_UC        0
#define XC_PC_UK_MOVIE_RATING_U         10
#define XC_PC_UK_MOVIE_RATING_PG        20
#define XC_PC_UK_MOVIE_RATING_12        30
#define XC_PC_UK_MOVIE_RATING_15        40
#define XC_PC_UK_MOVIE_RATING_18        50
#define XC_PC_UK_MOVIE_RATING_R18       60

// Ireland Movie Ratings
//
#define XC_PC_IRELAND_MOVIE_RATING_G    0
#define XC_PC_IRELAND_MOVIE_RATING_PG   10
#define XC_PC_IRELAND_MOVIE_RATING_12A  20
#define XC_PC_IRELAND_MOVIE_RATING_15A  30
#define XC_PC_IRELAND_MOVIE_RATING_16   40
#define XC_PC_IRELAND_MOVIE_RATING_18   50

// France Movie Ratings
//
#define XC_PC_FRANCE_MOVIE_RATING_U     0
#define XC_PC_FRANCE_MOVIE_RATING_12    10
#define XC_PC_FRANCE_MOVIE_RATING_16    20
#define XC_PC_FRANCE_MOVIE_RATING_18    30

// Germany Movie Ratings
//
#define XC_PC_GERMANY_MOVIE_RATING_FSK0     0
#define XC_PC_GERMANY_MOVIE_RATING_FSK6     10
#define XC_PC_GERMANY_MOVIE_RATING_FSK12    20
#define XC_PC_GERMANY_MOVIE_RATING_FSK16    30
#define XC_PC_GERMANY_MOVIE_RATING_FSK18    40
#define XC_PC_GERMANY_MOVIE_RATING_SPIO     50

// Australia Movie Ratings
//
#define XC_PC_AUSTRALIA_MOVIE_RATING_G      0
#define XC_PC_AUSTRALIA_MOVIE_RATING_PG     10
#define XC_PC_AUSTRALIA_MOVIE_RATING_M      20
#define XC_PC_AUSTRALIA_MOVIE_RATING_MA15   30
#define XC_PC_AUSTRALIA_MOVIE_RATING_R18    40
#define XC_PC_AUSTRALIA_MOVIE_RATING_E      50

// Netherlands Movie Ratings
//
#define XC_PC_NETHERLANDS_MOVIE_RATING_AL   0
#define XC_PC_NETHERLANDS_MOVIE_RATING_6    10
#define XC_PC_NETHERLANDS_MOVIE_RATING_9    20
#define XC_PC_NETHERLANDS_MOVIE_RATING_12   30
#define XC_PC_NETHERLANDS_MOVIE_RATING_16   40

// Austria Movie Ratings
//
#define XC_PC_AUSTRIA_MOVIE_RATING_0        0
#define XC_PC_AUSTRIA_MOVIE_RATING_6        10
#define XC_PC_AUSTRIA_MOVIE_RATING_10       20
#define XC_PC_AUSTRIA_MOVIE_RATING_12       30
#define XC_PC_AUSTRIA_MOVIE_RATING_14       40
#define XC_PC_AUSTRIA_MOVIE_RATING_16       50
#define XC_PC_AUSTRIA_MOVIE_RATING_18       60

// New Zealand Movie Ratings
//
#define XC_PC_NEW_ZEALAND_MOVIE_RATING_G    0
#define XC_PC_NEW_ZEALAND_MOVIE_RATING_PG   10
#define XC_PC_NEW_ZEALAND_MOVIE_RATING_R13  20
#define XC_PC_NEW_ZEALAND_MOVIE_RATING_R15  30
#define XC_PC_NEW_ZEALAND_MOVIE_RATING_R16  40
#define XC_PC_NEW_ZEALAND_MOVIE_RATING_M    50
#define XC_PC_NEW_ZEALAND_MOVIE_RATING_R18  60
#define XC_PC_NEW_ZEALAND_MOVIE_RATING_R    70

// Sweden Movie Ratings
//
#define XC_PC_SWEDEN_MOVIE_RATING_BTL       0
#define XC_PC_SWEDEN_MOVIE_RATING_7         10
#define XC_PC_SWEDEN_MOVIE_RATING_11        20
#define XC_PC_SWEDEN_MOVIE_RATING_15        30

// Norway Movie Ratings
//
#define XC_PC_NORWAY_MOVIE_RATING_A         0
#define XC_PC_NORWAY_MOVIE_RATING_7         10
#define XC_PC_NORWAY_MOVIE_RATING_11        20
#define XC_PC_NORWAY_MOVIE_RATING_15        30
#define XC_PC_NORWAY_MOVIE_RATING_18        40

// Denmark Movie Ratings
//
#define XC_PC_DENMARK_MOVIE_RATING_A        0
#define XC_PC_DENMARK_MOVIE_RATING_7        10
#define XC_PC_DENMARK_MOVIE_RATING_11       20
#define XC_PC_DENMARK_MOVIE_RATING_15       30

// Finland Movie Ratings
//
#define XC_PC_FINLAND_MOVIE_RATING_S        0
#define XC_PC_FINLAND_MOVIE_RATING_7       20
#define XC_PC_FINLAND_MOVIE_RATING_12      35
#define XC_PC_FINLAND_MOVIE_RATING_16      55
#define XC_PC_FINLAND_MOVIE_RATING_18      60

// Canadian Movie Ratings
//
#define XC_PC_CANADA_MOVIE_RATING_G         0
#define XC_PC_CANADA_MOVIE_RATING_PG        10
#define XC_PC_CANADA_MOVIE_RATING_14        20
#define XC_PC_CANADA_MOVIE_RATING_18        30
#define XC_PC_CANADA_MOVIE_RATING_R         40
#define XC_PC_CANADA_MOVIE_RATING_E         50

// Spain Movie Ratings
//
#define XC_PC_SPAIN_MOVIE_RATING_MINOR      5   // New rating, but lower than TP in display order
#define XC_PC_SPAIN_MOVIE_RATING_TP         0
#define XC_PC_SPAIN_MOVIE_RATING_7          10
#define XC_PC_SPAIN_MOVIE_RATING_12         20
#define XC_PC_SPAIN_MOVIE_RATING_13         20  // Obsolete, replaced by XC_PC_SPAIN_MOVIE_RATING_12
#define XC_PC_SPAIN_MOVIE_RATING_16         30
#define XC_PC_SPAIN_MOVIE_RATING_18         40

// Italy Movie Ratings
//
#define XC_PC_ITALY_MOVIE_RATING_T          0
#define XC_PC_ITALY_MOVIE_RATING_VM14       10
#define XC_PC_ITALY_MOVIE_RATING_VM18       20

// Mexico Movie Ratings
//
#define XC_PC_MEXICO_MOVIE_RATING_AA            0
#define XC_PC_MEXICO_MOVIE_RATING_A             10
#define XC_PC_MEXICO_MOVIE_RATING_B             20
#define XC_PC_MEXICO_MOVIE_RATING_B15           30
#define XC_PC_MEXICO_MOVIE_RATING_C             40
#define XC_PC_MEXICO_MOVIE_RATING_D             50

// Turkey Movie Ratings
//
#define XC_PC_TURKEY_MOVIE_RATING_G         0
#define XC_PC_TURKEY_MOVIE_RATING_7PLUS     10
#define XC_PC_TURKEY_MOVIE_RATING_7A        20
#define XC_PC_TURKEY_MOVIE_RATING_13PLUS    30
#define XC_PC_TURKEY_MOVIE_RATING_13A       40
#define XC_PC_TURKEY_MOVIE_RATING_15PLUS    50
#define XC_PC_TURKEY_MOVIE_RATING_15A       60
#define XC_PC_TURKEY_MOVIE_RATING_18PLUS    70

// Slovak Republic Movie Ratings
//
#define XC_PC_SLOVAK_REPUBLIC_MOVIE_RATING_U         0
#define XC_PC_SLOVAK_REPUBLIC_MOVIE_RATING_7        10
#define XC_PC_SLOVAK_REPUBLIC_MOVIE_RATING_7PLUS    20
#define XC_PC_SLOVAK_REPUBLIC_MOVIE_RATING_12       30
#define XC_PC_SLOVAK_REPUBLIC_MOVIE_RATING_12PLUS   40
#define XC_PC_SLOVAK_REPUBLIC_MOVIE_RATING_15       50
#define XC_PC_SLOVAK_REPUBLIC_MOVIE_RATING_18       60

// Brazil Movie Ratings
//
#define XC_PC_BRAZIL_MOVIE_ER                  0x00
#define XC_PC_BRAZIL_MOVIE_L                   0x10
#define XC_PC_BRAZIL_MOVIE_10                  0x20
#define XC_PC_BRAZIL_MOVIE_12                  0x30
#define XC_PC_BRAZIL_MOVIE_14                  0x40
#define XC_PC_BRAZIL_MOVIE_16                  0x50
#define XC_PC_BRAZIL_MOVIE_18                  0x60


#ifdef __cplusplus
}
#endif


#endif  // _XCONFIG_

