/**************************************************************************
 *
 * Copyright (c) Microsoft Corporation. All rights reserved.
 *
 * Module Name:    XFileCache.h
 * 
 * Abstract:
 * 
 *     Game Disc Caching API
 *
 **************************************************************************/

#ifndef XFILECACHE_H
#define XFILECACHE_H

#include <sal.h>

#define XFILECACHEAPI   /*nothing*/

#ifdef __cplusplus
extern "C" {
#endif


DWORD
XFILECACHEAPI
XFileCacheInit(
   __in DWORD      Flags,                               /* (see XFILECACHE_INIT_xxx below) */
   __in ULONGLONG  MaxCacheSize,                        /* maximum bytes to cache */
   __in DWORD      HardwareThread,                      /* where worker thread should execute */
   __in DWORD      ScratchBufferSize,                   /* working storage (zero=default, or at least 192K) */
   __in DWORD      AppVersion                           /* version for application (cache will wipe if no match) */
   );

#define XFILECACHE_MAX_SIZE                     (0xffffffffffffffffui64)
#define XFILECACHE_DEFAULT_THREAD               5           /* default hardware thread to use */

#define XFILECACHE_PRESERVE                     0           /* preserve existing files */
#define XFILECACHE_CLEAR_STARTUP                0x80000000  /* clear "startup" cache area */
#define XFILECACHE_CLEAR_NORMAL                 0x40000000  /* clear "normal" cache area */
#define XFILECACHE_CLEAR_ALL                    0xc0000000  /* clear everything */
#define XFILECACHE_OPPORTUNISTIC_ON             0x00000010  /* allow opportunistic caching (via CreateFile) */
#define XFILECACHE_OPPORTUNISTIC_OFF            0x00000020  /* forbid opportunistic caching */

VOID
XFILECACHEAPI
XFileCacheShutdown(
    );

DWORD
XFILECACHEAPI
XFileCacheControl(
   __in DWORD CacheMode
   );

#define XFILECACHE_BACKGROUND_OFF               (1)     /* caching off */
#define XFILECACHE_BACKGROUND_ON                (2)     /* caching on */
#define XFILECACHE_STARTUP_FILES                (4)     /* flag newly cached files as 'startup' */
#define XFILECACHE_NORMAL_FILES                 (8)     /* flag newly cached files as 'normal' */

DWORD
XFILECACHEAPI
XFileCachePreload(
   __in DWORD Flags,                                                /* type of files (startup / normal / etc.) */
   __in_z LPCSTR pszFileName                                        /* file or pattern to preload */
   );

DWORD
XFILECACHEAPI
XFileCachePreloadFiles(
   _In_ DWORD Flags,                                                /* type of files (startup / normal / etc.) */
   _In_count_(FileNameCount) _Deref_pre_z_ LPCSTR* ppszFileNames,   /* vector of files to preload */
   _In_ DWORD FileNameCount                                         /* number of files */
   );

DWORD
XFILECACHEAPI
XFileCacheRemove(
    __in_z LPCSTR pszFileName                                       /* possibly wildcarded filename */
    );

DWORD
XFILECACHEAPI
XFileCacheRemoveFiles(
    _In_count_(FileNameCount) _Deref_pre_z_ LPCSTR* ppszFileNames, /* vector of filenames (possibly wildcarded) */
    _In_ DWORD FileNameCount                                       /* number of files */
    );

DWORD
XFILECACHEAPI
XFileCacheControlFiles(
   _In_ DWORD Flags,                                                /* control flags (see below) */
   _In_count_(FileNameCount) _Deref_pre_z_ LPCSTR* ppszFileNames,   /* vector of files */
   _In_ DWORD FileNameCount                                         /* number of files */
   );

//
//  valid Flags for XFileCacheControlFiles
//

#define XFILECACHE_DONTCACHE                    1       /* don't cache directory or leaf pattern */


typedef DWORD (*XFILECACHE_READFILEADVISORY_CALLBACK)(
    __in_opt PVOID Context,
    __in_z const char* Path,
    __in PLARGE_INTEGER pFileOffset,
    __in DWORD nNumberOfBytesToRead
);

VOID
XFILECACHEAPI
XFileCacheSetFileIoCallbacks(
    __in XFILECACHE_READFILEADVISORY_CALLBACK ReadFileAdvisoryCallback,
    __in_opt PVOID Context
    );

#define XFILECACHE_READFILEADVISORY_DEFER_MASK      0x80000000
#define XFILECACHE_READFILEADVISORY_HOLDOFF(MS)     (XFILECACHE_READFILEADVISORY_DEFER_MASK | (MS))
#define XFILECACHE_READFILEADVISORY_ALLOW(MS)       (MS)
#define XFILECACHE_READFILEADVISORY_OK              0


//
//  XFileCacheCreateFileA is just like CreateFile, with an additional flags parameter to
//  control caching behavior (e.g., "don't cache")
//

WINBASEAPI
HANDLE
WINAPI
XFileCacheCreateUncachedFileA(
    __in_z LPCSTR lpFileName,
    __in DWORD dwDesiredAccess,
    __in DWORD dwShareMode,
    __in_opt LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    __in DWORD dwCreationDisposition,
    __in DWORD dwFlagsAndAttributes,
    __in_opt HANDLE hTemplateFile
   );
#define XFileCacheCreateUncachedFile XFileCacheCreateUncachedFileA


/* ---------------------------------------------------------------- */


#pragma pack(push, 4)
typedef struct _XFILECACHESTATS XFILECACHESTATS, *PXFILECACHESTATS;
struct _XFILECACHESTATS
{
    ULONGLONG           CachedTotal;
    DWORD               FileCount;

    DWORD               CacheHitOpens;
    DWORD               CacheMissOpens;
    DWORD               CacheHitReads;
    DWORD               CacheMissReads;
    LARGE_INTEGER       CacheHitBytesRequested;
    LARGE_INTEGER       CacheMissBytesRequested;
    LARGE_INTEGER       CacheCopyBytes;
    LARGE_INTEGER       CacheEvictBytes;
};
#pragma pack(pop)

DWORD
XFILECACHEAPI
XFileCacheGetStats(
   __out PXFILECACHESTATS lpFileCacheStats
   );


#if defined(_DEBUG)

typedef struct _XFILECACHE_FIND_DATA XFILECACHE_FIND_DATA, *PXFILECACHE_FIND_DATA;
struct _XFILECACHE_FIND_DATA
{
    DWORD dwFileAttributes;
    DWORD nFileSizeHigh;
    DWORD nFileSizeLow;
    DWORD dwMediaId;
    char cFileName[MAX_PATH];
};

HANDLE
XFILECACHEAPI
XFileCacheFindFirstFile(
    __in_z LPCSTR lpFileName,
    __out PXFILECACHE_FIND_DATA pFindFileData
    );

BOOL
XFILECACHEAPI
XFileCacheFindNextFile(
   __in HANDLE hFindFile,
   __out PXFILECACHE_FIND_DATA pFindFileData
   );

BOOL
XFILECACHEAPI
XFileCacheFindClose(
   __in HANDLE hFindFile
   );

#endif // _DEBUG



#ifdef __cplusplus
}
#endif

#endif // XFILECACHE_H
