

//-----------------------------------------------------------------------------
// File:        XJSON.h
//
// Copyright (C) Microsoft. All rights reserved.
//
// Contents:
//
// Comments:
//
//-----------------------------------------------------------------------------

#pragma once 

// various #defines for error codes
#define JSON_E_MISSING_CLOSING_QUOTE                _HRESULT_TYPEDEF_(0x803F0001)
#define JSON_E_NOT_FIELDNAME_TOKEN                  _HRESULT_TYPEDEF_(0x803F0002)
#define JSON_E_INVALID_PARAMETER                    _HRESULT_TYPEDEF_(0x803F0003)
#define JSON_E_NUMBER_TOO_LONG                      _HRESULT_TYPEDEF_(0x803F0004)
#define JSON_E_BUFFER_TOO_SMALL                     _HRESULT_TYPEDEF_(0x803F0005)
#define JSON_E_MISSING_NAME_SEPARATOR               _HRESULT_TYPEDEF_(0x803F0006)
#define JSON_E_UNEXPECTED_TOKEN                     _HRESULT_TYPEDEF_(0x803F0007)
#define JSON_E_UNEXPECTED_END_ARRAY                 _HRESULT_TYPEDEF_(0x803F0008)
#define JSON_E_UNEXPECTED_END_OBJECT                _HRESULT_TYPEDEF_(0x803F0009)
#define JSON_E_INVALID_TOKEN                        _HRESULT_TYPEDEF_(0x803F000A)
#define JSON_E_UNEXPECTED_NAME_SEPARATOR            _HRESULT_TYPEDEF_(0x803F000B)
#define JSON_E_UNEXPECTED_VALUE_SEPARATOR           _HRESULT_TYPEDEF_(0x803F000C)
#define JSON_E_MISSING_END_OBJECT                   _HRESULT_TYPEDEF_(0x803F000D)
#define JSON_E_MISSING_END_ARRAY                    _HRESULT_TYPEDEF_(0x803F000E)
#define JSON_E_NOT_STRING_TOKEN                     _HRESULT_TYPEDEF_(0x803F000F)
#define JSON_E_MAX_NESTING_EXCEEDED                 _HRESULT_TYPEDEF_(0x803F0010)
#define JSON_E_UNEXPECTED_ESCAPE_CHARACTER          _HRESULT_TYPEDEF_(0x803F0011)
#define JSON_E_INVALID_ESCAPED_CHARACTER            _HRESULT_TYPEDEF_(0x803F0012)
#define JSON_E_INVALID_UNICODE_ESCAPE               _HRESULT_TYPEDEF_(0x803F0013)
#define JSON_E_INVALID_STRING_CHARACTER             _HRESULT_TYPEDEF_(0x803F0014)
#define JSON_E_INVALID_NUMBER                       _HRESULT_TYPEDEF_(0x803F0015)
#define JSON_E_NOT_NUMBER_TOKEN                     _HRESULT_TYPEDEF_(0x803F0016)
#define JSON_E_NUMBER_OUT_OF_RANGE                  _HRESULT_TYPEDEF_(0x803F0017)
#define JSON_E_OUT_OF_BUFFER                        _HRESULT_TYPEDEF_(0x803F0018)
#define JSON_E_NOT_VALUE_TOKEN                      _HRESULT_TYPEDEF_(0x803F0019)
#define JSON_E_UNEXPECTED_BEGIN_OBJECT              _HRESULT_TYPEDEF_(0x803F001A)
#define JSON_E_UNEXPECTED_BEGIN_ARRAY               _HRESULT_TYPEDEF_(0x803F001B)
#define JSON_E_UNEXPECTED_BEGIN_FIELD               _HRESULT_TYPEDEF_(0x803F001C)
#define JSON_E_UNEXPECTED_END_FIELD                 _HRESULT_TYPEDEF_(0x803F001D)
#define JSON_E_UNEXPECTED_VALUE                     _HRESULT_TYPEDEF_(0x803F001E)
#define JSON_E_INVALID_UTF8_STRING                  _HRESULT_TYPEDEF_(0x803F001F)

typedef enum _JSONTokenType
{
    Json_NotStarted,
    Json_BeginArray,
    Json_EndArray,
    Json_BeginObject,
    Json_EndObject,
    Json_String,
    Json_Number,
    Json_True,
    Json_False,
    Json_Null,
    Json_FieldName,
    Json_NameSeparator,
    Json_ObjectSeparator, // ',' inside an object
    Json_ValueSeparator   // ',' inside an array
} JSONTOKENTYPE, *PJSONTOKENTYPE;

DECLARE_HANDLE(HJSONREADER);
DECLARE_HANDLE(HJSONWRITER);

HJSONREADER XJSONCreateReader();
HRESULT XJSONSetBuffer(__in HJSONREADER reader, __in_ecount(cb) const char* pBuffer, __in DWORD nBufferChars, __in BOOL final);
HRESULT XJSONSetBuffer(__in HJSONREADER reader, __in_ecount(cb) const WCHAR* pBuffer, __in DWORD nBufferChars, __in BOOL final);
HRESULT XJSONReadToken(__in HJSONREADER reader, __out JSONTOKENTYPE* pTokenType, __out DWORD* pTokenLength, __out DWORD* pParsed);
HRESULT XJSONGetTokenValue(__in HJSONREADER reader, __inout_ecount(cb) WCHAR* pBuffer, __in DWORD nBufferChars);
HRESULT XJSONGetTokenValue(__in HJSONREADER reader, __inout_ecount(cb) char* pBuffer, __in DWORD nBufferChars);
HRESULT XJSONCloseReader(__in HJSONREADER reader);

HJSONWRITER XJSONCreateWriter();
void XJSONReset(__in HJSONWRITER writer);
HRESULT XJSONBeginObject(__in HJSONWRITER writer);
HRESULT XJSONBeginArray(__in HJSONWRITER writer);
HRESULT XJSONBeginField(__in HJSONWRITER writer, __in const char* pFieldName, __in DWORD nFieldNameChars);
HRESULT XJSONBeginField(__in HJSONWRITER writer, __in const WCHAR* pFieldName, __in DWORD nFieldNameChars);
HRESULT XJSONEndObject(__in HJSONWRITER writer);
HRESULT XJSONEndArray(__in HJSONWRITER writer);
HRESULT XJSONEndField(__in HJSONWRITER writer);
HRESULT XJSONWriteFieldString(__in HJSONWRITER writer, __in const char* pFieldName, __in DWORD nFieldNameChars, __in const char* pValue, __in DWORD nValueChars);
HRESULT XJSONWriteFieldString(__in HJSONWRITER writer, __in const WCHAR* pFieldName, __in DWORD nFieldNameChars, __in const WCHAR* pValue, __in DWORD nValueChars);
HRESULT XJSONWriteFieldNumber(__in HJSONWRITER writer, __in const char* pFieldName, __in DWORD nFieldNameChars, __in double nValue);
HRESULT XJSONWriteFieldNumber(__in HJSONWRITER writer, __in const WCHAR* pFieldName, __in DWORD nFieldNameChars, __in double nValue);
HRESULT XJSONWriteFieldBoolean(__in HJSONWRITER writer, __in const char* pFieldName, __in DWORD nFieldNameChars, __in bool bValue);
HRESULT XJSONWriteFieldBoolean(__in HJSONWRITER writer, __in const WCHAR* pFieldName, __in DWORD nFieldNameChars, __in bool bValue);
HRESULT XJSONWriteFieldNull(__in HJSONWRITER writer, __in const char* pFieldName, __in DWORD nFieldNameChars);
HRESULT XJSONWriteFieldNull(__in HJSONWRITER writer, __in const WCHAR* pFieldName, __in DWORD nFieldNameChars);
HRESULT XJSONWriteStringValue(__in HJSONWRITER writer, __in const char* pValue, __in DWORD nValueChars);
HRESULT XJSONWriteStringValue(__in HJSONWRITER writer, __in const WCHAR* pValue, __in DWORD nValueChars);
HRESULT XJSONWriteNumberValue(__in HJSONWRITER writer, __in double nValue);
HRESULT XJSONWriteBooleanValue(__in HJSONWRITER writer, __in bool bValue);
HRESULT XJSONWriteNullValue(__in HJSONWRITER writer);
HRESULT XJSONGetBuffer(__in HJSONWRITER writer, __out char* pBuffer, __inout DWORD* pnBufferChars);
HRESULT XJSONGetBuffer(__in HJSONWRITER writer, __out WCHAR* pBuffer, __inout DWORD* pnBufferChars);
HRESULT XJSONCloseWriter(__in HJSONWRITER writer);


