/*****************************************************************************
 *
 *  Copyright (C) Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       xmedia2.h
 *  Content:    Definitions for the XMedia2 API.
 *
 ****************************************************************************/

#ifndef __XMEDIA2_INCLUDED__
#define __XMEDIA2_INCLUDED__

#pragma once

#include <xtl.h>
#include <xaudio2.h>

#ifdef __cplusplus
extern "C" {
#endif

#pragma warning(push)
#pragma warning(disable:4201)

//
// Miscellaneous defines.
//

#define XMEDIA_INLINE                   __inline
#define XMEDIA_SAFE_RELEASE(p)          {if (p) {(p)->Release(); (p) = NULL;}}

    #if _MSC_VER >= 1400
        #define XMEDIA2_METHOD_DEPRECATED __declspec(deprecated("This XMedia2 method is deprecated."))
    #else
        #define XMEDIA2_METHOD_DEPRECATED __declspec(deprecated)
    #endif

//
// Callback used with IXMedia2XmvPlayer::SetCallback() API.
//

typedef
VOID
(*XMEDIA_XMV_CALLBACK)(
    __in_opt PVOID pvContext
    );

//
// Callbacks used with user I/O routines.
//

typedef
HRESULT
(*XMEDIA_USER_STREAM_READ_CALLBACK)(
    __in_opt PVOID pvContext,
    ULONGLONG ullOffset,
    __in PVOID pvBuffer,
    DWORD dwBytesToRead,
    __out DWORD* pdwBytesRead
    );

//
// HRESULTs returned by IXMedia2XmvPlayer::GetNextFrame() API.
//

#define XMEDIA_WARNING                  0x10000000
#define XMEDIA_FACILITY                 0x06660000
#define XMEDIA_W_EOF                    _HRESULT_TYPEDEF_(XMEDIA_WARNING | XMEDIA_FACILITY | ERROR_HANDLE_EOF)
#define XMEDIA_W_NO_DATA                _HRESULT_TYPEDEF_(XMEDIA_WARNING | XMEDIA_FACILITY | ERROR_NO_DATA)

//
// Flags used with Create method.
//

#define XMEDIA_CREATE_BACKGROUND_MUSIC  0x00000001
#define XMEDIA_CREATE_CPU_AFFINITY      0x00000002
#define XMEDIA_CREATE_SHARE_IO_CACHE    0x00000004
#define XMEDIA_CREATE_FOR_LOOP          0x00000008

//
// Flags used with Play, GetNextFrame, and RenderNextFrame methods.
//

#define XMEDIA_PLAY_DISABLE_AV_SYNC         0x00000001
#define XMEDIA_PLAY_LOOP                    0x00000002
#define XMEDIA_PLAY_REPEAT_FRAME            0x00000004
#define XMEDIA_PLAY_UNALTERED_TIMETOPRESENT 0x00000008

//
// Flags used with Stop method.
//

#define XMEDIA_STOP_IMMEDIATE           0x00000000
#define XMEDIA_STOP_LOOP                0x00000001

//
// XMedia player status.
//

typedef enum
{
    XMEDIA_PLAYER_IDLE = 1,
    XMEDIA_PLAYER_BUFFERING,
    XMEDIA_PLAYER_PLAYING,
    XMEDIA_PLAYER_PAUSING,
    XMEDIA_PLAYER_PAUSED,
    XMEDIA_PLAYER_STOPPING,
    XMEDIA_PLAYER_STOPPED,
    XMEDIA_PLAYER_TERMINATING
} XMEDIA_PLAYER_STATUS, *PXMEDIA_PLAYER_STATUS;

//
// XMedia callback notification types.
//

typedef enum
{
    XMEDIA_NOTIFY_END_OF_FRAME = 1,
    XMEDIA_NOTIFY_STATE_CHANGE,
    XMEDIA_NOTIFY_BEGIN_SCENE,
    XMEDIA_NOTIFY_END_SCENE
} XMEDIA_CALLBACK_TYPE, *PXMEDIA_CALLBACK_TYPE;

//
// XMedia player creation types.
//

typedef enum
{
    XMEDIA_CREATE_FROM_FILE = 1,
    XMEDIA_CREATE_FROM_HANDLE,
    XMEDIA_CREATE_FROM_MEMORY,
    XMEDIA_CREATE_FROM_USER_IO
} XMEDIA_CREATE_TYPE, *PXMEDIA_CREATE_TYPE;

//
// XMedia video formats.
//

typedef enum
{
    XMEDIA_VIDEO_FORMAT_I420 = 1,
} XMEDIA_VIDEO_FORMAT, *PXMEDIA_VIDEO_FORMAT;

//
// XMedia player A/V stream numbers.
//

typedef enum
{
    XMEDIA_STREAM_ID_USE_DEFAULT = 0,
    XMEDIA_STREAM_ID_DONT_USE = 0xffffffff
} XMEDIA_STREAM_ID, *PXMEDIA_STREAM_ID;

//
// Audio and video descriptors.
//

typedef struct _XMEDIA_AUDIO_DESCRIPTOR
{
    DWORD dwNumberOfChannels;
    DWORD dwSampleRate;
    DWORD dwBitsPerSample;
    DWORD dwClipDuration;
} XMEDIA_AUDIO_DESCRIPTOR, *PXMEDIA_AUDIO_DESCRIPTOR;

typedef struct _XMEDIA_VIDEO_DESCRIPTOR
{
    DWORD dwWidth;
    DWORD dwHeight;
    FLOAT fFrameRate;
    DWORD dwClipDuration;
} XMEDIA_VIDEO_DESCRIPTOR, *PXMEDIA_VIDEO_DESCRIPTOR;

//
// Media playback status.
//

typedef struct _XMEDIA_PLAYBACK_STATUS
{
    XMEDIA_PLAYER_STATUS Status;
    LONG lDuration;
} XMEDIA_PLAYBACK_STATUS, *PXMEDIA_PLAYBACK_STATUS;

//
// Video screen (video playback surface) vertex definitions.
//

typedef struct _XMEDIA_VIDEO_SCREEN_VERTEX
{
    FLOAT fX, fY, fZ, fTu, fTv;
} XMEDIA_VIDEO_SCREEN_VERTEX, *PXMEDIA_VIDEO_SCREEN_VERTEX;

typedef union _XMEDIA_VIDEO_SCREEN
{
    struct
    {
        XMEDIA_VIDEO_SCREEN_VERTEX upperLeft;
        XMEDIA_VIDEO_SCREEN_VERTEX upperRight;
        XMEDIA_VIDEO_SCREEN_VERTEX lowerLeft;
        XMEDIA_VIDEO_SCREEN_VERTEX lowerRight;
    };
    XMEDIA_VIDEO_SCREEN_VERTEX aVertices[4];
} XMEDIA_VIDEO_SCREEN, *PXMEDIA_VIDEO_SCREEN;

//
// Video frame used with GetNextFrame method.
//

typedef struct _XMEDIA_VIDEO_FRAME
{
    XMEDIA_VIDEO_FORMAT videoFormat;
    DWORD dwFlags;
    LONG lTimeToPresent;

    union
    {
        struct
        {
            PVOID pvYBuffer;
            PVOID pvUBuffer;
            PVOID pvVBuffer;
            DWORD dwYBufferSize;
            DWORD dwUBufferSize;
            DWORD dwVBufferSize;
            DWORD dwYPitch;
            DWORD dwUPitch;
            DWORD dwVPitch;
        } i420;

    };
} XMEDIA_VIDEO_FRAME, *PXMEDIA_VIDEO_FRAME;

//
// Paramters used with XMV player Create method.
//

typedef struct _XMEDIA_XMV_CREATE_PARAMETERS
{
    XMEDIA_CREATE_TYPE createType;
    DWORD dwFlags;
    DWORD dwAudioDecoderCpu;
    DWORD dwVideoDecoderCpu;
    DWORD dwAudioRendererCpu;
    DWORD dwVideoRendererCpu;
    DWORD dwAudioStreamId;
    DWORD dwVideoStreamId;

    union
    {
        struct
        {
            LPCSTR szFileName;
            DWORD dwIoBlockSize;
            DWORD dwIoBlockCount;
            DWORD dwIoBlockJitter;
        } createFromFile;

        struct
        {
            HANDLE handle;
            ULONGLONG ullOffset;
            ULONGLONG ullLength;
            DWORD dwIoBlockSize;
            DWORD dwIoBlockCount;
            DWORD dwIoBlockJitter;
        } createFromHandle;

        struct
        {
            PVOID pvBuffer;
            DWORD dwBufferSize;
        } createFromMemory;

        struct
        {
            XMEDIA_USER_STREAM_READ_CALLBACK pfnAudioStreamReadCallback;
            XMEDIA_USER_STREAM_READ_CALLBACK pfnVideoStreamReadCallback;
            PVOID pvAudioStreamContext;
            PVOID pvVideoStreamContext;
        } createFromUserIo;
    };
} XMEDIA_XMV_CREATE_PARAMETERS, *PXMEDIA_XMV_CREATE_PARAMETERS;

//
// IXMedia2XmvPlayer interface.
//

typedef struct IXMedia2XmvPlayer IXMedia2XmvPlayer;

#ifdef __cplusplus

//
// IXMedia2XmvPlayer C++ interface definition.
//

struct IXMedia2XmvPlayer
{
    DWORD
    WINAPI
    AddRef();

    DWORD
    WINAPI
    Release();

    HRESULT
    WINAPI
    GetNextFrame(
        __inout PXMEDIA_VIDEO_FRAME pVideoFrame
        );

    HRESULT
    WINAPI
    RenderNextFrame(
        DWORD dwFlags,
        __out_opt PLONG plTimeToPresent
        );

    HRESULT
    WINAPI
    SetRectangle(
        __in PRECT pRect
        );

    HRESULT
    WINAPI
    GetRectangle(
        __inout PRECT pRect
        );

    HRESULT
    WINAPI
    SetVideoScreen(
        __in PXMEDIA_VIDEO_SCREEN pScreen
        );

    HRESULT
    WINAPI
    GetVideoScreen(
        __inout PXMEDIA_VIDEO_SCREEN pScreen
        );

    HRESULT
    WINAPI
    GetAudioDescriptor(
        __inout PXMEDIA_AUDIO_DESCRIPTOR pAudioDescriptor
        );

    HRESULT
    WINAPI
    GetVideoDescriptor(
        __inout PXMEDIA_VIDEO_DESCRIPTOR pVideoDescriptor
        );

    HRESULT
    WINAPI
    Play(
        __in DWORD dwFlags,
        __in_opt PXOVERLAPPED pOverlapped
        );

    HRESULT
    WINAPI
    Pause();

    HRESULT
    WINAPI
    Resume();

    HRESULT
    WINAPI
    Stop(
        DWORD dwFlags
        );


        XMEDIA2_METHOD_DEPRECATED
        HRESULT
        WINAPI
        Seek(
            DWORD dwSeekOffset,
            __out_opt DWORD* pdwSetOffset
            );

    HRESULT
    WINAPI
    SetPlaySpeed(
    FLOAT fPlaySpeed
    );

    HRESULT
    WINAPI
    GetStatus(
        __inout PXMEDIA_PLAYBACK_STATUS pStatus
        );

    HRESULT
    WINAPI
    GetSourceVoice(
        __deref_out IXAudio2SourceVoice** ppSourceVoice
        );

    HRESULT
    WINAPI
    SetCallback(
        XMEDIA_CALLBACK_TYPE Type,
        __in XMEDIA_XMV_CALLBACK pfnCallback,
        __in_opt PVOID pvContext
        );
};

#endif // __cplusplus

//
// IXMedia2XmvPlayer interface create methods.
//

HRESULT
WINAPI
XMedia2CreateXmvPlayer(
    __in IDirect3DDevice9* pD3dDevice,
    __in IXAudio2* pXAudio2Instance,
    __in PXMEDIA_XMV_CREATE_PARAMETERS pCreateParameters,
    __deref_out IXMedia2XmvPlayer** ppPlayer
    );


//
// IXMedia2XmvPlayer C interface definition.
//

DWORD
WINAPI
IXMedia2XmvPlayer_AddRef(
    __in IXMedia2XmvPlayer* pPlayer
    );

DWORD
WINAPI
IXMedia2XmvPlayer_Release(
    __in IXMedia2XmvPlayer* pPlayer
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_GetNextFrame(
    __in IXMedia2XmvPlayer* pPlayer,
    __inout PXMEDIA_VIDEO_FRAME pVideoFrame
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_RenderNextFrame(
    __in IXMedia2XmvPlayer* pPlayer,
    DWORD dwFlags,
    __out_opt PLONG plTimeToPresent
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_SetRectangle(
    __in IXMedia2XmvPlayer* pPlayer,
    __in PRECT pRect
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_GetRectangle(
    __in IXMedia2XmvPlayer* pPlayer,
    __inout PRECT pRect
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_SetVideoScreen(
    __in IXMedia2XmvPlayer* pPlayer,
    __in PXMEDIA_VIDEO_SCREEN pScreen
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_GetVideoScreen(
    __in IXMedia2XmvPlayer* pPlayer,
    __inout PXMEDIA_VIDEO_SCREEN pScreen
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_GetAudioDescriptor(
    __in IXMedia2XmvPlayer* pPlayer,
    __inout PXMEDIA_AUDIO_DESCRIPTOR pAudioDescriptor
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_GetVideoDescriptor(
    __in IXMedia2XmvPlayer* pPlayer,
    __inout PXMEDIA_VIDEO_DESCRIPTOR pVideoDescriptor
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_Play(
    __in IXMedia2XmvPlayer* pPlayer,
    DWORD dwFlags,
    __in_opt PXOVERLAPPED pOverlapped
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_Pause(
    __in IXMedia2XmvPlayer* pPlayer
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_Resume(
    __in IXMedia2XmvPlayer* pPlayer
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_Stop(
    __in IXMedia2XmvPlayer* pPlayer,
    DWORD dwFlags
    );

    XMEDIA2_METHOD_DEPRECATED
    HRESULT
    WINAPI
    IXMedia2XmvPlayer_Seek(
        __in IXMedia2XmvPlayer* pPlayer,
        DWORD dwSeekOffset,
        __out_opt DWORD* pdwSetOffset
        );

HRESULT
WINAPI
IXMedia2XmvPlayer_SetPlaySpeed(
    __in IXMedia2XmvPlayer* pPlayer,
    FLOAT fPlaySpeed
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_GetStatus(
    __in IXMedia2XmvPlayer* pPlayer,
    __inout PXMEDIA_PLAYBACK_STATUS pStatus
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_GetSourceVoice(
    __in IXMedia2XmvPlayer* pPlayer,
    __deref_out IXAudio2SourceVoice** ppSourceVoice
    );

HRESULT
WINAPI
IXMedia2XmvPlayer_SetCallback(
    __in IXMedia2XmvPlayer* pPlayer,
    XMEDIA_CALLBACK_TYPE Type,
    __in XMEDIA_XMV_CALLBACK pfnCallback,
    __in_opt PVOID pvContext
    );

#ifdef __cplusplus

//
// IXMedia2XmvPlayer C++ interface definition.
//

XMEDIA_INLINE
DWORD
WINAPI
IXMedia2XmvPlayer::AddRef()
{
    return IXMedia2XmvPlayer_AddRef(this);
}

XMEDIA_INLINE
DWORD
WINAPI
IXMedia2XmvPlayer::Release()
{
    return IXMedia2XmvPlayer_Release(this);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::GetNextFrame(
    __inout PXMEDIA_VIDEO_FRAME pVideoFrame
    )
{
    return IXMedia2XmvPlayer_GetNextFrame(this, pVideoFrame);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::RenderNextFrame(
    DWORD dwFlags,
    __out_opt PLONG plTimeToPresent
    )
{
    return IXMedia2XmvPlayer_RenderNextFrame(this, dwFlags, plTimeToPresent);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::SetRectangle(
    __in PRECT pRect
    )
{
    return IXMedia2XmvPlayer_SetRectangle(this, pRect);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::GetRectangle(
    __inout PRECT pRect
    )
{
    return IXMedia2XmvPlayer_GetRectangle(this, pRect);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::SetVideoScreen(
    __in PXMEDIA_VIDEO_SCREEN pScreen
    )
{
    return IXMedia2XmvPlayer_SetVideoScreen(this, pScreen);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::GetVideoScreen(
    __inout PXMEDIA_VIDEO_SCREEN pScreen
    )
{
    return IXMedia2XmvPlayer_GetVideoScreen(this, pScreen);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::GetAudioDescriptor(
    __inout PXMEDIA_AUDIO_DESCRIPTOR pAudioDescriptor
    )
{
    return IXMedia2XmvPlayer_GetAudioDescriptor(this, pAudioDescriptor);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::GetVideoDescriptor(
    __inout PXMEDIA_VIDEO_DESCRIPTOR pVideoDescriptor
    )
{
    return IXMedia2XmvPlayer_GetVideoDescriptor(this, pVideoDescriptor);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::Play(
    DWORD dwFlags,
    __in_opt PXOVERLAPPED pOverlapped
    )
{
    return IXMedia2XmvPlayer_Play(this, dwFlags, pOverlapped);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::Pause()
{
    return IXMedia2XmvPlayer_Pause(this);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::Resume()
{
    return IXMedia2XmvPlayer_Resume(this);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::Stop(
    DWORD dwFlags
    )
{
    return IXMedia2XmvPlayer_Stop(this, dwFlags);
}

    XMEDIA2_METHOD_DEPRECATED
    XMEDIA_INLINE
    HRESULT
    WINAPI
    IXMedia2XmvPlayer::Seek(
        DWORD dwSeekOffset,
        __out_opt DWORD* pdwSetOffset
        )
    {
        #pragma warning(push)
        #pragma warning(disable: 4996) // Disable "This XMedia2 method is deprecated" compile warning as IXMedia2XmvPlayer_Seek is also deprecated.
                                       // Thus users will get the warning ONLY if they actually use the seek methods.
        return IXMedia2XmvPlayer_Seek(this, dwSeekOffset, pdwSetOffset);
        #pragma warning(pop)
    }

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::SetPlaySpeed(
    FLOAT fPlaySpeed
    )
{
    return IXMedia2XmvPlayer_SetPlaySpeed(this, fPlaySpeed);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::GetStatus(
    __inout PXMEDIA_PLAYBACK_STATUS pStatus
    )
{
    return IXMedia2XmvPlayer_GetStatus(this, pStatus);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::GetSourceVoice(
    __deref_out IXAudio2SourceVoice** ppSourceVoice
    )
{
    return IXMedia2XmvPlayer_GetSourceVoice(this, ppSourceVoice);
}

XMEDIA_INLINE
HRESULT
WINAPI
IXMedia2XmvPlayer::SetCallback(
    XMEDIA_CALLBACK_TYPE Type,
    __in XMEDIA_XMV_CALLBACK pfnCallback,
    __in_opt PVOID pvContext
    )
{
    return IXMedia2XmvPlayer_SetCallback(this, Type, pfnCallback, pvContext);
}


#endif // __cplusplus

#pragma warning(pop)

#ifdef __cplusplus
}
#endif

#endif // __XMEDIA2_INCLUDED__

