
/************************************************************************
*                                                                       *
*   XMic.h -- This module defines the Xbox USB Microphone APIs          *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/

#pragma once
#ifndef __XMIC_H__
#define __XMIC_H__


#ifdef __cplusplus
extern "C" {
#endif


//------------------------------------------------------------------------------
// XMic Typedefs and Defines
//------------------------------------------------------------------------------

#define XMIC_COUNT_MAX                                  4
#define XMIC_FRAME_MAX                                  8

//
// Define the microphone status
//

typedef DWORD XMICSTATUS;

#define XMICSTATUS_DISCONNECTED                         0
#define XMICSTATUS_CONNECTED                            1
#define XMICSTATUS_STARTED                              2
#define XMICSTATUS_ERROR                                3

//
// Define the microphone capabilities
//

typedef struct _XMICCAPABILITIES {
    DWORD dwFeatures;
    WORD wFormatTag;
    WORD nChannels;
    DWORD dwSampleRatesSupported;
    WORD wBitsPerSample;
    WORD dwFrameLength;
    BYTE bColor;
    
    //
    // Device Identification data
    //
    WORD wVendorId;
    WORD wProductId;
    WORD wDeviceRevision;
    DWORD dwXid;

} XMICCAPABILITIES, *PXMICCAPABILITIES;

//
// Define the feature values
//

typedef DWORD XMICFEATURE;

#define XMICFEATURE_WIRELESS                            0x00000001
#define XMICFEATURE_ACCELEROMETER                       0x00000002
#define XMICFEATURE_OUTPUTLED                           0x00000004

//
// Define the sample rate values
//

typedef DWORD XMICSAMPLERATE;

#define XMICSAMPLERATE_32000                            0x00000040
#define XMICSAMPLERATE_48000                            0x00000100

//
// Define the different microphone gain values
//

#define XMICGAIN_CURRENT                                0
#define XMICGAIN_MIN                                    1
#define XMICGAIN_MAX                                    2

//
// Define the possible microphone colors
//

#define XMICCOLOR_BLACK                                 0
#define XMICCOLOR_WHITE                                 1
#define XMICCOLOR_UNKNOWN                               255

//------------------------------------------------------------------------------
// XMic Microphone API's
//------------------------------------------------------------------------------

XBOXAPI
DWORD
XMicGetStatus(
    _In_ DWORD dwMicrophoneIndex
    );

XBOXAPI
DWORD
XMicStart(
    _In_ DWORD dwMicrophoneIndex,
    _In_ DWORD dwSampleRate,
    _Out_opt_ PDWORD pdwFrameTransferLength,
    _Out_opt_ PXOVERLAPPED pOverlapped
    );

XBOXAPI
DWORD
XMicStop(
    _In_ DWORD dwMicrophoneIndex,
    _Out_opt_ PXOVERLAPPED pOverlapped
    );

XBOXAPI
DWORD
XMicRequestData(
    _In_ DWORD dwMicrophoneIndex,
    _In_ DWORD dwFrameCount,
    _Out_ PBYTE pbTransferBuffer,
    _Out_cap_(dwFrameCount) PWORD pwFrameTransferLengths,
    _Out_opt_ PXOVERLAPPED pOverlapped
    );

XBOXAPI
DWORD
XMicGetGain(
    _In_ DWORD dwMicrophoneIndex,
    _In_ DWORD dwGainIndex,
    _Out_ PFLOAT pfltGain
    );

XBOXAPI
DWORD
XMicSetGain(
    _In_ DWORD dwMicrophoneIndex,
    _In_ FLOAT fltGain,
    _Out_opt_ PXOVERLAPPED pOverlapped
    );

XBOXAPI
DWORD
XMicGetCapabilities(
    _In_ DWORD dwMicrophoneIndex,
    _Out_ PXMICCAPABILITIES pCapabilities
    );


#ifdef __cplusplus
}
#endif

#endif // __XMIC_H__
