/**************************************************************************
 *
 *  Copyright (C) Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       xmp.h
 *  Content:    Xenon Music Player API.
 *
 **************************************************************************/

#ifndef __XMP_INCLUDED__
#define __XMP_INCLUDED__

#include <xtl.h>
#include <xam.h>

#ifdef __cplusplus
extern "C" {
#endif

//
// Create title playlist flags
//

#define XMP_CREATETITLEPLAYLISTFLAG_NONE        (0x0000000)

//
// Pass this flag to XMPCreateTitlePlaylist() to make title playlists show up
// in the HUD and be accessible to the user.
// If this flag is set the playlist can never be deleted.
//

#define XMP_CREATETITLEPLAYLISTFLAG_PERMANENT   (0x0000001)

//
// Enumeration flags
//

#define XMP_CREATEENUMERATORFLAG_NONE           (0x0000000)
#define XMP_CREATEENUMERATORFLAG_FOREGROUNDPRI  (0x0000001)

//
// XMP playback flags
//

#define XMP_PLAYBACKFLAG_NONE                   (0x0000000)

//
// This flag will cause playback to goto into a paused state just prior
// to starting playback.
//
#define XMP_PLAYBACKFLAG_AUTOPAUSE              (0x0000001)

//
// Character lengths of metadata buffers.
//

#define XMP_MAX_METADATASTRING                  (40)

//
// Handle to title playlists and songs.
//

typedef LPVOID XMP_HANDLE, *LPXMP_HANDLE;

#define XMP_MAX_MEDIASOURCEDISPLAYNAME          (64)

//
// Metadata for a media source
//

typedef enum XMP_MEDIASOURCETYPE
{
    XMP_MEDIASOURCETYPE_LOCAL  = 0,
    XMP_MEDIASOURCETYPE_IPOD   = 1,
    XMP_MEDIASOURCETYPE_USB    = 2,
    XMP_MEDIASOURCETYPE_MTP    = 3,
    XMP_MEDIASOURCETYPE_WMC    = 4,
} XMP_MEDIASOURCETYPE, *LPXMP_MEDIASOURCETYPE;

#define XMP_MAX_MEDIASOURCEID                   (40)
typedef BYTE XMP_MEDIASOURCEID[XMP_MAX_MEDIASOURCEID];
typedef XMP_MEDIASOURCEID *LPXMP_MEDIASOURCEID;

typedef struct XMP_MEDIASOURCEINFO
{
    XMP_MEDIASOURCEID Id;                                   // Id.
    XMP_MEDIASOURCETYPE eType;                              // Type of media source
    DWORD dwReserved1;                                      // Reserved field.
    DWORD dwReserved2;                                      // Reserved field.
    WCHAR wszTitle[XMP_MAX_MEDIASOURCEDISPLAYNAME];         // Media source title.
} XMP_MEDIASOURCEINFO, *LPXMP_MEDIASOURCEINFO;

typedef const LPXMP_MEDIASOURCEINFO LPCXMP_MEDIASOURCEINFO;

//
// Metadata for media folders
//

#define XMP_MAX_FOLDERID                        (572)
typedef BYTE XMP_FOLDERID[XMP_MAX_FOLDERID];
typedef XMP_FOLDERID *LPXMP_FOLDERID;

#define XMP_FOLDERRESERVEDFIELDSIZE             (168)

typedef struct XMP_FOLDERINFO
{
    XMP_FOLDERID Id;                                        // Id.
    WCHAR wszTitle[XMP_MAX_METADATASTRING];                 // Folder title.
    BYTE Reserved[XMP_FOLDERRESERVEDFIELDSIZE];             // Reserved field.
} XMP_FOLDERINFO, *LPXMP_FOLDERINFO;

typedef const LPXMP_FOLDERINFO LPCXMP_FOLDERINFO;

//
// Metadata for a song
//

#define XMP_MAX_SONGID                          (572)
typedef BYTE XMP_SONGID[XMP_MAX_SONGID];
typedef XMP_SONGID *LPXMP_SONGID;

typedef struct XMP_SONGDETAILS
{
    XMP_SONGID Id;                                          // Id.
    WCHAR wszTitle[XMP_MAX_METADATASTRING];                 // Song title.
    WCHAR wszArtist[XMP_MAX_METADATASTRING];                // Song artist.
    WCHAR wszAlbum[XMP_MAX_METADATASTRING];                 // Song album.
    WCHAR wszAlbumArtist[XMP_MAX_METADATASTRING];           // Song album artist.
    WCHAR wszGenre[XMP_MAX_METADATASTRING];                 // Genre.
    DWORD dwTrackNumber;                                    // Track number in the album (0 indicates an invalid number).
    DWORD dwDuration;                                       // Duration in milliseconds.
    DWORD dwReserved1;                                      // Reserved field.
} XMP_SONGDETAILS, *LPXMP_SONGDETAILS;

typedef const LPXMP_SONGDETAILS LPCXMP_SONGDETAILS;

#define XMP_SONGRESERVEDFIELDSIZE               (572)

// Disable warning for unnamed structs and unions raised during Code Analysis build
#pragma warning(push)
#pragma warning(disable:4201)

typedef struct XMP_SONGINFO
{
    XMP_HANDLE hSong;                                       // Song handle.
    union
    {
        XMP_SONGDETAILS Details;
        struct
        {
            BYTE  abReserved[XMP_SONGRESERVEDFIELDSIZE];    // Reserved field.
            WCHAR wszTitle[XMP_MAX_METADATASTRING];         // Song title.
            WCHAR wszArtist[XMP_MAX_METADATASTRING];        // Song artist.
            WCHAR wszAlbum[XMP_MAX_METADATASTRING];         // Song album.
            WCHAR wszAlbumArtist[XMP_MAX_METADATASTRING];   // Song album artist.
            WCHAR wszGenre[XMP_MAX_METADATASTRING];         // Genre.
            DWORD dwTrackNumber;                            // Track number in the album (0 indicates an invalid number).
            DWORD dwDuration;                               // Duration in milliseconds.
            DWORD dwReserved1;                              // Reserved field.
        };
    };
    DWORD dwReserved2;                                      // Reserved field.
} XMP_SONGINFO, *LPXMP_SONGINFO;

#pragma warning(pop)
 
typedef const LPXMP_SONGINFO LPCXMP_SONGINFO;

typedef struct XMP_SONGART
{
    XMP_SONGID             Id;                              // Id
    __in_ecount(cbSongArtBufferSize) BYTE* bSongArtBuffer;  // Song art
    DWORD                  cbSongArtBufferSize;             // On input, the size of the Song Art buffer. On output, the actual size of the Song Art
                                                            // If bSongArtBuffer is NULL, on output the buffer size needed
    DWORD dwReserved1;                                      // Reserved field
} XMP_SONGART, *LPXMP_SONGART;

typedef const LPXMP_SONGART LPCXMP_SONGART;

//
// Metadata for a user playlist.
//

#define XMP_MAX_USERPLAYLISTID                  (572)
typedef BYTE XMP_USERPLAYLISTID[XMP_MAX_USERPLAYLISTID], *LPXMP_USERPLAYLISTID;

#define XMP_USERPLAYLISTRESERVEDFIELDSIZE       (168)

typedef struct XMP_USERPLAYLISTINFO
{
    XMP_USERPLAYLISTID Id;                                  // Id.
    WCHAR wszTitle[XMP_MAX_METADATASTRING];                 // Container title.
    BYTE Reserved[XMP_USERPLAYLISTRESERVEDFIELDSIZE];       // Reserved field.
} XMP_USERPLAYLISTINFO, *LPXMP_USERPLAYLISTINFO;

typedef const LPXMP_USERPLAYLISTINFO LPCXMP_USERPLAYLISTINFO;

//
// Status returned by calling XMPGetStatus().
//

typedef enum XMP_STATE
{
    XMP_STATE_IDLE = 0,                                     // XMP is idle and not playing anything.
    XMP_STATE_PLAYING,                                      // Playback of a song is occuring.
    XMP_STATE_PAUSED,                                       // Playback of a song has been paused.
} XMP_STATE, *LPXMP_STATE;

typedef const LPXMP_STATE LPCXMP_STATE;

//
// Playback modes
//

typedef enum XMP_PLAYBACKMODE
{
    XMP_PLAYBACKMODE_INORDER = 0,
    XMP_PLAYBACKMODE_SHUFFLE,
} XMP_PLAYBACKMODE, *LPXMP_PLAYBACKMODE;

typedef const LPXMP_PLAYBACKMODE LPCXMP_PLAYBACKMODE;

//
// Repeat modes.
//

typedef enum XMP_REPEATMODE
{
    XMP_REPEATMODE_PLAYLIST = 0,
    XMP_REPEATMODE_NOREPEAT,
} XMP_REPEATMODE, *LPXMP_REPEATMODE;

typedef const LPXMP_REPEATMODE LPCXMP_REPEATMODE;

//
// Types of songs that can be added to a playlist.
//

typedef enum XMP_SONGFORMAT
{
    XMP_SONGFORMAT_WMA = 0,
    XMP_SONGFORMAT_MP3,
} XMP_SONGFORMAT, *LPXMP_SONGFORMAT;

typedef const LPXMP_SONGFORMAT LPCXMP_SONGFORMAT;

//
// Song to be added to a title playlist.
//

typedef struct XMP_SONGDESCRIPTOR
{
    LPCWSTR        pwszFilePath;                            // Path to the song being added to the playlist.
    LPCWSTR        pwszTitle;                               // Song title.
    LPCWSTR        pwszArtist;                              // Song artist.
    LPCWSTR        pwszAlbum;                               // Song album.
    LPCWSTR        pwszAlbumArtist;                         // Song album artist.
    LPCWSTR        pwszGenre;                               // Genre.
    DWORD          dwTrackNumber;                           // Track number in the album.
    DWORD          dwDuration;                              // Duration in milliseconds.
    XMP_SONGFORMAT eSongFormat;                             // Format of the song.
} XMP_SONGDESCRIPTOR, *LPXMP_SONGDESCRIPTOR;

typedef const LPXMP_SONGDESCRIPTOR LPCXMP_SONGDESCRIPTOR;

//
// Creates a title playlist to playback songs.
//

DWORD
WINAPI
XMPCreateTitlePlaylist
(
    __in LPCXMP_SONGDESCRIPTOR  paSongs,                    // Array of songs to be placed into the playlist.
    DWORD                       dwNumSongs,                 // The length of the song array.
    DWORD                       dwFlags,                    // Flags.
    __in LPCWSTR                pwszPlaylistName,           // Friendly name for display in the UI.
                                                            // (Cannot be NULL if the flag XMP_CREATETITLEPLAYLISTFLAG_PERMANENT is used).
    __out_opt LPXMP_HANDLE      pahSongs,                   // Array of handles to the added songs.
                                                            // (If non NULL this array must be at least dwNumSongs in length).
    __out LPXMP_HANDLE          phPlaylist                  // Handle to a playlist.
);

//
// Deletes a title playlist. All song ids from the playlist become invalid.
// Any attempt to delete a permanent title playlist will cause return ERROR_ACCESS_DENIED.
//

DWORD
WINAPI
XMPDeleteTitlePlaylist
(
    __in XMP_HANDLE             hTitlePlaylist              // Handle to a playlist.
);
//
// Creates a song playlist to playback song-enumerated songs.
//

DWORD
WINAPI
XMPCreateSongPlaylist
(
    __in LPCXMP_SONGDETAILS     paSongs,                    // Array of songs to be placed into the playlist.
    DWORD                       dwNumSongs,                 // The length of the song array.
    __out_opt LPXMP_HANDLE      pahSongs,                   // Array of handles to the added songs.
                                                            // (If non NULL this array must be at least dwNumSongs in length).
    __out LPXMP_HANDLE          phPlaylist                  // Handle to a playlist.
);

//
// Deletes a song playlist.
//

DWORD
WINAPI
XMPDeleteSongPlaylist
(
    __in XMP_HANDLE             hPlaylist                   // Handle to a playlist.
);

//
// Returns information about the currently active song.
//

DWORD
WINAPI
XMPGetCurrentSong
(
    __out LPXMP_SONGINFO        pSongInfo,                  // Currently playing song.
    __inout_opt PXOVERLAPPED    pOverlapped                 // Overlapped structure.
);

//
// Tries to start playback of a title created playlist.
//

DWORD
WINAPI
XMPPlayTitlePlaylist
(
    __in     XMP_HANDLE         hPlaylist,                  // Title created playlist to play.
    __in_opt XMP_HANDLE         hSong,                      // Song to begin playback from.
                                                            // (NULL indicates start from the first song).
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure.
);

//
// Tries to start playback of a user owned playlist.
//

DWORD
WINAPI
XMPPlayUserPlaylist
(
    __in LPXMP_USERPLAYLISTINFO pUserPlaylist,              // Users playlist to playback.
    __inout_opt PXOVERLAPPED    pOverlapped                 // Overlapped structure.
);

//
// Tries to resume previously paused playback.
//

DWORD
WINAPI
XMPContinue
(
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure.
);

//
// Tries to stop playback.
//

DWORD
WINAPI
XMPStop
(
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure.
);

//
// Tries to pause playback.
//

DWORD
WINAPI
XMPPause
(
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure.
);

//
// Tries to start playback of the next song.
//

DWORD
WINAPI
XMPNext
(
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure.
);

//
// Tries to start playback of the previous song.
//

DWORD
WINAPI
XMPPrevious
(
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure.
);

//
// Tries to set XMP's playback behavior (repeat mode, playback mode...)
//

DWORD
WINAPI
XMPSetPlaybackBehavior
(
    XMP_PLAYBACKMODE ePlaybackMode,                         // The playback mode to use.
    XMP_REPEATMODE eRepeatMode,                             // The repeat mode to use.
    DWORD dwFlags,                                          // Other playback flags.
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure for async requests.
);

//
// Gets the current state of XMP.
//

DWORD
WINAPI
XMPGetStatus
(
    __out LPXMP_STATE peState                               // The current state of XMP
);

//
// Gets the volume that XMP is using in the range 0-1
//

DWORD
WINAPI
XMPGetVolume
(
    __out FLOAT* pfVolume                                   // The volume XMP is using.
);

//
// Sets the volume that XMP uses.
//

DWORD
WINAPI
XMPSetVolume
(
    FLOAT fVolume,                                          // The new volume
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure for async requests.
);

//
// Gets the playback behavior (repeat mode, playback mode, ..)
// of XMP.
//

DWORD
WINAPI
XMPGetPlaybackBehavior
(
    __out LPXMP_PLAYBACKMODE pePlaybackMode,                // The playback mode.
    __out LPXMP_REPEATMODE peRepeatMode,                    // The repeat mode.
    __out LPDWORD pdwFlags                                  // Other playback flags.
);

//
// Creates a object used to enumerate all the user playlists that are
// currently available. Subsequent calls to XEnumerate() using the
// created enumerator will retrieve the playlists.
//
// Memory for the enumerator is created out of the caller's budget.
//

DWORD
WINAPI
XMPCreateUserPlaylistEnumerator
(
    DWORD   dwFlags,                                        // Enumeration flags.
    DWORD   cItems,                                         // Number of items we want to enumerate.
    __out DWORD*  pcbBuffer,                                // Size of the buffer required to get the playlists.
    __out HANDLE* ph                                        // Handle to the enumerator object.
);

//
// Obtains the list of media sources currently available for music playback.
// These sources can then be passed to the various XMPCreateEnumerator APIs
// in order to search for music to playback.
//

DWORD
WINAPI
XMPGetConnectedMediaSources
(
    __out_ecount_part(dwMaxSources, *pdwSourcesReturned) LPXMP_MEDIASOURCEINFO paMediaSources, // Array of media sources to be filled.
    __in  DWORD dwMaxSources,                               // The size of the media sources array.
    __out LPDWORD pdwSourcesReturned                        // The number of media sources returned.
);

//
// Creates an object used to enumerate the folders within the specified media
// source or sub-folder. Subsequent calls to XEnumerate() using the created
// enumerator will retrieve the folders via XMPLIPS_FOLDERINFO structures.
// Passing NULL for pFolderId will create an enumerator for the top-level
// folders from the media source.  Otherwise, it will create an enumerator for
// the sub-folders of the specified folder.
//

DWORD
WINAPI
XMPCreateFolderEnumerator
(
    __in LPXMP_MEDIASOURCEID pSourceId,                     // Media source to enumerate folders from.
    __in_opt LPXMP_FOLDERID pFolderId,                      // Folder to enumerate sub-folders from.
    __in DWORD cItems,                                      // Number of containers we want each time we call XEnumerate.
    __out DWORD* pcbBuffer,                                 // Size of the buffer required.
    __out HANDLE* ph                                        // Enumerator object.
);

//
// Creates an object used to enumerate the songs from the specified media source.
// Subsequent calls to XEnumerate() using the created enumerator will retrieve the songs via XMP_SONGDETAILS
// structures.
//

DWORD
WINAPI
XMPCreateUserSongEnumerator
(
    __in LPXMP_MEDIASOURCEID pSourceId,                     // Media source to enumerate songs from.
    __in_opt LPXMP_FOLDERID pFolderId,                      // Folder to enumerate songs from.
    __in BOOL fRecursive,                                   // Should the enumeration recurse into sub-folders
    __in DWORD cItems,                                      // Number of containers we want, each time we call XEnumerate.
    __out DWORD* pcbBuffer,                                 // Size of the buffer required
    __out HANDLE* ph                                        // Enumerator object.
);

//
// [Need documentation]
//

DWORD
WINAPI
XMPGetTotalNumberOfItems
(
    __in HANDLE hEnumerator,                                // Enumerator object.
    __out LPDWORD pdwTotalItems                             // Total number of items.
);

//
// Retrieves the core metadata for a song
//

DWORD
WINAPI
XMPGetSongDetailsById
(
    __in LPXMP_SONGID pMediaId,                             // The song we want metadata for.
    __out LPXMP_SONGDETAILS pSongInfo,                      // The populated song info struct.
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure for async requests.
);

//
// Retrieves the song artwork from the ID3 tag in a song if it is present
//

DWORD
WINAPI
XMPGetSongArtById
(
    __in LPXMP_SONGID pMediaId,                             // The song we want metadata for.
    __inout LPXMP_SONGART pSongArt,                         // The populated song art struct.
    __inout_opt PXOVERLAPPED pOverlapped                    // Overlapped structure for async requests.
);

//
// Determines if the title has playback control.
//

DWORD
WINAPI
XMPTitleHasPlaybackControl
(
    __out LPBOOL pfTitleHasControl                          // Is the title in control of playback.
);

//
// Disables the users music, allowing the game to control XMP and
// preventing the user from re-gaining control, the Mini Player will
// be disabled, until XMPRestorebackgroundMusic.
// Even if the user wasn't controlling XMP at the time of this call
// the Mini Player is still disabled to prevent them from taking control.
//

DWORD
WINAPI
XMPOverrideBackgroundMusic();

//
// Restores the previous controller prior to the call of
// XMPOverrideBackgroundMusic and enables the Mini Player.
//

DWORD
WINAPI
XMPRestoreBackgroundMusic();

//
// Retrieves output samples from the media player so they can be monitored
// or rendered by the title.  The callback supplied to XMPCaptureOutput
// will receive one audio frame of stereo samples.  The capture buffer will
// contain one 5.333ms frame of stereo floating-point audio samples at 48 kHz.
//
// The fTitleRendering parameter passed to XMPCaptureOutput sets
// whether the title wishes to render the music decoded by XMP.  If
// fTitleRendering is TRUE, the title must render the audio itself.  As long
// as XMP is playing a title playlist, it will not render any of the audio
// it's decoding.  If the user chooses to override the title music, XMP will
// automatically start rendering music again.  At that point, the capture
// callback will start receiving FALSE as its fTitleRendering parameter and
// the title should immediately stop rendering data, although monitoring it
// is still allowed.  If the title is rendering using an XAudio voice tagged as
// XAUDIOVOICECATEGORY_BGMUSIC, the voice will automatically be muted and
// unmuted when the user takes or relenquishes control of the music playback.
//

#define XMP_CAPTURE_CHANNEL_COUNT   2
#define XMP_CAPTURE_SAMPLE_COUNT    (XMP_CAPTURE_CHANNEL_COUNT * 256)

typedef FLOAT XMP_CAPTURE_BUFFER[XMP_CAPTURE_SAMPLE_COUNT], *LPXMP_CAPTURE_BUFFER;
typedef const XMP_CAPTURE_BUFFER *LPCXMP_CAPTURE_BUFFER;

typedef void (CALLBACK *LPFNXMP_CAPTURE_CALLBACK)
(
    __in LPCXMP_CAPTURE_BUFFER pSampleBuffer,
    __in LPVOID pContext,
    BOOL fTitleRendering
);

DWORD
WINAPI
XMPCaptureOutput
(
    __in LPFNXMP_CAPTURE_CALLBACK pfnCallback,
    __in LPVOID pContext,
    BOOL fTitleRendering,
    __inout_opt PXOVERLAPPED pOverlapped
);

//
// XMP decoder
//

// XMP decoder property types
enum XMP_DECODER_PROPERTY_TYPE
{
    XMP_DECODER_OUTPUT_WAVEFORMATEX = 0,                    // Read only
    XMP_DECODER_THREAD_PROCESSOR,
};

//
// Createa a decoder for a song.
//

DWORD
XMPCreateDecoder
(
    __in LPXMP_SONGDETAILS pSongDetails,
    __out LPXMP_HANDLE phDecoder
);

//
// Gets a property from the decoder.
//

DWORD
XMPGetDecoderProperty
(
    __in XMP_HANDLE hDecoder,
    __in XMP_DECODER_PROPERTY_TYPE eType,
    __out_bcount(cbValueBufferSize) BYTE *pbValue,
    __in DWORD cbValueBufferSize,
    __out_opt DWORD *pcbValueReturnedSize
);

//
// Sets a property to the decoder.
//
DWORD
XMPSetDecoderProperty
(
    __in XMP_HANDLE hDecoder,
    __in XMP_DECODER_PROPERTY_TYPE eType,
    __in_bcount(cbValueSize) BYTE *pbValue,
    __in DWORD cbValueSize
);

//
// Gets decoded data from the decoder.
//

DWORD
XMPGetDecodedData
(
    __in XMP_HANDLE hDecoder,
    __out_bcount(cbBufferSize) BYTE *pbBuffer,
    __in DWORD cbBufferSize,
    __out DWORD *pcbDecodedSize
);
 
//
// Destroys a decoder.
//

DWORD
XMPDestroyDecoder
(
    __in XMP_HANDLE hDecoder
);

#ifdef __cplusplus
}
#endif

#endif // __XMP_INCLUDED__
