
/***************************************************************************
 *
 *  XParty.h
 *
 *  This module defines the Xbox Party interface
 *
 *  Copyright (C) Microsoft Corporation.  All Rights Reserved.
 *
 ***************************************************************************/



#pragma once

#ifdef __cplusplus
extern "C" {
#endif



//-----------------------------------------------------------------------------
// Error definitions
//-----------------------------------------------------------------------------
#define FACILITY_XPARTY                   0x7D

#define XPARTY_S_OK                       S_OK                                      // 0x00000000
#define XPARTY_E_ABORT                    E_ABORT                                   // 0x80004004
#define XPARTY_E_ALREADY_IN_PARTY         _HRESULT_TYPEDEF_(0x807D0001L)            // 0x807D0001
#define XPARTY_E_INSUFFICIENT_PRIVILEGES  XONLINE_E_SESSION_INSUFFICIENT_PRIVILEGES // 0x80155201
#define XPARTY_E_INVALID_ARG              E_INVALIDARG                              // 0x80070057
#define XPARTY_E_INVALID_USER             _HRESULT_TYPEDEF_(0x807D0002L)            // 0x807D0002
#define XPARTY_E_NOT_IMPLEMENTED          E_NOTIMPL                                 // 0x80004001
#define XPARTY_E_NOT_IN_PARTY             _HRESULT_TYPEDEF_(0x807D0003L)            // 0x807D0003
#define XPARTY_E_NOT_SIGNED_IN_TO_LIVE    XONLINE_E_SESSION_NOT_LOGGED_ON           // 0x80155209
#define XPARTY_E_OUT_OF_MEMORY            E_OUTOFMEMORY                             // 0x8007000E
#define XPARTY_E_UNEXPECTED_FAILURE       E_FAIL                                    // 0x80004005


//-----------------------------------------------------------------------------
//  Party Users Definitions
//-----------------------------------------------------------------------------
#define XPARTY_MAX_USERS               32

#define XPARTY_USER_ISLOCAL                    0x00000001
#define XPARTY_USER_ISINPARTYVOICE             0x00000002
#define XPARTY_USER_ISTALKING                  0x00000004
#define XPARTY_USER_ISINGAMESESSION            0x00000008

typedef struct _XPARTY_CUSTOM_DATA {
    ULONGLONG                          qwFirst;
    ULONGLONG                          qwSecond;
} XPARTY_CUSTOM_DATA;

typedef struct _XPARTY_USER_INFO {
    XUID                               Xuid;
    CHAR                               GamerTag[XUSER_NAME_SIZE];
    DWORD                              dwUserIndex;
    XONLINE_NAT_TYPE                   NatType;
    DWORD                              dwTitleId;
    DWORD                              dwFlags;
    XSESSION_INFO                      SessionInfo;
    XPARTY_CUSTOM_DATA                 CustomData;
} XPARTY_USER_INFO;

typedef struct _XPARTY_USER_LIST {
    DWORD                              dwUserCount;
    XPARTY_USER_INFO                   Users[XPARTY_MAX_USERS];
} XPARTY_USER_LIST;

typedef enum _XPARTY_BANDWIDTH_TYPE {
    XPARTY_BANDWIDTH_UPLOAD_TOTAL_BYTES     = 0,
} XPARTY_BANDWIDTH_TYPE;


XBOXAPI
HRESULT
WINAPI
XPartyGetUserList(
    __out XPARTY_USER_LIST*            pUserList
    );

// Overlapped Extended Error: HRESULT
XBOXAPI
DWORD
WINAPI
XPartySendGameInvites(
    __in DWORD                         dwUserIndex,
    __inout_opt XOVERLAPPED*           pOverlapped
    );

XBOXAPI
HRESULT
WINAPI
XPartySetCustomData(
    __in DWORD                         dwUserIndex,
    __in const XPARTY_CUSTOM_DATA*     pCustomData
    );

XBOXAPI
HRESULT
WINAPI
XPartyGetBandwidth(
    __in XPARTY_BANDWIDTH_TYPE         Type,
    __out ULONGLONG*                   pqwBandwidth
    );


#ifdef __cplusplus
}
#endif
