/**************************************************************************
 *
 * Copyright (c) Microsoft Corporation. All rights reserved.
 *
 * Defines the error codes for XRN
 *
 **************************************************************************/

#ifndef __XRNERRORS_H__
#define __XRNERRORS_H__


#define FACILITY_XRN        0x7A
#define ERRBASE_XRN         0x1000

#ifndef MAKE_HRESULT
#define MAKE_HRESULT(sev,fac,code) \
    ((HRESULT) (((unsigned long)(sev) << 31) | ((unsigned long)(fac) << 16) | ((unsigned long)(code))) )
#endif
    
#define MAKE_XRNRESULTSUCCESS(code)     MAKE_HRESULT(0, FACILITY_XRN, (ERRBASE_XRN + (code)))
#define MAKE_XRNRESULTERROR(code)       MAKE_HRESULT(1, FACILITY_XRN, (ERRBASE_XRN + (code)))


//
// Success codes
//

// 0x00000000 - XRN_S_OK
// No error
#define XRN_S_OK                                S_OK

// 0x00000001 - XRN_S_NOEVENTS
// No events currently available
#define XRN_S_NOEVENTS                          S_FALSE


//
// Failure codes
//

// 0x8007000E - XRN_E_OUTOFMEMORY
// Operation failed due to lack of memory
#define XRN_E_OUTOFMEMORY                       E_OUTOFMEMORY

// 0x807A1001 - XRN_E_INSUFFICIENTBUFFER
// Insufficient buffer space was available to return the necessary data
// Repeat the call with a larger buffer space
#define XRN_E_INSUFFICIENTBUFFER                MAKE_XRNRESULTERROR(0x01)

// 0x807A1002 - XRN_E_NETNOTREADY
// Network interface is reporting its not ready
#define XRN_E_NETNOTREADY                       MAKE_XRNRESULTERROR(0x02)

// 0x807A1003 - XRN_E_NETDOWN
// Network interface has failed
#define XRN_E_NETDOWN                           MAKE_XRNRESULTERROR(0x03)

// 0x807A1004 - XRN_E_ADDRESSINUSE
// Either a bind attempt failed as the local address is already in use
// or a link can't be created because one already exists to that remote address
#define XRN_E_ADDRESSINUSE                      MAKE_XRNRESULTERROR(0x04)

// 0x807A1005 - XRN_E_LIMITREACHED
// An application or system defined limit was reached preventing the operation completing
#define XRN_E_LIMITREACHED                      MAKE_XRNRESULTERROR(0x05)

// 0x807A1006 - XRN_E_DATATOOLARGE
// Send data supplied to a create or deny link method was too large 
#define XRN_E_DATATOOLARGE                      MAKE_XRNRESULTERROR(0x06)

// 0x807A1007 - XRN_E_NORESPONSE
// There was no reponse from the remote side
#define XRN_E_NORESPONSE                        MAKE_XRNRESULTERROR(0x07)

// 0x807A1008 - XRN_E_DENIED
// Far side denied an incoming connection
#define XRN_E_DENIED                            MAKE_XRNRESULTERROR(0x08)

// 0x807A1009 - XRN_E_INVALIDOPERATION
// Attempted operation was invalid with current state or supplied parameters
#define XRN_E_INVALIDOPERATION                  MAKE_XRNRESULTERROR(0x09)

// 0x807A100A - XRN_E_NETGENERIC
// An undefined network error occurred
#define XRN_E_NETGENERIC                        MAKE_XRNRESULTERROR(0x0A)

// 0x807A100B - XRN_E_GENERIC
// An undefined error occurred
#define XRN_E_GENERIC                           MAKE_XRNRESULTERROR(0x0B)

// 0x807A100C - XRN_E_PKTDROPPED
// A packet was dropped on the wire
#define XRN_E_PKTDROPPED                        MAKE_XRNRESULTERROR(0x0C)

// 0x807A100D - XRN_E_BADPKT
// A packet with malformed data was received
#define XRN_E_BADPKT                            MAKE_XRNRESULTERROR(0x0D)

// 0x807A100E - XRN_E_DUPLICATEHANDLE
// A duplicated handle was supplied
#define XRN_E_DUPLICATEHANDLE                   MAKE_XRNRESULTERROR(0x0E)

// 0x807A100F - XRN_E_BADCHANNEL
// Bad channel identity was used
#define XRN_E_BADCHANNEL                        MAKE_XRNRESULTERROR(0x0F)

// 0x807A1010 - XRN_E_CHANNELTERMINATED
// Channel was terminated 
#define XRN_E_CHANNELTERMINATED                 MAKE_XRNRESULTERROR(0x10)

// 0x807A1011 - XRN_E_REMOTETERMINATE
// Link was remotely terminated
#define XRN_E_REMOTETERMINATE                   MAKE_XRNRESULTERROR(0x11)

// 0x807A1012 - XRN_E_LOCALTERMINATE
// Link was locally terminated
#define XRN_E_LOCALTERMINATE                    MAKE_XRNRESULTERROR(0x12)

// 0x807A1013 - XRN_E_INVALIDADDRESS
// Invalid address specified
#define XRN_E_INVALIDADDRESS                    MAKE_XRNRESULTERROR(0x13)

// 0x807A1014 - XRN_E_CONNRESET
// Remote side forcibly closed socket 
#define XRN_E_CONNRESET                         MAKE_XRNRESULTERROR(0x14)

// 0x807A1015 - XRN_E_USERCANCEL
// Operation was cancelled by the user
#define XRN_E_USERCANCEL                        MAKE_XRNRESULTERROR(0x15)

// 0x807A1016 - XRN_E_USERTIMEOUT
// A timeout specified by a user expired
#define XRN_E_USERTIMEOUT                       MAKE_XRNRESULTERROR(0x16)

// 0x807A1017 - XRN_E_REMOVED
// Session was removed
#define XRN_E_REMOVED                           MAKE_XRNRESULTERROR(0x17)

// 0x807A1018 - XRN_E_PARTIALCONNECTIVITY
// Couldn't connect or lost existing connections to one or more participants
#define XRN_E_PARTIALCONNECTIVITY               MAKE_XRNRESULTERROR(0x18)

// 0x807A1019 - XRN_E_DISJOINTCONNECTIVITY
// Disconnected because not all participants were able to communicate with all others
#define XRN_E_DISJOINTCONNECTIVITY              MAKE_XRNRESULTERROR(0x19)


#endif // end if not defined __XRNERRORS_H__
