
/************************************************************************
*                                                                       *
*   XSocialPost.h -- This module defines the API for posting to social  *
*                    networks                                           *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/
#pragma once

#include <sal.h>

#ifdef __cplusplus
extern "C" {
#endif

// SocialNetwork IDs.
#define XSOCIAL_NETWORKID_FACEBOOK   0x00000002

// Capability bits returned from XSocialGetCapabilities
#define XSOCIAL_CAPABILITY_POSTIMAGE   0x00000001 // title can post images to social networks
#define XSOCIAL_CAPABILITY_POSTLINK    0x00000002 // title can post links to social networks

// Flags for posting.
// It may be necessary to use more than one.
// Setting these correctly is important for proper parental control
// and content filtering to happen.
#define XSOCIAL_POST_USERGENERATEDCONTENT   0x00000001 // Image is user generated content
#define XSOCIAL_POST_KINECTCONTENT          0x00000002 // Image came from the Kinect sensor
#define XSOCIAL_POST_GAMECONTENT            0x00000004 // Post is the result of miscellaneous game activity
#define XSOCIAL_POST_ACHIEVEMENTCONTENT     0x00000008 // Post is the result of an earned achievement
#define XSOCIAL_POST_MEDIACONTENT           0x00000010 // Post is for media

//
// If the function returns ERROR_SUCCESS then *pCapabilities will have been set.
// If this returns ERROR_IO_PENDING, callers need to wait for the overlapped to
// complete before checking *pCapabilities.
//
XBOXAPI
DWORD
WINAPI
XSocialGetCapabilities(
    _Out_ DWORD*                        pCapabilities,
    _Inout_opt_ XOVERLAPPED*            pOverlapped
    );

//
// Get the token that the title can use to act on behalf of the user on the
// social network.  Will not put up UI.  Titles must be enabled on Xbox
// servers for this to work.
//
XBOXAPI
DWORD
WINAPI
XSocialGetUserToken(
    _In_    DWORD                       userIndex,
    _In_    DWORD                       dwSocialNetworkId,
    _In_z_  LPCWSTR                     desiredPermissions,
    _Out_cap_( *pcchKeyText ) LPWSTR     wszKeyText,
    _Inout_ DWORD*                      pcchKeyText,
    _Inout_opt_ XOVERLAPPED*            pOverlapped
    );

//
// Get the token that the title can use to act on behalf of the user on the
// social network.  Will bring up UI.  Only call these when XSocialGetUserToken
// didn't work.  Titles must be enabled on Xbox servers for this to work.
//
XBOXAPI
DWORD
WINAPI
XShowSocialGetUserToken(
    _In_    DWORD                       userIndex,
    _In_    DWORD                       dwSocialNetworkId,
    _In_z_  LPCWSTR                     desiredPermissions,
    _Out_cap_( *pcchKeyText ) LPWSTR     wszKeyText,
    _Inout_ DWORD*                      pcchKeyText,
    _Inout_opt_ XOVERLAPPED*            pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XShowNuiSocialGetUserToken(
    _In_    DWORD                       dwTrackingID,
    _In_    DWORD                       userIndex,
    _In_    DWORD                       dwSocialNetworkId,
    _In_z_  LPCWSTR                     desiredPermissions,
    _Out_cap_( *pcchKeyText ) LPWSTR     wszKeyText,
    _Inout_ DWORD*                      pcchKeyText,
    _Inout_opt_ XOVERLAPPED*            pOverlapped
    );

//
// Structure that holds information about the preview image
// for the posting UI.
//

typedef struct _XSOCIAL_PREVIEWIMAGE
{
    CONST BYTE* pBytes;
    DWORD       Pitch;
    DWORD       Width;
    DWORD       Height;
    D3DFORMAT   Format;
} XSOCIAL_PREVIEWIMAGE, *PXSOCIAL_PREVIEWIMAGE;


//
// Post an image to a social network
//
typedef struct _XSOCIAL_IMAGEPOSTPARAMS
{
    DWORD                       Size;
    LPCWSTR                     TitleText;
    LPCWSTR                     PictureCaption;
    LPCWSTR                     PictureDescription;
    XSOCIAL_PREVIEWIMAGE        PreviewImage;
    CONST BYTE*                 pFullImage;
    DWORD                       FullImageByteCount;
    DWORD                       Flags;
} XSOCIAL_IMAGEPOSTPARAMS, *PXSOCIAL_IMAGEPOSTPARAMS;

XBOXAPI
DWORD
WINAPI
XShowSocialNetworkImagePostUI(
    _In_        DWORD                           UserIndex,
    _In_        CONST XSOCIAL_IMAGEPOSTPARAMS*  pParams,
    _Inout_opt_ XOVERLAPPED*                    pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XShowNuiSocialNetworkImagePostUI(
    _In_        DWORD                           TrackingID,
    _In_        DWORD                           UserIndex,
    _In_        CONST XSOCIAL_IMAGEPOSTPARAMS*  pParams,
    _Inout_opt_ XOVERLAPPED*                    pOverlapped
    );


//
// Post a link to a social network
//

typedef struct _XSOCIAL_LINKPOSTPARAMS
{
    DWORD                       Size;
    LPCWSTR                     TitleText;
    LPCWSTR                     TitleURL;
    LPCWSTR                     PictureCaption;
    LPCWSTR                     PictureDescription;
    LPCWSTR                     PictureURL;
    XSOCIAL_PREVIEWIMAGE        PreviewImage;
    DWORD                       Flags;
} XSOCIAL_LINKPOSTPARAMS, *PXSOCIAL_LINKPOSTPARAMS;

XBOXAPI
DWORD
WINAPI
XShowSocialNetworkLinkPostUI(
    _In_    DWORD                           UserIndex,
    _In_    CONST XSOCIAL_LINKPOSTPARAMS*   pParams,
    _Inout_opt_ XOVERLAPPED*                pOverlapped
    );

XBOXAPI
DWORD
WINAPI
XShowNuiSocialNetworkLinkPostUI(
    _In_    DWORD                           TrackingID,
    _In_    DWORD                           UserIndex,
    _In_    CONST XSOCIAL_LINKPOSTPARAMS*   pParams,
    _Inout_opt_ XOVERLAPPED*                pOverlapped
    );

#ifdef __cplusplus
}
#endif
