/*++

Copyright (c) Microsoft Corporation.  All rights reserved.

Module Name:

	xtms.h

--*/
#pragma once

#if defined(__cplusplus)
extern "C" {
#endif

CONST INT MAX_RESPONSESIZE    =65536;
CONST INT ETAG_SIZE           =32;

#define TMS_E_CLIENT_SHUTDOWN                _HRESULT_TYPEDEF_(0x803F0001)
#define TMS_E_CLIENT_ALREADY_INITIALIZED     _HRESULT_TYPEDEF_(0x803F0002)
#define TMS_E_CLIENT_PENDING_REQUESTS        _HRESULT_TYPEDEF_(0x803F0003)

DECLARE_HANDLE(HTMSCLIENT);

// etag enum
enum ETAGFLAG
{
	IGNORE_ETAG,
	IF_MATCH,
	IF_NOT_MATCH
};

typedef struct
{
	DWORD returnCode;
	CHAR buffer[MAX_RESPONSESIZE];
	CHAR etag[ETAG_SIZE];  
}
HttpResponse, * PHttpResponse;

typedef
VOID
(CALLBACK * TMSCLIENT_CALLBACK)(
	__in HRESULT wrapperResult,
	__in HttpResponse *response,
	__in_opt void* userCallbackData
	);

XBOXAPI
HTMSCLIENT
WINAPI
XTmsCreateClient();

XBOXAPI
HRESULT
WINAPI
XTmsInitialize(
	__in      HTMSCLIENT          tmsClient,
	__in_z    LPCSTR              szGroupId,
	__in      DWORD               dwUserIndex
	);

XBOXAPI
HRESULT
WINAPI
XTmsShutdown(
	__in      HTMSCLIENT          tmsClient);

XBOXAPI
HRESULT
WINAPI
XTmsDoWork(
	__in      HTMSCLIENT          tmsClient);

XBOXAPI
HRESULT
WINAPI
XTmsGetUserQuotaInfo(
	__in      HTMSCLIENT          tmsClient,
	__in      XUID                xuid,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_opt  PVOID               userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsGetGlobalQuotaInfo(
	__in      HTMSCLIENT          tmsClient,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_opt  PVOID               userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsGetUserFileList(
	__in      HTMSCLIENT          tmsClient,
	__in      XUID                xuid,
	__in_z    LPCSTR              path,
	__in      INT                 skipItems,
	__in      INT                 maxItems,
	__in_z_opt LPCSTR             continuationToken,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_opt  PVOID               userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsGetGlobalFileList(
	__in      HTMSCLIENT          tmsClient,
	__in_z    LPCSTR              path,
	__in      INT                 skipItems,
	__in      INT                 maxItems,
	__in_z_opt LPCSTR             continuationToken,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_z_opt PVOID              userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsPutUserFile(
	__in      HTMSCLIENT          tmsClient,
	__in      XUID                xuid, 
	__in_z    LPCSTR              filePath, 
	__in_z    LPCSTR              fileType, 
	__in_z    LPCSTR              displayName,
	__in_z    LPCSTR              ugcContentType,
	__in_bcount(fileSize) CONST CHAR* fileBuffer,
	__in      DWORD               fileSize,
	__in_opt  DWORD               bufferSize,
	__in_z_opt LPCSTR             etag,
	__in      ETAGFLAG            etagFlag,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_opt  PVOID               userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsGetUserFile(
	__in      HTMSCLIENT          tmsClient,
	__in	  XUID                xuid,
	__in_z    LPCSTR              filePath,
	__in_z    LPCSTR              fileType,
	__out_bcount(fileSize) BYTE*  fileBuffer,
	__in      DWORD               fileSize,
	__in_z_opt LPCSTR             customSelector,
	__in_z_opt LPCSTR             jsonSelect,
	__in_z_opt LPCSTR             etag,
	__in      ETAGFLAG            etagFlag,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_opt  PVOID               userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsGetGlobalFile(
	__in      HTMSCLIENT          tmsClient,
	__in_z     LPCSTR              filePath,
	__in_z     LPCSTR              fileType,
	__out_bcount(fileSize) BYTE*  fileBuffer,
	__in      DWORD               fileSize,
	__in_z_opt LPCSTR             customSelector,
	__in_z_opt LPCSTR             jsonSelect,
	__in_z_opt LPCSTR             etag,
	__in      ETAGFLAG            etagFlag,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_opt  PVOID               userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsGetUserFileWithRange(
	__in      HTMSCLIENT          tmsClient,
	__in	  XUID                xuid,
	__in_z    LPCSTR              filePath,
	__in_z    LPCSTR              fileType,
	__out_bcount(fileSize) BYTE*  fileBuffer,
	__in      DWORD               fileSize,
	__in_z_opt LPCSTR             customSelector,
	__in_z_opt LPCSTR             jsonSelect,
	__in      DWORD               minRange,
	__in      DWORD               maxRange,
	__in_z_opt LPCSTR             etag,
	__in      ETAGFLAG            etagFlag,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_opt  PVOID               userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsGetGlobalFileWithRange(
	__in      HTMSCLIENT          tmsClient,
	__in_z     LPCSTR              filePath,
	__in_z     LPCSTR              fileType,
	__out_bcount(fileSize) BYTE*  fileBuffer,
	__in      DWORD               fileSize,
	__in_z_opt LPCSTR             customSelector,
	__in_z_opt LPCSTR             jsonSelect,
	__in      DWORD               minRange,
	__in      DWORD               maxRange,
	__in_z_opt LPCSTR             etag,
	__in      ETAGFLAG            etagFlag,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_opt  PVOID               userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsDeleteUserFile(
	__in      HTMSCLIENT          tmsClient,
	__in	  XUID                xuid,
	__in_z    LPCSTR              filePath,
	__in_z    LPCSTR              fileType,
	__in_z_opt LPCSTR             etag,
	__in      ETAGFLAG            etagFlag,
	__in      TMSCLIENT_CALLBACK  clientCallback,
	__in_opt  PVOID               userCallbackData
	);

XBOXAPI
HRESULT
WINAPI
XTmsRefreshAuthToken(
	__in      HTMSCLIENT          tmsClient);

#if defined(__cplusplus)
}
#endif
