/************************************************************************
*                                                                       *
*   XWMADecode.h -- This module defines the xWMA decoder API's          *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/
#pragma once

#ifndef _XWMADECODE_H_
#define _XWMADECODE_H_

#include <sal.h>

#ifdef __cplusplus
namespace XAUDIO2
{
#endif

enum XWMADECODE_STATE {
    XWMADECODE_STATE_DONE = 1,
    XWMADECODE_STATE_READY_TO_PROCESS
};

typedef struct XWMADECODE_INPUT_BUFFER_INFO
{
    const BYTE*                         pInputData;
    DWORD                               InputDataBytes;
    BOOL                                NoMoreInput;
} XWMADECODE_INPUT_BUFFER_INFO;

typedef HRESULT (*PFNGETMOREXWMADATA)(void* pContext, XWMADECODE_INPUT_BUFFER_INFO* pInfo);

typedef struct XWMADECODE
{
    BYTE*                               pDecodedData;
    DWORD                               FramesReady;
    DWORD                               XWMAPacketByteSize;
    DWORD                               SampleSize;
    WORD                                OutputChannels;
    XWMADECODE_STATE                    DecoderState;

    // The following fields are used internally.
    void*                               pDecoderContext;        // Internal.
    DWORD                               InternalDecodingStage;  // Internal.
    DWORD                               InternalFrameCount;     // Internal.
    PFNGETMOREXWMADATA                  pfnGetXWMAData;         // Internal.
    void*                               pUserContext;           // Internal.
    DWORD                               DecodedDataBytes;       // Internal.
} XWMADECODE;

XBOXAPI
DWORD
WINAPI
XWMADecodeGetRequiredBufferSize(
    __in    const WAVEFORMATEX*         pSourceFormat
    );

XBOXAPI
HRESULT
WINAPI
XWMADecodeCreate(
    __in    const WAVEFORMATEX*         pSourceFormat,
    __in    PFNGETMOREXWMADATA          pfnGetXWMAData,
    __in    void*                       pUserContext,
    __out_ecount(MemoryBlockSize) BYTE* pMemoryBlockToUse,
    __in    DWORD                       MemoryBlockSize,
    __deref_out XWMADECODE**            ppDecoder
    );

XBOXAPI
HRESULT
WINAPI
XWMADecodeProcessData(
    __inout XWMADECODE*                 pDecoder
    );

XBOXAPI
VOID
WINAPI
XWMADecodeDestroy(
    __in    XWMADECODE*                 pDecoder
    );

#ifdef __cplusplus
}; // namespace XAUDIO2
#endif

#endif // _XWMADECODE_H_
